@@ -3,7 +3,8 @@
 // Wrapper function
 function do_pie(name, update_url, url_params, units, refresh) {
   var pie = new PieChart(name, update_url, url_params, units, refresh);
-  pie.setInterval(setInterval(function(){pie.update();}, refresh));
+  if (refresh)
+    pie.setInterval(setInterval(function(){pie.update();}, refresh));
 
   // Return new class instance, with
   return pie;
@@ -319,7 +320,7 @@ PieChart.prototype.startInterval = function() {
 ///////////////////////////////////////////////////////////
 
 function create_pie_chart(name, units) {
-    var w = 600; //380 - Please keep in sync with pie-chart.css
+    var w = 500; //380 - Please keep in sync with pie-chart.css
     var h = 325; //280
     var ir = 52; //45
     var textOffset = 14;
@@ -348,9 +349,9 @@ function create_pie_chart(name, units) {
     ///////////////////////////////////////////////////////////
 
     var vis = d3.select(name).append("svg:svg")
-	// .attr("width", w)
-	// .attr("height", h)
-	.attr("viewBox","0 0 600 325") 
+	.attr("width", w)
+	.attr("height", h)
+	.attr("viewBox","0 0 500 325") 
 	.attr("preserveAspectRatio","xMidYMid");
 
     //GROUP FOR ARCS/PATHS
