@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,8 @@ class DB {
   virtual ~DB();
   
   virtual bool dumpFlow(time_t when, bool partial_dump, Flow *f, char *json);
-  virtual int exec_sql_query(lua_State *vm, char *sql);
+  virtual int exec_sql_query(lua_State *vm, char *sql, bool limit_rows);
+  virtual void startDBLoop();
 };
 
 #endif /* _DB_CLASS_H_ */
