@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can addresstribute it and/or modify
@@ -35,8 +35,8 @@ class HTTPBL {
   HTTPBL(char *_api_key);
   ~HTTPBL();
 
-  char* findHTTPBL(char *name, char *buf, u_int buf_len, bool add_if_needed); 
-  void startHTTPBLLoop();
+  char* findCategory(char *name, char *buf, u_int buf_len, bool add_if_needed); 
+  void startLoop();
   void* httpblLoop(void* ptr);
 };
 
