@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,17 +28,22 @@ class MySQLDB : public DB {
  private:
   MYSQL mysql;
   bool db_operational;
+  pthread_t queryThreadLoop;
+
+  bool connectToDB(MYSQL *conn, bool select_db);
+  char* get_last_db_error(MYSQL *conn) { return((char*)mysql_error(conn)); }
+  char* get_insert_into_values(IPVersion vers);
+  int insert_batch(MYSQL *conn, IPVersion vers);
+  int exec_sql_query(MYSQL *conn, char *sql, bool doReconnect = true, bool ignoreErrors = false, bool doLock = true);
 
-  bool connectToDB(bool select_db);
-  char* get_last_db_error() { return((char*)mysql_error(&mysql)); }
-  int exec_sql_query(char *sql, bool doReconnect = true, bool ignoreErrors = false);
-  
  public:
   MySQLDB(NetworkInterface *_iface = NULL);
   ~MySQLDB();
-  
+
+  void* queryLoop();
   bool dumpFlow(time_t when, bool partial_dump, Flow *f, char *json);
-  int exec_sql_query(lua_State *vm, char *sql);
+  int exec_sql_query(lua_State *vm, char *sql, bool limitRows);
+  void startDBLoop();
 };
 
 #endif /* _MYSQL_DB_CLASS_H_ */
