@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -34,7 +34,7 @@ class PeriodicActivities {
   pthread_t secondLoop, minuteLoop, hourLoop, dayLoop;
 
   u_int32_t roundTime(u_int32_t now, u_int32_t rounder);
-  void runScript(char *path);
+  void runScript(char *path, u_int32_t when);
 
  public:
   PeriodicActivities();
