@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -26,7 +26,7 @@
 
 class ProtoStats {
  private:
-  u_int64_t numPkts, numBytes;
+  u_int64_t volatile numPkts, numBytes;
 
  public:
   ProtoStats();
@@ -35,6 +35,8 @@ class ProtoStats {
   inline void inc(u_int32_t pkts, u_int32_t bytes) { numPkts += pkts, numBytes += bytes; };
   inline u_int64_t getPkts()                       { return(numPkts);  };
   inline u_int64_t getBytes()                      { return(numBytes); };
+  inline void setPkts(u_int64_t v)                 { numPkts = v;  };
+  inline void setBytes(u_int64_t v)                { numBytes = v; };
   void lua(lua_State *vm, const char *prefix);
   void print(const char *prefix);
 };
