@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,6 +27,7 @@
 class Host;
 
 class Redis {
+ private:
   redisContext *redis;
   Mutex *l;
   char *redis_host;
@@ -38,6 +39,11 @@ class Redis {
   void setDefaults();
   void reconnectRedis();
   int msg_push(const char *cmd, const char *queue_name, char *msg, u_int queue_trim_size);
+  int oneOperator(const char *operation, char *key);
+  int twoOperators(const char *operation, char *op1, char *op2);
+  int pushHost(const char* ns_cache, const char* ns_list, char *hostname,
+	       bool dont_check_for_existance, bool localHost);
+  int popHost(const char* ns_list, char *hostname, u_int hostname_len);
 
  public:
   Redis(char *redis_host = (char*)"127.0.0.1", u_int16_t redis_port = 6379, u_int8_t _redis_db_id = 0);
@@ -51,29 +57,29 @@ class Redis {
   int hashGet(char *key, char *member, char *rsp, u_int rsp_len);
   int hashDel(char *key, char *field);
   int hashSet(char *key, char *field, char *value);
-  int delHash(char *key, char *member); 
+  int delHash(char *key, char *member);
   int set(char *key, char *value, u_int expire_secs=0);
   char* popSet(char *pop_name, char *rsp, u_int rsp_len);
   int keys(const char *pattern, char ***keys_p);
   int hashKeys(const char *pattern, char ***keys_p);
-  int del(char *key); 
-  int zincrbyAndTrim(char *key, char *member, u_int value, u_int trim_len);
-
+  int del(char *key);
+  int zIncr(char *key, char *member);
+  int zTrim(char *key, u_int trim_len);
+  int zRevRange(const char *pattern, char ***keys_p);
   int pushHostToResolve(char *hostname, bool dont_check_for_existance, bool localHost);
   int popHostToResolve(char *hostname, u_int hostname_len);
 
-  int pushHostToHTTPBL(char *hostname, bool dont_check_for_existance, bool localHost);
-  int popHostToHTTPBL(char *hostname, u_int hostname_len);
+  int pushHostToTrafficFiltering(char *hostname, bool dont_check_for_existance, bool localHost);
+  int popHostToTrafficFiltering(char *hostname, u_int hostname_len);
 
-  char* getHTTPBLCategory(char *numeric_ip, char *buf, u_int buf_len, bool query_httpbl_if_unknown);
-  char* getFlowCategory(char *domainname, char *buf, u_int buf_len, bool categorize_if_unknown);
+  char* getTrafficFilteringCategory(char *numeric_ip, char *buf, u_int buf_len, bool query_httpbl_if_unknown);
   int popDomainToCategorize(char *domainname, u_int domainname_len);
-  
+
   int getAddress(char *numeric_ip, char *rsp, u_int rsp_len, bool queue_if_not_found);
-  int getAddressHTTPBL(char *numeric_ip, NetworkInterface *iface,
+  int getAddressTrafficFiltering(char *numeric_ip, NetworkInterface *iface,
 		       char *rsp, u_int rsp_len, bool queue_if_not_found);
   int setResolvedAddress(char *numeric_ip, char *symbolic_ip);
-  int setHTTPBLAddress(char* numeric_ip, char* httpbl);
+  int setTrafficFilteringAddress(char* numeric_ip, char* httpbl);
 
   int hashIncr(char *key, char *field, u_int32_t value);
 
@@ -84,13 +90,12 @@ class Redis {
   void setHostId(NetworkInterface *iface, char *daybuf, char *host_name, u_int32_t id);
   u_int32_t host_to_id(NetworkInterface *iface, char *daybuf, char *host_name, bool *new_key);
   int id_to_host(char *daybuf, char *host_idx, char *buf, u_int buf_len);
-  void pushEStemplate();
-  void indexESdata();
   int lpush(const char *queue_name, char *msg, u_int queue_trim_size);
   int rpush(const char *queue_name, char *msg, u_int queue_trim_size);
+  int lindex(const char *queue_name, int idx, char *buf, u_int buf_len);
   u_int llen(const char *queue_name);
   int lpop(const char *queue_name, char *buf, u_int buf_len);
-  void startFlowDump();
+  int lpop(const char *queue_name, char ***elements, u_int num_elements);
 
   /**
    * @brief Increment a redis key and return its new value
@@ -103,10 +108,11 @@ class Redis {
    * @brief Queue an alert in redis
    *
    * @param level The alert level
+   * @param s     The alert status (alert on/off)
    * @param t     The alert type
    * @param msg   The alert message
    */
-  void queueAlert(AlertLevel level, AlertType t, char *msg);
+  void queueAlert(AlertLevel level, AlertStatus s, AlertType t, char *msg);
 
   /**
    * @brief Returns the number of queued alerts in redis generated by ntopng
@@ -137,12 +143,9 @@ class Redis {
    * @brief Flush all queued alerts
    *
    */
-  inline void flushAllQueuedAlerts() { del((char*)CONST_ALERT_MSG_QUEUE); };
-
- private:
-  int pushHost(const char* ns_cache, const char* ns_list, char *hostname,
-	       bool dont_check_for_existance, bool localHost);
-  int popHost(const char* ns_list, char *hostname, u_int hostname_len);
+  inline void flushAllQueuedAlerts() { delKey((char*)CONST_ALERT_MSG_QUEUE);       };
+  int delKey(char *key)              { return(oneOperator("DEL", key));            };
+  int rename(char *oldk, char *newk) { return(twoOperators("RENAME", oldk, newk)); };
 };
 
 #endif /* _REDIS_H_ */
