@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -122,36 +122,40 @@
 #define PURGE_FRACTION           32 /* check 1/32 of hashes per iteration */
 #define MAX_NUM_QUEUED_ADDRS    500 /* Maximum number of queued address for resolution */
 #define MAX_NUM_QUEUED_CONTACTS 25000
-#define NTOP_COPYRIGHT          "(C) 1998-15 ntop.org"
-#define DEFAULT_PID_PATH        "/var/tmp/ntopng.pid"
+#define NTOP_COPYRIGHT          "(C) 1998-2016 ntop.org"
+#define DEFAULT_PID_PATH        "/var/run/ntopng.pid"
 #define DOMAIN_CATEGORY         "ntopng.domain.category"
 #define DOMAIN_TO_CATEGORIZE    "ntopng.domain.tocategorize"
 #define DOMAIN_WHITELIST_CAT    "ntopng.domain.whitelist"
 #define DNS_CACHE               "ntopng.dns.cache"
 #define DNS_TO_RESOLVE          "ntopng.dns.toresolve"
-#define HTTPBL_CACHE            "ntopng.httpbl.cache"
-#define HTTPBL_TO_RESOLVE       "ntopng.httpbl.toresolve"
+#define NTOPNG_TRACE            "ntopng.trace"
+#define MAX_NUM_NTOPNG_TRACES   32
+#define TRAFFIC_FILTERING_CACHE            "ntopng.trafficfiltering.cache"
+#define TRAFFIC_FILTERING_TO_RESOLVE       "ntopng.trafficfiltering.toresolve"
 #define DROP_HOST_TRAFFIC       "ntopng.prefs.drop_host_traffic"
 #define DUMP_HOST_TRAFFIC       "ntopng.prefs.dump_host_traffic"
 #define HOST_TRAFFIC_QUOTA      "ntopng.prefs.hosts_quota"
-#define HTTPBL_CACHE_DURATIION  43200 /* 12 h */
+#define TRAFFIC_FILTERING_CACHE_DURATIION  43200 /* 12 h */
 #define DNS_CACHE_DURATION      3600  /*  1 h */
 #define HOST_LABEL_NAMES        "ntopng.host_labels"
 #define NTOP_HOSTS_SERIAL       "ntopng.host_serial"
-#define DUMMY_IFACE_ID          (-1)
+#define DUMMY_IFACE_ID          (MAX_NUM_INTERFACES-2)
+#define STDIN_IFACE_ID          (MAX_NUM_INTERFACES-3)
 #define MAX_CSRF_DURATION       300 /* 5 mins */
 #define NTOP_NOLOGIN_USER	"nologin"
 
-#define CONST_ADMINISTRATOR_USER  "administrator"
-#define CONST_UNPRIVILEGED_USER   "unprivileged"
-#define CONST_STR_NTOPNG_LICENSE  "ntopng.license"
-#define CONST_STR_USER_GROUP      "ntopng.user.%s.group"
-#define CONST_STR_USER_FULL_NAME  "ntopng.user.%s.full_name"
-#define CONST_STR_USER_PASSWORD   "ntopng.user.%s.password"
-#define CONST_STR_USER_NETS       "ntopng.user.%s.allowed_nets"
-#define CONST_ALLOWED_NETS        "allowed_nets"
-#define CONST_USER                "user"
-#define CONST_ES_QUEUE_NAME       "ntopng.es"
+#define CONST_ADMINISTRATOR_USER      "administrator"
+#define CONST_UNPRIVILEGED_USER       "unprivileged"
+#define CONST_STR_NTOPNG_LICENSE      "ntopng.license"
+#define CONST_STR_USER_GROUP          "ntopng.user.%s.group"
+#define CONST_STR_USER_FULL_NAME      "ntopng.user.%s.full_name"
+#define CONST_STR_USER_PASSWORD       "ntopng.user.%s.password"
+#define CONST_STR_USER_NETS           "ntopng.user.%s.allowed_nets"
+#define CONST_STR_USER_ALLOWED_IFNAME "ntopng.user.%s.allowed_ifname"
+#define CONST_ALLOWED_NETS            "allowed_nets"
+#define CONST_ALLOWED_IFNAME          "allowed_ifname"
+#define CONST_USER                     "user"
 
 #define CONST_INTERFACE_TYPE_PCAP      "pcap"
 #define CONST_INTERFACE_TYPE_PCAP_DUMP "pcap dump"
@@ -159,6 +163,9 @@
 #define CONST_INTERFACE_TYPE_SQLITE    "sqlite"
 #define CONST_INTERFACE_TYPE_PF_RING   "PF_RING"
 #define CONST_INTERFACE_TYPE_NETFILTER "netfilter"
+#define CONST_INTERFACE_TYPE_DIVERT    "divert"
+#define CONST_INTERFACE_TYPE_DUMMY     "dummy"
+#define CONST_INTERFACE_TYPE_ZC_FLOW   "ZC-flow"
 #define CONST_INTERFACE_TYPE_UNKNOWN   "unknown"
 
 #define CONST_DEMO_MODE_DURATION       600 /* 10 min */
@@ -169,6 +176,9 @@
 
 #define CONST_EST_MAX_FLOWS            200000
 #define CONST_EST_MAX_HOSTS            200000
+#define MIN_HOST_RESOLUTION_FREQUENCY  60  /* 1 min */
+#define HOST_SITES_REFRESH             300 /* 5 min */
+#define HOST_MAX_SERIALIZED_LEN        16348 /* bytes */
 
 #define BATADV_COMPAT_VERSION_15 15
 #define BATADV_COMPAT_VERSION_14 14
@@ -180,8 +190,8 @@
 #define BATADV14_BCAST		 0x04
 #define BATADV14_VIS		 0x05
 #define BATADV14_UNICAST_FRAG	 0x06
-#define BATADV14_TT_QUERY	 0x07 
-#define BATADV14_ROAM_ADV	 0x08 
+#define BATADV14_TT_QUERY	 0x07
+#define BATADV14_ROAM_ADV	 0x08
 #define BATADV14_UNICAST_4ADDR	 0x09
 #define BATADV14_CODED		 0x0a
 
@@ -241,6 +251,7 @@
 #define CONST_NUM_CONTACT_DBS          8
 #define CONST_MAX_NUM_ZMQ_SUBSCRIBERS 32
 #define CONST_MAX_NUM_FIND_HITS       10
+#define CONST_MAX_NUM_HITS         32768 /* Decrease it for small installations */
 #ifdef WIN32
 #define ntop_mkdir(a, b) _mkdir(a)
 #define CONST_PATH_SEP                    '\\'
@@ -255,6 +266,7 @@
 #define NTOPNG_NDPI_OS_PROTO_ID      (NDPI_LAST_IMPLEMENTED_PROTOCOL+NDPI_MAX_NUM_CUSTOM_PROTOCOLS-2)
 #define CONST_DEFAULT_HOME_NET       "192.168.1.0/24"
 #define CONST_DEFAULT_DATA_DIR       "/var/tmp/ntopng"
+#define CONST_DEFAULT_IS_AUTOLOGOUT_ENABLED 1
 #define CONST_DEFAULT_DOCS_DIR       "httpdocs"
 #define CONST_DEFAULT_SCRIPTS_DIR    "scripts"
 #define CONST_DEFAULT_CALLBACKS_DIR  "scripts/callbacks"
@@ -262,6 +274,7 @@
 #define CONST_DEFAULT_WRITABLE_DIR   "/var/tmp"
 #define CONST_DEFAULT_INSTALL_DIR    (DATA_DIR "/ntopng")
 #define CONST_ALT_INSTALL_DIR        "/usr/share/ntopng"
+#define CONST_ALT2_INSTALL_DIR       "/usr/local/share/ntopng"
 #define CONST_HTTP_PREFIX_STRING     "@HTTP_PREFIX@"
 #define CONST_DEFAULT_NTOP_USER      "nobody"
 #define CONST_TOO_EARLY              "(Too Early)"
@@ -284,13 +297,21 @@
 #define CONST_AGGREGATIONS            "aggregations"
 #define CONST_HOST_CONTACTS           "host_contacts"
 
-#define CONST_ALERT_MSG_QUEUE         "ntopng.alert_queue"
-#define CONST_ALERT_PREFS             "ntopng.prefs.alerts"
+#define CONST_ALERT_MSG_QUEUE              "ntopng.alert_queue"
+#define CONST_SQL_QUEUE                    "ntopng.sql_queue"
+#define CONST_SQL_BATCH_SIZE               32
+#define CONST_MAX_SQL_QUERY_LEN            8192
+#define CONST_ALERT_PREFS                  "ntopng.prefs.alerts"
 #ifdef NTOPNG_PRO
-#define CONST_NAGIOS_HOST_PREFS       "ntopng.prefs.nagios_host"
-#define CONST_NAGIOS_PORT_PREFS       "ntopng.prefs.nagios_port"
-#define CONST_NAGIOS_CONF_PREFS       "ntopng.prefs.nagios_config"
+#define CONST_NAGIOS_NSCA_HOST_PREFS       "ntopng.prefs.nagios_nsca_host"
+#define CONST_NAGIOS_NSCA_PORT_PREFS       "ntopng.prefs.nagios_nsca_port"
+#define CONST_NAGIOS_SEND_NSCA_EXEC_PREFS  "ntopng.prefs.nagios_send_nsca_executable"
+#define CONST_NAGIOS_SEND_NSCA_CONF_PREFS  "ntopng.prefs.nagios_send_nsca_config"
+#define CONST_NAGIOS_HOST_NAME_PREFS       "ntopng.prefs.nagios_host_name"
+#define CONST_NAGIOS_SERVICE_NAME_PREFS    "ntopng.prefs.nagios_service_name"
 #endif
+#define CONST_NBOX_USER               "ntopng.prefs.nbox_user"
+#define CONST_NBOX_PASSWORD           "ntopng.prefs.nbox_password"
 #define CONST_IFACE_ID_PREFS          "ntopng.prefs.iface_id"
 #define CONST_LOCAL_HOST_IDLE_PREFS   "ntopng.prefs.local_host_max_idle"
 #define CONST_REMOTE_HOST_IDLE_PREFS  "ntopng.prefs.non_local_host_max_idle"
@@ -308,17 +329,24 @@
 #define CONST_OTHER_RRD_1D_DAYS       "ntopng.prefs.other_rrd_1d_days"
 #define CONST_PROFILES_PREFS          "ntopng.prefs.profiles"
 
+
+#define CONST_RUNTIME_IS_AUTOLOGOUT_ENABLED          "ntopng.prefs.is_autologon_enabled"
+#define CONST_RUNTIME_PREFS_HOUSEKEEPING_FREQUENCY   "ntopng.prefs.housekeeping_frequency"
 #define CONST_RUNTIME_PREFS_HOST_RRD_CREATION        "ntopng.prefs.host_rrd_creation" /* 0 / 1 */
 #define CONST_RUNTIME_PREFS_HOST_NDPI_RRD_CREATION   "ntopng.prefs.host_ndpi_rrd_creation" /* 0 / 1 */
-#define CONST_RUNTIME_PREFS_THPT_CONTENT             "ntopng.prefs.thpt_content"  /* bps / pps */
-#define CONST_RUNTIME_PREFS_ALERT_SYSLOG             "ntopng.prefs.alerts_syslog"  /* 0 / 1 */
+#define CONST_RUNTIME_PREFS_HOST_CATE_RRD_CREATION   "ntopng.prefs.host_categories_rrd_creation" /* 0 / 1 */
+#define CONST_RUNTIME_PREFS_THPT_CONTENT             "ntopng.prefs.thpt_content"     /* bps / pps */
+#define CONST_RUNTIME_PREFS_ALERT_SYSLOG             "ntopng.prefs.alerts_syslog"    /* 0 / 1 */
+#define CONST_RUNTIME_PREFS_ALERT_PROBING            "ntopng.prefs.probing_alerts"   /* 0 / 1 */
+#define CONST_RUNTIME_PREFS_NBOX_INTEGRATION         "ntopng.prefs.nbox_integration" /* 0 / 1 */
 #ifdef NTOPNG_PRO
-#define CONST_RUNTIME_PREFS_ALERT_NAGIOS             "ntopng.prefs.alerts_nagios"  /* 0 / 1 */
-#define CONST_RUNTIME_PREFS_DAILY_REPORTS            "ntopng.prefs.daily_reports"  /* 0 / 1 */
+#define CONST_RUNTIME_PREFS_ALERT_NAGIOS             "ntopng.prefs.alerts_nagios"    /* 0 / 1 */
+#define CONST_RUNTIME_PREFS_DAILY_REPORTS            "ntopng.prefs.daily_reports"    /* 0 / 1 */
 #endif
 
 #define CONST_MAX_ALERT_MSG_QUEUE_LEN 8192
 #define CONST_MAX_ES_MSG_QUEUE_LEN    8192
+#define CONST_MAX_MYSQL_QUEUE_LEN     8192
 #define CONST_MAX_NUM_READ_ALERTS     32
 #define CONST_MAX_THRESHOLD_CROSS_DURATION 3
 #define CONST_MAX_ACTIVITY_DURATION    86400 /* sec */
@@ -337,7 +365,14 @@
 
 #define CONST_ANY_ADDRESS              "" /* Good for v4 and v6 */
 #define CONST_LOOPBACK_ADDRESS         "127.0.0.1"
-
+#define CONST_MAX_IDLE_INTERARRIVAL_TIME  60000 /* 1 min (msec) */
+#define CONST_MAX_IDLE_INTERARRIVAL_TIME_NO_TWH         3000  /* 3 sec (msec) */
+#define CONST_MAX_IDLE_INTERARRIVAL_TIME_NO_TWH_SYN_ACK 6  /* 6 sec */
+#define CONST_MAX_IDLE_NO_DATA_AFTER_ACK  10 /* 10 sec */
+#define CONST_SSL_MAX_DELTA            CONST_MAX_IDLE_INTERARRIVAL_TIME_NO_TWH
+#define CONST_MAX_IDLE_PKT_TIME        CONST_MAX_IDLE_INTERARRIVAL_TIME_NO_TWH
+#define CONST_MAX_IDLE_FLOW_TIME       10*CONST_MAX_IDLE_INTERARRIVAL_TIME
+#define CONST_MAX_SSL_IDLE_TIME        46000 /* 46 sec */
 #define CONST_EPP_MAX_CMD_NUM          34
 #define CONST_DEFAULT_MTU              1514
 
@@ -345,18 +380,17 @@
 #define NULL_BL                        "''"
 //#define DEBUG_HTTPBL
 
-#define CATEGORIZATION_URL             "https://sb-ssl.google.com/safebrowsing/api/lookup"
-#define CATEGORIZATION_CLIENT          "ntopng"
-#define CATEGORIZATION_APPVER          "1.5.2"
-#define CATEGORIZATION_PVER            "3.1"
-#define CATEGORIZATION_NULL_CATEGORY   "''"
-#define CATEGORIZATION_SAFE_SITE       "safe"
-
 #ifdef NTOPNG_PRO
-#define CONST_DEFAULT_NAGIOS_HOST      "localhost"
-#define CONST_DEFAULT_NAGIOS_PORT      "5667"
-#define CONST_DEFAULT_NAGIOS_CONF      "/etc/nagios/send_nsca.cfg"
+#define CONST_DEFAULT_NAGIOS_NSCA_HOST      "localhost"
+#define CONST_DEFAULT_NAGIOS_NSCA_PORT      "5667"
+#define CONST_DEFAULT_NAGIOS_SEND_NSCA_EXEC "/usr/local/nagios/bin/send_nsca"
+#define CONST_DEFAULT_NAGIOS_SEND_NSCA_CONF "/usr/local/nagios/etc/send_nsca.cfg"
+#define CONST_DEFAULT_NAGIOS_HOST_NAME      "ntopng-host"
+#define CONST_DEFAULT_NAGIOS_SERVICE_NAME   "NtopngAlert"
 #endif
+#define CONST_DEFAULT_NBOX_HOST      "localhost"
+#define CONST_DEFAULT_NBOX_USER      "nbox"
+#define CONST_DEFAULT_NBOX_PASSWORD  "nbox"
 
 #ifdef __cplusplus
 #define EXTERNC extern "C"
@@ -365,7 +399,6 @@
 #endif
 
 #ifdef WIN32
-
 #ifndef _CRT_SECURE_NO_WARNINGS
 #define _CRT_SECURE_NO_WARNINGS
 #endif
@@ -410,7 +443,7 @@
 #define unlink(a) _unlink(a)
 #endif
 
-#if defined(__arm__) || defined(__mips__) 
+#if defined(__arm__) || defined(__mips__)
 #define NTOPNG_EMBEDDED_EDITION         1
 #endif
 
@@ -426,5 +459,53 @@
 #define closesocket(c)		close(c)
 #endif
 
-#define NTOP_ES_TEMPLATE        "ntopng_template_elk.json"
+#ifndef DLT_IPV4
+#define DLT_IPV4  228
+#endif
+
+#define HTTPBL_STRING              "httpbl:"
+#define FLASHSTART_STRING          "flashstart:"
+#define NUM_FLASHSTART_SERVERS     2
+
+#define MAX_NUM_CATEGORIES         3
+#define NTOP_UNKNOWN_CATEGORY_STR  "???"
+#define NTOP_UNKNOWN_CATEGORY_ID   0
+// MySQL-related defined
+#define MYSQL_MAX_NUM_FIELDS  255
+#define MYSQL_MAX_NUM_ROWS    999
+
+/* GRE (Generic Route Encapsulation) */
+#ifndef IPPROTO_GRE
+#define IPPROTO_GRE 47
+#endif
+
+#define GRE_HEADER_CHECKSUM      0x8000 /* 32 bit */
+#define GRE_HEADER_ROUTING       0x4000 /* 32 bit */
+#define GRE_HEADER_KEY           0x2000 /* 32 bit */
+#define GRE_HEADER_SEQ_NUM       0x1000 /* 32 bit */
+
+#define HOST_LOW_GOODPUT_THRESHOLD  25 /* No more than X low goodput flows per host */
+#define FLOW_GOODPUT_THRESHOLD      40 /* 40% */
+
+#define PREF_NTOP_AUTHENTICATION_TYPE "ntopng.prefs.auth_type"
+#define PREF_LDAP_ACCOUNT_TYPE        "ntopng.prefs.ldap.account_type"
+#define PREF_LDAP_SERVER              "ntopng.prefs.ldap.server"
+#define PREF_LDAP_BIND_ANONYMOUS      "ntopng.prefs.ldap.anonymous_bind"
+#define PREF_LDAP_BIND_DN             "ntopng.prefs.ldap.bind_dn"
+#define PREF_LDAP_BIND_PWD            "ntopng.prefs.ldap.bind_pwd"
+#define PREF_LDAP_SEARCH_PATH         "ntopng.prefs.ldap.search_path"
+#define PREF_LDAP_USER_GROUP          "ntopng.prefs.ldap.user_group"
+#define PREF_LDAP_ADMIN_GROUP         "ntopng.prefs.ldap.admin_group"
+#define PREF_LDAP_GROUP_OF_USER       "ntopng.prefs.ldap.%s.group_of_user"
+#define PREF_USER_TYPE_LOG            "ntopng.prefs.user.%s.type_log"
+
+/* Elastic Search */
+#define NTOP_ES_TEMPLATE              "ntopng_template_elk.json"
+#define ES_MAX_QUEUE_LEN              32768
+
+/* Unkown values for host groups */
+#define UNKNOWN_COUNTRY       ""
+#define UNKNOWN_OS            ""
+#define UNKNOWN_ASN           "Private ASN"
+#define UNKNOWN_LOCAL_NETWORK "Remote Networks"
 #endif /* _NTOP_DEFINES_H_ */
