@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -28,6 +28,7 @@
 #ifndef _NTOP_FLOW_H_
 #define _NTOP_FLOW_H_
 
+#define NTOP_PEN     35632
 #define NTOP_BASE_ID 57472
 
 /* 1024 custom ntop elements for v9 should be enough */
@@ -89,6 +90,9 @@
 #define IN_SRC_MAC   56
 #define SRC_VLAN   58
 #define DST_VLAN   59
+#define DOT1Q_SRC_VLAN   243
+#define DOT1Q_DST_VLAN   254
+
 #define IP_PROTOCOL_VERSION   60
 #define DIRECTION   61
 #define IPV6_NEXT_HOP   62
@@ -136,16 +140,10 @@
 #define PACKETS_SELECTED   319
 #define SELECTOR_NAME   335
 #define FRAGMENTS   NTOP_BASE_ID+80
+#define SRC_AS_MAP  NTOP_BASE_ID+443
+#define DST_AS_MAP  NTOP_BASE_ID+444
 
-#ifdef DEPRECATED_INFORMATION_ELEMENTS
-#define CLIENT_NW_DELAY_SEC   NTOP_BASE_ID+82
-#define CLIENT_NW_DELAY_USEC   NTOP_BASE_ID+83
-#define SERVER_NW_DELAY_SEC   NTOP_BASE_ID+84
-#define SERVER_NW_DELAY_USEC   NTOP_BASE_ID+85
-#endif
-
-#define APPL_LATENCY_SEC   NTOP_BASE_ID+86
-#define APPL_LATENCY_USEC   NTOP_BASE_ID+87
+/* NTOP_BASE_ID+82 ->87  Available (deprecated) */
 #define NUM_PKTS_UP_TO_128_BYTES   NTOP_BASE_ID+88
 #define NUM_PKTS_128_TO_256_BYTES   NTOP_BASE_ID+89
 #define NUM_PKTS_256_TO_512_BYTES   NTOP_BASE_ID+90
@@ -181,6 +179,10 @@
 #define PLUGIN_NAME           NTOP_BASE_ID+126
 #define RETRANSMITTED_IN_BYTES  NTOP_BASE_ID+127
 #define RETRANSMITTED_OUT_BYTES NTOP_BASE_ID+128
+#define SRC_IP_LONG           NTOP_BASE_ID+448
+#define SRC_IP_LAT            NTOP_BASE_ID+449
+#define DST_IP_LONG           NTOP_BASE_ID+450
+#define DST_IP_LAT            NTOP_BASE_ID+451
 #define UNTUNNELED_IPV6_SRC_ADDR NTOP_BASE_ID+396
 #define UNTUNNELED_IPV6_DST_ADDR NTOP_BASE_ID+397
 #define NUM_PKTS_TTL_EQ_1     NTOP_BASE_ID+347
@@ -205,7 +207,11 @@
 #define TCP_WIN_MAX_OUT       NTOP_BASE_ID+420
 #define TCP_WIN_MSS_OUT       NTOP_BASE_ID+421
 #define TCP_WIN_SCALE_OUT     NTOP_BASE_ID+422
-
+#define PAYLOAD_HASH          NTOP_BASE_ID+438
+#define UPSTREAM_SESSION_ID   NTOP_BASE_ID+446
+#define DOWNSTREAM_SESSION_ID NTOP_BASE_ID+447
+#define SSL_SERVER_NAME       NTOP_BASE_ID+188
+#define BITTORRENT_HASH       NTOP_BASE_ID+189
 
 /* SIP */
 #define SIP_CALL_ID NTOP_BASE_ID+130
@@ -299,7 +305,6 @@
 #define HTTP_UA NTOP_BASE_ID+183
 #define HTTP_MIME NTOP_BASE_ID+184
 #define HTTP_HOST NTOP_BASE_ID+187
-#define HTTP_FBOOK_CHAT NTOP_BASE_ID+188
 #define HTTP_SITE      NTOP_BASE_ID+361
 
 /* SMTP */
@@ -355,6 +360,7 @@
 #define GTPV1_ULI_CELL_CI NTOP_BASE_ID+234
 #define GTPV1_ULI_SAC NTOP_BASE_ID+235
 #define GTPV1_RESPONSE_CAUSE  NTOP_BASE_ID+332
+#define GTPV1_RAT_TYPE NTOP_BASE_ID+236
 
 /* Radius */
 #define RADIUS_REQ_MSG_TYPE NTOP_BASE_ID+240
@@ -391,8 +397,15 @@
 #define GTPV2_ULI_MNC NTOP_BASE_ID+280
 #define GTPV2_ULI_CELL_TAC NTOP_BASE_ID+281
 #define GTPV2_ULI_CELL_ID NTOP_BASE_ID+282
+#define GTPV2_RAT_TYPE NTOP_BASE_ID+283
+#define GTPV2_PDN_IP NTOP_BASE_ID+284
+#define GTPV2_END_USER_IMEI NTOP_BASE_ID+285
 #define GTPV2_RESPONSE_CAUSE  NTOP_BASE_ID+333
 #define GTPV2_S5_S8_GTPC_TEID NTOP_BASE_ID+435
+#define GTPV2_C2S_S5_S8_GTPU_TEID NTOP_BASE_ID+439
+#define GTPV2_S2C_S5_S8_GTPU_TEID NTOP_BASE_ID+440
+#define GTPV2_C2S_S5_S8_GTPU_IP NTOP_BASE_ID+441
+#define GTPV2_S2C_S5_S8_GTPU_IP NTOP_BASE_ID+442
 
 /* BGP */
 #define SRC_AS_PATH_1 NTOP_BASE_ID+290
@@ -453,6 +466,7 @@
 #define DIAMETER_RSP_RESULT_CODE      NTOP_BASE_ID+404
 #define DIAMETER_EXP_RES_VENDOR_ID    NTOP_BASE_ID+405
 #define DIAMETER_EXP_RES_RESULT_CODE  NTOP_BASE_ID+406
+#define DIAMETER_HOP_BY_HOP_ID        NTOP_BASE_ID+445
 
 /* S1AP */
 #define S1AP_ENB_UE_S1AP_ID                    NTOP_BASE_ID+407
@@ -468,6 +482,5 @@
 #define APPLICATION_NAME                       NTOP_BASE_ID+427 /* App-Id */
 #define USER_NAME                              NTOP_BASE_ID+428 /* User-Id */
 
-/* Last used identifier is NTOP_BASE_ID+437 (RTP_SSRC) */
 
 #endif /* _NTOP_FLOW_H_ */
