@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -16,6 +16,7 @@ require "graph_utils"
 local enable_second_debug = 0
 
 ifnames = interface.getIfNames()
+
 for _,ifname in pairs(ifnames) do
    a = string.ends(ifname, ".pcap")
    if(not(a)) then
@@ -31,6 +32,8 @@ for _,ifname in pairs(ifnames) do
 	 ntop.mkdir(basedir)
       end
 
+      interface.setSecondTraffic()
+
       -- Traffic stats
       makeRRD(basedir, ifname, "bytes", 1, ifstats.bytes)
       makeRRD(basedir, ifname, "packets", 1, ifstats.packets)
@@ -40,5 +43,10 @@ for _,ifname in pairs(ifnames) do
       makeRRD(basedir, ifname, "num_hosts", 1, ifstats.hosts)
       makeRRD(basedir, ifname, "num_flows", 1, ifstats.flows)
       makeRRD(basedir, ifname, "num_http_hosts", 1, ifstats.http_hosts)
+
+      -- TCP stats
+      makeRRD(basedir, ifname, "tcp_retransmissions", 1, ifstats.tcpPacketStats.retransmissions)
+      makeRRD(basedir, ifname, "tcp_ooo", 1, ifstats.tcpPacketStats.out_of_order)
+      makeRRD(basedir, ifname, "tcp_lost", 1, ifstats.tcpPacketStats.lost)
    end
 end -- for _,ifname in pairs(ifnames) do
