@@ -71,17 +71,11 @@ function getStats ()
 --! @return "Host-to-Host Contact" if protocol id is equal to host family id, the protocol name if network interface is not null and null otherwise.
 function getnDPIProtoName (proto)
 
---! @memberof interface
---! @brief Get the hosts information of network interface.
---! @details For more information please read the @ref ntop_get_interface_hosts documentation.
---! @return An hashtable that contain the hosts information of the network interface.
-function getHosts ()
-
 --! @memberof interface
 --! @brief Get details hosts information of network interface.
 --! @details For more information please read the @ref ntop_get_interface_hosts_info documentation.
 --! @tparam bool show_details Boolean variable that define the details level. Optional, for default it is set to true.
---! @return An hashtable that contain the hosts information of the network interface.
+--! @return An hashtable that contain the hosts information of the network interface, and number of hits.
 function getHostsInfo (show_details)
 
 --! @memberof interface
@@ -111,9 +105,16 @@ function restoreHost(host_ip)
 --! @memberof interface
 --! @brief Get the flow information (minimal details) of network interface.
 --! @details For more information please read the @ref ntop_get_interface_flows_info documentation.
---! @return An hashtable containing the flow information of network interface with minimal details.
+--! @return An hashtable containing the flow information of network interface with minimal details, and number of hits.
 function getFlowsInfo()
 
+--! @memberof interface
+--! @memberof interface
+--! @brief Get the flow information (minimal details) of network interface for local hosts.
+--! @details Same as getFlowsInfo() but restricted to local hosts only
+--! @return An hashtable containing the flow information of network interface with minimal details, and number of hits.
+function getLocalFlowsInfo()
+
 --! @memberof interface
 --! @brief Get the flow peers information of network interface.
 --! @details For more information please read the @ref ntop_get_interface_flows_peers documentation.
@@ -292,10 +293,11 @@ function getQueuedAlerts(initial_idx, max_num_alerts)
 --! @memberof ntop
 --! @brief Returns the redis queued alerts
 --! @tparam integer alert_level The alert level. See AlertLevel in ntop_typedefs.h
+--! @tparam integer alert_status The alert status. See AlertStatus in ntop_typedefs.h
 --! @tparam integer alert_type The alert type. See AlertType in ntop_typedefs.h
 --! @tparam string alert_message A string message that describes the alert being reported.
 --! @details Returns up to the the number of queued alerts
-   function queueAlert(alert_level, alert_type, alert_message)
+   function queueAlert(alert_level, alert_status, alert_type, alert_message)
 
 --! @memberof ntop
 --! @brief Get the host name associated with the host id in the hash stored at key.
