@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -15,7 +15,6 @@ interface.select(ifname)
 ifstats = aggregateInterfaceStats(interface.getStats())
 -- printGETParameters(_GET)
 
-
 -- Table parameters
 all = _GET["all"]
 currentPage = _GET["currentPage"]
@@ -47,20 +46,18 @@ if(network_id ~= nil) then
    network_id = tonumber(network_id)
 end
 
-if ((sortColumn == nil) or (sortColumn == "column_"))then
-  sortColumn = getDefaultTableSort("flows")
+if sortColumn == nil or sortColumn == "column_" or sortColumn == "" then
+   sortColumn = getDefaultTableSort("flows")
+elseif sortColumn ~= "column_" and  sortColumn ~= "" then
+   tablePreferences("sort_flows",sortColumn)
 else
-   if((sortColumn ~= "column_") and (sortColumn ~= "")) then  
-    tablePreferences("sort_flows",sortColumn)
-  end
+   sortColumn = "column_client"
 end
 
-if(sortOrder == nil) then
+if sortOrder == nil then
   sortOrder = getDefaultTableSortOrder("flows")
-else
-  if ((sortColumn ~= "column_") and (sortColumn ~= "")) then   
-    tablePreferences("sort_order_flows",sortOrder) 
-  end
+elseif sortColumn ~= "column_" and sortColumn ~= "" then
+  tablePreferences("sort_order_flows",sortOrder)
 end
 
 if(currentPage == nil) then
@@ -80,26 +77,41 @@ if(port ~= nil) then port = tonumber(port) end
 
 to_skip = (currentPage-1) * perPage
 
-if (all ~= nil) then
+if(all ~= nil) then
    perPage = 0
    currentPage = 0
 end
 
+-- io.write("->"..sortColumn.."/"..perPage.."/"..sortOrder.."/"..sortColumn.."\n")
 interface.select(ifname)
-flows_stats = interface.getFlowsInfo(host)
+local a2z = false
+if(sortOrder == "desc") then a2z = false else a2z = true end
+
+local paginfo = {
+   ["sortColumn"] = sortColumn, ["toSkip"] = to_skip, ["maxHits"] = perPage,
+   ["a2zSortOrder"] = a2z,
+   ["hostFilter"] = host,
+   ["portFilter"] = port,
+   ["LocalNetworkFilter"] = network_id
+}
+
+if application ~= nil and application ~= "" then
+   paginfo["l7protoFilter"] = interface.getnDPIProtoId(application)
+   --print(paginfo["l7protoFilter"].." / "..application)
+end
+
+res = interface.getFlowsInfo(host, paginfo)
+flows_stats,total = aggregateFlowsStats(res)
 
 print ("{ \"currentPage\" : " .. currentPage .. ",\n \"data\" : [\n")
-total = 0
-
---host = "a"
-
+-- tprint(flows_stats)
 -- Prepare host
 host_list = {}
 num_host_list = 0
 single_host = 0
 
-if (hosts ~= nil) then host_list, num_host_list = getHostCommaSeparatedList(hosts) end
-if (host ~= nil) then
+if(hosts ~= nil) then host_list, num_host_list = getHostCommaSeparatedList(hosts) end
+if(host ~= nil) then
    single_host = 1
    num_host_list = 1
 end
@@ -109,27 +121,25 @@ num = 0
 
 if(flows_stats == nil) then flows_stats = { } end
 
-for key, value in pairs(flows_stats) do
-   -- print(key.."\n")
-   if (debug) then io.write("==================\n")end
+for key, value in ipairs(flows_stats) do
+   -- io.write(">>>> "..key.."\n")
+   if(debug) then io.write("==================\n") end
 
    process = true
    client_process = 0
    server_process = 0
 
-   if (debug) then io.write("Cli:"..flows_stats[key]["cli.ip"].."\t") end
-   if (debug) then io.write("Srv:"..flows_stats[key]["srv.ip"].."\n") end
+   if(debug) then io.write("Cli:"..flows_stats[key]["cli.ip"].."\t") end
+   if(debug) then io.write("Srv:"..flows_stats[key]["srv.ip"].."\n") end
 
-  
    if(vhost ~= nil) then
-      if((flows_stats[key]["cli.host"] ~= vhost) 
+      if((flows_stats[key]["cli.host"] ~= vhost)
       and (flows_stats[key]["srv.host"] ~= vhost)
-   and (flows_stats[key]["http.server_name"] ~= vhost)
-   and (flows_stats[key]["dns.last_query"] ~= vhost)) then
+   and (flows_stats[key]["protos.http.server_name"] ~= vhost)
+   and (flows_stats[key]["protos.dns.last_query"] ~= vhost)) then
 	 process = false
       end
    end
-   
 
    if(network_id ~= nil) then
       process = process and ((flows_stats[key]["cli.network_id"] == network_id) or (flows_stats[key]["srv.network_id"] == network_id))
@@ -139,103 +149,103 @@ for key, value in pairs(flows_stats) do
    if(l4proto ~= nil) then
       process = process and (flows_stats[key]["proto.l4"] == l4proto)
    end
-   if (debug and (not process)) then io.write("Stop L4\n") end
+   if(debug and (not process)) then io.write("Stop L4\n") end
 
    ---------------- USER ----------------
    if(user ~= nil) then
-      if (debug) then io.write("User:"..user.."\n")end
+      if(debug) then io.write("User:"..user.."\n") end
 
-      if (flows_stats[key]["client_process"] ~= nil) then
-         if (debug) then io.write("Client user:"..flows_stats[key]["client_process"]["user_name"].."\n") end
-         if ((flows_stats[key]["client_process"]["user_name"] == user)) then
+      if(flows_stats[key]["client_process"] ~= nil) then
+         if(debug) then io.write("Client user:"..flows_stats[key]["client_process"]["user_name"].."\n") end
+         if((flows_stats[key]["client_process"]["user_name"] == user)) then
             client_process = 1
          end
-         if (debug) then io.write("USER: => ClientProcess -\t"..client_process.."\n")end
+         if(debug) then io.write("USER: => ClientProcess -\t"..client_process.."\n") end
       end
 
-      if (flows_stats[key]["server_process"] ~= nil) then
-         if (debug) then io.write("Server user:"..flows_stats[key]["server_process"]["user_name"].."\n") end
-         if ((flows_stats[key]["server_process"]["user_name"] == user)) then
+      if(flows_stats[key]["server_process"] ~= nil) then
+         if(debug) then io.write("Server user:"..flows_stats[key]["server_process"]["user_name"].."\n") end
+         if((flows_stats[key]["server_process"]["user_name"] == user)) then
             server_process = 1
-            if (debug) then io.write("USER: => 1ServerProcess -\t"..server_process.."\n")end
+            if(debug) then io.write("USER: => 1ServerProcess -\t"..server_process.."\n") end
          end
-         if (debug) then io.write("USER: => ServerProcess -\t"..server_process.."\n")end
+         if(debug) then io.write("USER: => ServerProcess -\t"..server_process.."\n") end
       end
 
       process = process and ((client_process == 1) or (server_process == 1))
 
    end
-   if (debug and (not process)) then io.write("Stop user\n")end
+   if(debug and (not process)) then io.write("Stop user\n") end
 
    ---------------- PID ----------------
    if(pid ~= nil) then
-      if (debug) then io.write("Pid:"..pid.."\n")end
+      if(debug) then io.write("Pid:"..pid.."\n") end
 
-      if (flows_stats[key]["client_process"] ~= nil) then
-         if (debug) then io.write("Client pid:"..flows_stats[key]["client_process"]["pid"].."\n") end
-         if ((flows_stats[key]["client_process"]["pid"] == pid)) then
+      if(flows_stats[key]["client_process"] ~= nil) then
+         if(debug) then io.write("Client pid:"..flows_stats[key]["client_process"]["pid"].."\n") end
+         if((flows_stats[key]["client_process"]["pid"] == pid)) then
             client_process = 1
          end
-         if (debug) then io.write("PID: => ClientProcess -\t"..client_process.."\n")end
+         if(debug) then io.write("PID: => ClientProcess -\t"..client_process.."\n") end
       end
 
-      if (flows_stats[key]["server_process"] ~= nil) then
-         if (debug) then io.write("Server pid:"..flows_stats[key]["server_process"]["pid"].."\n") end
-         if ((flows_stats[key]["server_process"]["pid"] == pid)) then
+      if(flows_stats[key]["server_process"] ~= nil) then
+         if(debug) then io.write("Server pid:"..flows_stats[key]["server_process"]["pid"].."\n") end
+         if((flows_stats[key]["server_process"]["pid"] == pid)) then
             server_process = 1
          end
-         if (debug) then io.write("PID: => ServerProcess -\t"..server_process.."\n")end
+         if(debug) then io.write("PID: => ServerProcess -\t"..server_process.."\n") end
       end
 
        process = process and  ((client_process == 1) or (server_process == 1))
 
    end
-   if (debug and (not process)) then io.write("Stop Pid\n")end
+   if(debug and (not process)) then io.write("Stop Pid\n") end
 
    ---------------- NAME ----------------
    if(name ~= nil) then
-      if (debug) then io.write("Name:"..name.."\n")end
+      if(debug) then io.write("Name:"..name.."\n") end
 
-      if (flows_stats[key]["client_process"] ~= nil) then
-         if (debug) then io.write("Client name:"..flows_stats[key]["client_process"]["name"].."\n") end
-         if ((flows_stats[key]["client_process"]["name"] == name)) then
+      if(flows_stats[key]["client_process"] ~= nil) then
+         if(debug) then io.write("Client name:"..flows_stats[key]["client_process"]["name"].."\n") end
+         if((flows_stats[key]["client_process"]["name"] == name)) then
             client_process = 1
          end
-         if (debug) then io.write("ClientProcess -\t"..client_process.."\n")end
+         if(debug) then io.write("ClientProcess -\t"..client_process.."\n") end
       end
 
-      if (flows_stats[key]["server_process"] ~= nil) then
-         if (debug) then io.write("Server name:"..flows_stats[key]["server_process"]["name"].."\n") end
-         if ((flows_stats[key]["server_process"]["name"] == name)) then
+      if(flows_stats[key]["server_process"] ~= nil) then
+         if(debug) then io.write("Server name:"..flows_stats[key]["server_process"]["name"].."\n") end
+         if((flows_stats[key]["server_process"]["name"] == name)) then
             server_process = 1
          end
-         if (debug) then io.write("ServerProcess -\t"..server_process.."\n")end
+         if(debug) then io.write("ServerProcess -\t"..server_process.."\n") end
       end
       process = process and ((client_process == 1) or (server_process == 1))
 
    end
-   if (debug and (not process)) then io.write("Stop name\n")end
+   if(debug and (not process)) then io.write("Stop name\n") end
 
    ---------------- APP ----------------
    if(application ~= nil) then
       process = process and (string.ends(flows_stats[key]["proto.ndpi"], application))
+      -- print(flows_stats[key]["proto.ndpi"].." "..application.."<br>")
    end
-   if (debug and (not process)) then io.write("Stop ndpi\n")end
+   if(debug and (not process)) then io.write("Stop ndpi\n") end
 
    ---------------- PORT ----------------
    if(port ~= nil) then
       process = process and ((flows_stats[key]["cli.port"] == port) or (flows_stats[key]["srv.port"] == port))
    end
-   if (debug and (not process)) then io.write("Stop port\n")end
+   if(debug and (not process)) then io.write("Stop port\n") end
 
    ---------------- HOST ----------------
-   if(num_host_list > 0) then
+   if((num_host_list > 0) and process) then
       if(single_host == 1) then
-
-      if (debug) then io.write("Host:"..host_info["host"].."\n")end
-      if (debug) then io.write("Cli:"..flows_stats[key]["cli.ip"].."\n")end
-      if (debug) then io.write("Srv:"..flows_stats[key]["srv.ip"].."\n")end
-      if (debug) then io.write("vlan:"..flows_stats[key]["vlan"].."  ".. host_info["vlan"].."\n")end
+      if(debug) then io.write("Host:"..host_info["host"].."\n") end
+      if(debug) then io.write("Cli:"..flows_stats[key]["cli.ip"].."\n") end
+      if(debug) then io.write("Srv:"..flows_stats[key]["srv.ip"].."\n") end
+      if(debug) then io.write("vlan:"..flows_stats[key]["vlan"].."  ".. host_info["vlan"].."\n") end
 
       process = process and ((flows_stats[key]["cli.ip"] == host_info["host"]) or (flows_stats[key]["srv.ip"] == host_info["host"]))
       process = process and (flows_stats[key]["vlan"] == host_info["vlan"])
@@ -244,9 +254,8 @@ for key, value in pairs(flows_stats) do
       cli_num = findStringArray(flows_stats[key]["cli.ip"],host_list)
       srv_num = findStringArray(flows_stats[key]["srv.ip"],host_list)
 
-
-      if ( (cli_num ~= nil) and (srv_num ~= nil) ) then
-         if (cli_num and srv_num) then
+      if( (cli_num ~= nil) and (srv_num ~= nil) ) then
+         if(cli_num and srv_num) then
             process = process and (flows_stats[key]["cli.ip"] ~= flows_stats[key]["srv.ip"])
          else
             process = process and false
@@ -254,80 +263,84 @@ for key, value in pairs(flows_stats) do
       else
          process = process and false
       end
-
       end
    end
 
-   if (debug and (not process)) then io.write("Stop Host\n")end
-   
-   info = ""
-   if(flows_stats[key]["dns.last_query"] ~= nil) then 
-      info = shortenString(flows_stats[key]["dns.last_query"])
-      elseif(flows_stats[key]["http.last_url"] ~= nil) then
-      info = shortenString(flows_stats[key]["http.last_url"])
-      elseif(flows_stats[key]["ssl.certificate"] ~= nil) then
-      info = shortenString(flows_stats[key]["ssl.certificate"])
-   end
+   if(debug and (not process)) then io.write("Stop Host\n") end
 
-   flows_stats[key]["info"] = info	
-   if(flows_stats[key]["profile"] ~= nil) then 
+   info = ""
+   if(flows_stats[key]["protos.dns.last_query"] ~= nil) then
+      info = shortenString(flows_stats[key]["protos.dns.last_query"])
+      elseif(flows_stats[key]["protos.http.last_url"] ~= nil) then
+      info = shortenString(flows_stats[key]["protos.http.last_url"])
+      elseif(flows_stats[key]["protos.ssl.certificate"] ~= nil) then
+      info = shortenString(flows_stats[key]["protos.ssl.certificate"])
+      elseif(flows_stats[key]["bittorrent_hash"] ~= nil) then
+      info = shortenString(flows_stats[key]["bittorrent_hash"])
+      elseif(flows_stats[key]["host_server_name"] ~= nil) then
+      info = shortenString(flows_stats[key]["host_server_name"])
+      elseif(flows_stats[key]["icmp"] ~= nil) then
+      info = getICMPTypeCode(flows_stats[key]["icmp"])
+   end
+   flows_stats[key]["info"] = info
+
+   if(flows_stats[key]["profile"] ~= nil) then
       flows_stats[key]["info"] = "<span class='label label-primary'>"..flows_stats[key]["profile"].."</span> "..info
    end
-   
+
    ---------------- TABLE SORTING ----------------
+   -- TODO: as flows are returned already ordered via getFlowsInfo
+   -- this table sorting shall be removed. However, there are special
+   -- sort columns that I am not aware of, e.g., column_server_process
    if(process) then
-      if (debug_process) then io.write("Flow Processing\n")end
-      if (debug_process) then io.write("Cli: "..flows_stats[key]["cli.ip"].."\t") end
-      if (debug_process) then io.write("Srv: "..flows_stats[key]["srv.ip"].."\n") end
+      if(debug_process) then io.write("Flow Processing\n") end
+      if(debug_process) then io.write("Cli: "..flows_stats[key]["cli.ip"].."\t") end
+      if(debug_process) then io.write("Srv: "..flows_stats[key]["srv.ip"].."\n") end
       -- postfix is used to create a unique key otherwise entries with the same key will disappear
       num = num + 1
-      postfix = string.format("0.%04u", num)
       if(sortColumn == "column_client") then
-   vkey = flows_stats[key]["cli.ip"]..postfix
-   elseif(sortColumn == "column_server") then
-   vkey = flows_stats[key]["srv.ip"]..postfix
-   elseif(sortColumn == "column_bytes") then
-   vkey = flows_stats[key]["bytes"]+postfix
-   elseif(sortColumn == "column_vlan") then
-   vkey = flows_stats[key]["vlan"]+postfix
-   elseif(sortColumn == "column_bytes_last") then
-   vkey = flows_stats[key]["bytes.last"]+postfix
-   elseif(sortColumn == "column_info") then
-   vkey = info .. postfix
-   elseif(sortColumn == "column_ndpi") then
-   vkey = flows_stats[key]["proto.ndpi"]..postfix
-   elseif(sortColumn == "column_server_process") then
-   if(flows_stats[key]["server_process"] ~= nil) then
-      vkey = flows_stats[key]["server_process"]["name"]..postfix
-   else
-      vkey = postfix
-   end
-   elseif(sortColumn == "column_client_process") then
-   if(flows_stats[key]["client_process"] ~= nil) then
-      vkey = flows_stats[key]["client_process"]["name"]..postfix
-   else
-      vkey = postfix
-   end
-   elseif(sortColumn == "column_duration") then
-   vkey = flows_stats[key]["duration"]+postfix
-   elseif(sortColumn == "column_thpt") then
-   vkey = flows_stats[key]["throughput_"..throughput_type]+postfix
-   elseif(sortColumn == "column_proto_l4") then
-   vkey = flows_stats[key]["proto.l4"]..postfix
-   elseif(sortColumn == "column_ID") then
-   vkey = flows_stats[key]["ID"]..postfix
+	 vkey = flows_stats[key]["cli.ip"]
+      elseif(sortColumn == "column_server") then
+	 vkey = flows_stats[key]["srv.ip"]
+      elseif(sortColumn == "column_bytes") then
+	 vkey = flows_stats[key]["bytes"]
+      elseif(sortColumn == "column_vlan") then
+	 vkey = flows_stats[key]["vlan"]
+      elseif(sortColumn == "column_info") then
+	 -- add spaces before postfix as we sort for string
+	 vkey = flows_stats[key]["info"]
+      elseif(sortColumn == "column_ndpi") then
+	 vkey = flows_stats[key]["proto.ndpi"]
+      elseif(sortColumn == "column_server_process") then
+	 if(flows_stats[key]["server_process"] ~= nil) then
+	    vkey = flows_stats[key]["server_process"]["name"]
+	 else
+	    vkey = ""
+	 end
+      elseif(sortColumn == "column_client_process") then
+	 if(flows_stats[key]["client_process"] ~= nil) then
+	    vkey = flows_stats[key]["client_process"]["name"]
+	 else
+	    vkey = ""
+	 end
+      elseif(sortColumn == "column_duration") then
+	 vkey = flows_stats[key]["duration"]
+      elseif(sortColumn == "column_thpt") then
+	 vkey = flows_stats[key]["throughput_"..throughput_type]
+      elseif(sortColumn == "column_proto_l4") then
+	 vkey = flows_stats[key]["proto.l4"]
       else
-   -- By default sort by bytes
-   vkey = flows_stats[key]["bytes"]+postfix
+	 -- By default sort by bytes
+	 vkey = flows_stats[key]["bytes"]
       end
 
-      --      print("-->"..num.."="..vkey.."\n")
-      vals[vkey] = key
+      --io.write("-->"..key.."="..vkey.."\n")
+      -- vals[vkey] = key
+      vals[key] = vkey
    end
 end
 
 num = 0
-table.sort(vals)
 
 if(sortOrder == "asc") then
    funct = asc
@@ -335,20 +348,16 @@ else
    funct = rev
 end
 
-for k,v in pairs(vals) do
-   total = total + 1
-end
-
+--[[ TODO: check that actually vals is no longer needed
 for _key, _value in pairsByKeys(vals, funct) do
    key = vals[_key]
    value = flows_stats[key]
+--]]
+for _key, _value in pairsByValues(vals, funct) do
+   value = flows_stats[_key]
+   key = value["ntopng.key"]
 
-   --   print(key.."="..flows_stats[key]["duration"].."\n");
-   --   print(key.."=".."\n");
-   -- print(key.."/num="..num.."/perPage="..perPage.."/toSkip="..to_skip.."\n")
-   if (to_skip > 0) then
-      to_skip = to_skip-1
-   else
+     if(key ~= nil) then
       if((num < perPage) or (all ~= nil))then
    if(num > 0) then
       print ",\n"
@@ -362,12 +371,15 @@ for _key, _value in pairsByKeys(vals, funct) do
    if(cli_name == nil) then cli_name = "???" end
    if(srv_name == nil) then srv_name = "???" end
 
+   local cli_tooltip = cli_name
+   local srv_tooltip = srv_name
+
    if((value["tcp.nw_latency.client"] ~= nil) and (value["tcp.nw_latency.client"] > 0)) then
-      cli_tooltip = "client nw latency: "..string.format("%.3f", value["tcp.nw_latency.client"]).." ms"
+      cli_tooltip = cli_tooltip.."&#10;nw latency: "..string.format("%.3f", value["tcp.nw_latency.client"]).." ms"
    end
 
    if((value["tcp.nw_latency.server"] ~= nil) and (value["tcp.nw_latency.server"] > 0)) then
-      srv_tooltip = "server nw latency: "..string.format("%.3f", value["tcp.nw_latency.server"]).." ms"
+      srv_tooltip = srv_tooltip.."&#10;nw latency: "..string.format("%.3f", value["tcp.nw_latency.server"]).." ms"
    end
 
    if(value["cli.allowed_host"]) then
@@ -376,11 +388,11 @@ for _key, _value in pairsByKeys(vals, funct) do
 
    -- Flow username
    i, j = nil
-   if (flows_stats[key]["moreinfo.json"] ~= nil) then
-      i, j = string.find(flows_stats[key]["moreinfo.json"], '"57593":')
+   if(value["moreinfo.json"] ~= nil) then
+      i, j = string.find(value["moreinfo.json"], '"57593":')
    end
    if(i ~= nil) then
-      has_user = string.sub(flows_stats[key]["moreinfo.json"], j+2, j+3)
+      has_user = string.sub(value["moreinfo.json"], j+2, j+3)
       if(has_user == '""') then has_user = nil end
    end
    if(has_user ~= nil) then src_key = src_key .. " <i class='fa fa-user'></i>" end
@@ -391,10 +403,10 @@ for _key, _value in pairsByKeys(vals, funct) do
          else
       src_port=""
          end
-   else			    
+   else
      src_key = abbreviateString(cli_name, 20)
      src_port=":"..value["cli.port"]
-   end     
+   end
 
    if(value["srv.allowed_host"]) then
    dst_key="<A HREF='"..ntop.getHttpPrefix().."/lua/host_details.lua?".. hostinfo2url(value,"srv").. "' data-toggle='tooltip' title='" ..srv_tooltip.. "' >".. abbreviateString(srv_name, 20)
@@ -406,15 +418,14 @@ for _key, _value in pairsByKeys(vals, funct) do
          else
       dst_port=""
          end
-  else			    
+  else
      dst_key = abbreviateString(srv_name, 20)
      dst_port=":"..value["srv.port"]
-   end     
-
-   print ("{ \"key\" : \"" .. key..'\"')
+   end
 
+   print ("{ \"key\" : \"" .. value["ntopng.key"]..'\"')
    descr=cli_name..":"..value["cli.port"].." &lt;-&gt; "..srv_name..":"..value["srv.port"]
-   print (", \"column_key\" : \"<A HREF='"..ntop.getHttpPrefix().."/lua/flow_details.lua?flow_key=" .. key .. "&label=" .. descr)
+   print (", \"column_key\" : \"<A HREF='"..ntop.getHttpPrefix().."/lua/flow_details.lua?flow_key=" .. value["ntopng.key"] .. "&label=" .. descr)
    print ("'><span class='label label-info'>Info</span></A>")
    print ("\", \"column_client\" : \"" .. src_key)
 
@@ -443,7 +454,14 @@ for _key, _value in pairsByKeys(vals, funct) do
    -- if(value["category"] ~= nil) then print (", \"column_category\" : \"" .. value["category"] .. "\", ") else print (",") end
    print (", \"column_proto_l4\" : \"")
 
-   if(value["tcp.seq_problems"] == true) then
+   if(interface.isPacketInterface()) then
+      if(value["flow.status"] ~= 0) then
+	 print("<i class='fa fa-warning fa-lg' style='color: orange;'></i> ")
+      end
+   end
+
+   if ((((value["proto.l4"] == "TCP") or (value["proto.l4"] == "UDP")))
+      and ((value["tcp.seq_problems"] == true) or (value["flow_goodput.low"] == true))) then
       print("<font color=#B94A48>"..value["proto.l4"].."</font>")
    else
       print(value["proto.l4"])
@@ -469,12 +487,10 @@ for _key, _value in pairsByKeys(vals, funct) do
    print ("\", \"column_duration\" : \"" .. secondsToTime(value["duration"]))
    print ("\", \"column_bytes\" : \"" .. bytesToSize(value["bytes"]) .. "")
 
-   if (debug) then io.write ("throughput_type: "..throughput_type.."\n") end
-   if ( (value["throughput_trend_"..throughput_type] ~= nil) and
-     (value["throughput_trend_"..throughput_type] > 0)
-     ) then
-
-      if (throughput_type == "pps") then
+   if(debug) then io.write ("throughput_type: "..throughput_type.."\n") end
+   if((value["throughput_trend_"..throughput_type] ~= nil)
+       and (value["throughput_trend_"..throughput_type] > 0)) then
+      if(throughput_type == "pps") then
          print ("\", \"column_thpt\" : \"" .. pktsToSize(value["throughput_pps"]).. " ")
       else
          print ("\", \"column_thpt\" : \"" .. bitsToSize(8*value["throughput_bps"]).. " ")
@@ -498,7 +514,7 @@ for _key, _value in pairsByKeys(vals, funct) do
 
    print ("\", \"column_info\" : \"".. value["info"])
 
-   if(prefs.is_categorization_enabled and (value["info"] ~= "")) then
+   if(prefs.is_categorization_enabled and (value["info"] ~= "") and (key ~= nil)) then
       flow = interface.findFlowByKey(tonumber(key))
       if(flow ~= nil) then value["category"] = flow["category"] end
       if(value["category"] ~= "") then
@@ -507,10 +523,10 @@ for _key, _value in pairsByKeys(vals, funct) do
    end
 
    print(" \" }\n")
-   
+
    num = num + 1
-end
    end
+end
 end -- for
 
 print ("\n], \"perPage\" : " .. perPage.. ",\n")
