@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -35,7 +35,7 @@ end
 if((mode == nil) or (mode == "")) then mode = "contacted_peers" end
 
 interface.select(ifname)
-hosts_stats = interface.getHostsInfo()
+hosts_stats,total = aggregateHostsStats(interface.getHostsInfo())
 host_info = interface.getHostInfo(host)
 if(protocol_id == "") then protocol_id = nil end
 if(protocol_id ~= nil) then protocol_id = tonumber(protocol_id) end
