@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -62,7 +62,7 @@ if(sortOrder == nil) then
 end
 
 interface.select(ifname)
-flows_stats = interface.getFlowsInfo()
+flows_stats,total = aggregateFlowsStats(interface.getFlowsInfo())
 
 
 if (mode == "table") then
