@@ -6,11 +6,13 @@ print [[
       <div id="table-users"></div>
 	 <script>
 	 $("#table-users").datatable({
-	 	url: "/lua/admin/get_users.lua",
+	 	url: "]]
+print (ntop.getHttpPrefix())
+print [[/lua/admin/get_users.lua",
 	 	showPagination: true,
 	 	title: "Users",
 		buttons: [
-			"<a href='#add_user_dialog' role='button' class='add-on btn' data-toggle='modal' ><i class='glyphicon glyphicon-plus'></i></a>"
+			"<a href='#add_user_dialog' role='button' class='add-on btn' data-toggle='modal'><i class='fa fa-user-plus fa-lg'></i></a>"
 		],
 	 	columns: [
 			{
@@ -49,4 +51,4 @@ print [[
 	 });
          </script>
 
-   ]]
\ No newline at end of file
+   ]]
