@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -474,10 +474,29 @@ local mobile_country_code = {
 -- #######################
 
 function handleCustomFlowField(key, value)
-   if(key == 'TCP_FLAGS') then
+   if((key == 'TCP_FLAGS') or (key == '6')) then
       return(formatTcpFlags(value))
-   elseif((key == 'INPUT_SNMP') or (key == 'OUTPUT_SNMP')) then
+   elseif((key == 'INPUT_SNMP') or (key == '10')
+	     or (key == 'OUTPUT_SNMP') or (key == '14')) then
       return(formatInterfaceId(value))
+   elseif((key == 'EXPORTER_IPV4_ADDRESS') or (key == '130')) then
+      local b1, b2, b3, b4 = value:match("^(%d+)%.(%d+)%.(%d+)%.(%d+)$")
+      b1 = tonumber(b1)
+      b2 = tonumber(b2)
+      b3 = tonumber(b3)
+      b4 = tonumber(b4)
+      local ipaddr = string.format('%d.%d.%d.%d', b4, b3, b2, b1)
+      local res = ntop.getResolvedAddress(ipaddr)
+
+      local ret = "<A HREF="..ntop.getHttpPrefix().."/lua/host_details.lua?host="..ipaddr..">"
+      
+      if((res == "") or (res == nil)) then
+	 ret = ret .. ipaddr
+      else
+	 ret = ret .. res
+      end
+
+      return(ret .. "</A>")      
    elseif((key == 'FLOW_USER_NAME') or (key == '57593')) then
       elems = string.split(value, ';')
 
@@ -509,7 +528,7 @@ function handleCustomFlowField(key, value)
     if(value ~= '0') then
       return(formatEpoch(value))
     else
-      return ""
+      return "0"
     end
   elseif((rtemplate[tonumber(key)] == 'RTP_IN_JITTER') or (rtemplate[tonumber(key)] == 'RTP_OUT_JITTER')) then
     if(value ~= nil and value ~= '0') then
@@ -559,6 +578,8 @@ end
 
 -- #######################
 
+-- IMPORTANT: keep it in sync with ParserInterface::ParserInterface()
+
 local flow_fields_description = {
     ["IN_BYTES"] = "Incoming flow bytes (src->dst)",
     ["IN_PKTS"] = "Incoming flow packets (src->dst)",
@@ -688,6 +709,10 @@ local flow_fields_description = {
     ["PLUGIN_NAME"] = "Plugin name used by this flow (if any)",
     ["UNTUNNELED_IPV6_SRC_ADDR"] = "Untunneled IPv6 source address",
     ["UNTUNNELED_IPV6_DST_ADDR"] = "Untunneled IPv6 destination address",
+    ["SRC_IP_LONG"] = "Longitude where the src IP is located",
+    ["SRC_IP_LAT"] = "Latitude where the src IP is located",
+    ["DST_IP_LONG"] = "Longitude where the dst IP is located",
+    ["DST_IP_LAT"] = "Latitude where the dst IP is located",
     ["NUM_PKTS_TTL_EQ_1"] = "# packets with TTL = 1",
     ["NUM_PKTS_TTL_2_5"] = "# packets with TTL > 1 and TTL <= 5",
     ["NUM_PKTS_TTL_5_32"] = "# packets with TTL > 5 and TTL <= 32",
@@ -925,7 +950,6 @@ local flow_fields_description = {
     ['SIP_RTP_L4_SRC_PORT'] = "SIP RTP stream source port",
     ['SIP_RTP_IPV4_DST_ADDR'] = "SIP RTP stream dest IP",
     ['SIP_RTP_L4_DST_PORT'] = "SIP RTP stream dest port",
-    ['SIP_FAILURE_CODE'] = "SIP failure response code",
     ['SIP_RESPONSE_CODE'] = "SIP failure response code",
     ['SIP_REASON_CAUSE'] = "SIP Cancel/Bye/Failure reason cause",
     ['SIP_C_IP'] = "SIP C IP adresses",
@@ -1003,7 +1027,7 @@ end
 
 function getFlowValue(info, field)
    local return_value = ""
-   local value_original = ""
+   local value_original = "0"
    for key,value in pairs(info) do
      if(rtemplate[tonumber(key)] == field)then
        return_value = handleCustomFlowField(key, value)
@@ -1012,6 +1036,8 @@ function getFlowValue(info, field)
    end
    return_value = string.gsub(return_value, "<", "&lt;")
    return_value = string.gsub(return_value, ">", "&gt;")
+   return_value = string.gsub(return_value, "\"", "\\\"")
+
    return return_value , value_original
 end
 
@@ -1029,32 +1055,64 @@ end
 
 -- #######################
 
+function isThereSIPCall(info)
+  local retVal = 0
+  local call_state = getFlowValue(info, "SIP_CALL_STATE")
+  if((call_state ~= nil) and
+     ( (call_state == "CALL_STARTED")
+       or (call_state == "CALL_IN_PROGRESS")
+       or (call_state == "CALL_COMPLETED")
+       or (call_state == "CALL_ERROR")
+       or (call_state == "CALL_CANCELED") ) ) then
+    retVal = 1
+  end
+  return retVal
+end
+
+-- #######################
+
 function getSIPTableRows(info)
    local string_table = ""
+   local called_party = ""
+   local calling_party = ""
+   local sip_rtp_src_addr = 0
+   local sip_rtp_dst_addr = 0
    -- check if there is a SIP field
    sip_found = isThereProtocol("SIP", info)
 
+   if(sip_found == 1) then
+     sip_found = isThereSIPCall(info)
+   end
    if(sip_found == 1) then
      string_table = string_table.."<tr><th colspan=3 class=\"info\" >SIP Protocol Information</th></tr>\n"
      string_table = string_table.."<tr><th width=30%> Call-ID </th><td colspan=2><div id=call_id>" .. getFlowValue(info, "SIP_CALL_ID") .. "</div></td></tr>\n"
-     string_table = string_table.."<tr><th>Call Initiator <i class=\"fa fa-exchange fa-lg\"></i> Called Party</th><td colspan=2><div id=calling_called_party>" .. getFlowValue(info, "SIP_CALLING_PARTY") .. " <i class=\"fa fa-exchange fa-lg\"></i> " .. getFlowValue(info, "SIP_CALLED_PARTY") .. "</div></td></tr>\n"
+     called_party = getFlowValue(info, "SIP_CALLED_PARTY")
+     calling_party = getFlowValue(info, "SIP_CALLING_PARTY")
+     called_party = string.gsub(called_party, "\\\"","\"")
+     calling_party = string.gsub(calling_party, "\\\"","\"")
+     if(((called_party == nil) or (called_party == "")) and ((calling_party == nil) or (calling_party == ""))) then
+       string_table = string_table.."<tr><th>Call Initiator <i class=\"fa fa-exchange fa-lg\"></i> Called Party</th><td colspan=2><div id=calling_called_party></div></td></tr>\n"
+     else
+       string_table = string_table.."<tr><th>Call Initiator <i class=\"fa fa-exchange fa-lg\"></i> Called Party</th><td colspan=2><div id=calling_called_party>" .. calling_party .. " <i class=\"fa fa-exchange fa-lg\"></i> " .. called_party .. "</div></td></tr>\n"
+     end
+
      string_table = string_table.."<tr><th width=30%>RTP Codecs</th><td colspan=2> <div id=rtp_codecs>" .. getFlowValue(info, "SIP_RTP_CODECS") .. "</></td></tr>\n"
 
      time, time_epoch = getFlowValue(info, "SIP_INVITE_TIME")
      if(time_epoch ~= "0") then
-       string_table = string_table.."<tr><th width=30%>Time of Invite</th><td colspan=2> <div id=time_invite>" .. time .." [" .. secondsToTime(os.time()-time_epoch) .. " ago]</div></td></tr>\n"
+	string_table = string_table.."<tr><th width=30%>Time of Invite</th><td colspan=2> <div id=time_invite>" .. time .." [" .. secondsToTime(os.time()-tonumber(time_epoch)) .. " ago]</div></td></tr>\n"
      else
        string_table = string_table.."<tr><th width=30%>Time of Invite</th><td colspan=2></td><div id=time_invite></div></tr>\n"
      end
      time, time_epoch = getFlowValue(info, "SIP_TRYING_TIME")
      if(time_epoch ~= "0") then
-       string_table = string_table.."<tr><th width=30%>Time of Trying</th><td colspan=2><div id=time_trying>" .. time .." [" .. secondsToTime(os.time()-time_epoch) .. " ago]</div></td></tr>\n"
+       string_table = string_table.."<tr><th width=30%>Time of Trying</th><td colspan=2><div id=time_trying>" .. time .." [" .. secondsToTime(os.time()-tonumber(time_epoch)) .. " ago]</div></td></tr>\n"
      else
        string_table = string_table.."<tr><th width=30%>Time of Trying</th><td colspan=2><div id=time_trying></div></td></tr>\n"
      end
      time, time_epoch = getFlowValue(info, "SIP_RINGING_TIME")
      if(time_epoch ~= "0") then
-       string_table = string_table.."<tr><th width=30%>Time of Ringing</th><td colspan=2><div id=time_ringing>" .. time .." [" .. secondsToTime(os.time()-time_epoch) .. " ago]</div></td></tr>\n"
+       string_table = string_table.."<tr><th width=30%>Time of Ringing</th><td colspan=2><div id=time_ringing>" .. time .." [" .. secondsToTime(os.time()-tonumber(time_epoch)) .. " ago]</div></td></tr>\n"
      else
        string_table = string_table.."<tr><th width=30%>Time of Ringing</th><td colspan=2><div id=time_ringing></div></td></tr>\n"
      end
@@ -1063,11 +1121,11 @@ function getSIPTableRows(info)
      time_1, time_epoch_1 = getFlowValue(info, "SIP_INVITE_FAILURE_TIME")
      string_table = string_table .. "<tr><th width=30%>Time of Invite Ok / Invite Failure</th><td><div id=time_invite_ok>"
      if(time_epoch ~= "0") then
-        string_table = string_table .. time .." [" .. secondsToTime(os.time()-time_epoch) .. " ago]"
+        string_table = string_table .. time .." [" .. secondsToTime(os.time()-tonumber(time_epoch)) .. " ago]"
      end
      string_table = string_table .. "</div></td><td><div id=time_invite_failure>\n"
      if(time_epoch_1 ~= "0") then
-        string_table = string_table ..  time_1 .." [" .. secondsToTime(os.time()-time_epoch_1) .. " ago]"
+        string_table = string_table ..  time_1 .." [" .. secondsToTime(os.time()-tonumber(time_epoch_1)) .. " ago]"
      end
      string_table = string_table .. "</div></td></tr>\n"
 
@@ -1075,11 +1133,11 @@ function getSIPTableRows(info)
      time_1, time_epoch_1 = getFlowValue(info, "SIP_BYE_OK_TIME")
      string_table = string_table .. "<tr><th width=30%>Time of Bye / Bye Ok</th><td><div id=time_bye>"
      if(time_epoch ~= "0") then
-        string_table = string_table .. time .." [" .. secondsToTime(os.time()-time_epoch) .. " ago]"
+        string_table = string_table .. time .." [" .. secondsToTime(os.time()-tonumber(time_epoch)) .. " ago]"
      end
      string_table = string_table .. "</div></td><td><div id=time_bye_ok>\n"
      if(time_epoch_1 ~= "0") then
-        string_table = string_table ..  time_1 .." [" .. secondsToTime(os.time()-time_epoch_1) .. " ago]"
+        string_table = string_table ..  time_1 .." [" .. secondsToTime(os.time()-tonumber(time_epoch_1)) .. " ago]"
      end
      string_table = string_table .. "</div></td></tr>\n"
 
@@ -1087,18 +1145,35 @@ function getSIPTableRows(info)
      time_1, time_epoch_1 = getFlowValue(info, "SIP_CANCEL_OK_TIME")
      string_table = string_table .. "<tr><th width=30%>Time of Cancel / Cancel Ok</th><td><div id=time_cancel>"
      if(time_epoch ~= "0") then
-        string_table = string_table .. time .." [" .. secondsToTime(os.time()-time_epoch) .. " ago]"
+        string_table = string_table .. time .." [" .. secondsToTime(os.time()-tonumber(time_epoch)) .. " ago]"
      end
      string_table = string_table .. "</div></td><td><div id=time_cancel_ok>\n"
      if(time_epoch_1 ~= "0") then
-        string_table = string_table ..  time_1 .." [" .. secondsToTime(os.time()-time_epoch_1) .. " ago]"
+        string_table = string_table ..  time_1 .." [" .. secondsToTime(os.time()-tonumber(time_epoch_1)) .. " ago]"
      end
      string_table = string_table .. "</div></td></tr>\n"
 
      string_table = string_table.."<tr><th width=30%>RTP Stream Pears (src <i class=\"fa fa-exchange fa-lg\"></i> dst)</th><td colspan=2><div id=rtp_stream>"
-     if((getFlowValue(info, "SIP_RTP_IPV4_SRC_ADDR")~=nil) and (getFlowValue(info, "SIP_RTP_L4_DST_PORT")~=nil) and (getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~=nil) and (getFlowValue(info, "SIP_RTP_L4_DST_PORT")~=nil))then
-       string_table = string_table .. getFlowValue(info, "SIP_RTP_IPV4_SRC_ADDR") .. ":"..getFlowValue(info, "SIP_RTP_L4_SRC_PORT").." <i class=\"fa fa-exchange fa-lg\"></i> "..getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR") .. ":"..getFlowValue(info, "SIP_RTP_L4_DST_PORT")
+
+
+     if((getFlowValue(info, "SIP_RTP_IPV4_SRC_ADDR")~=nil) and (getFlowValue(info, "SIP_RTP_IPV4_SRC_ADDR")~="")) then
+       sip_rtp_src_addr = 1
+       string_table = string_table .. getFlowValue(info, "SIP_RTP_IPV4_SRC_ADDR")
      end
+     if((getFlowValue(info, "SIP_RTP_L4_SRC_PORT")~=nil) and (getFlowValue(info, "SIP_RTP_L4_SRC_PORT")~="") and (sip_rtp_src_addr == 1)) then
+       string_table = string_table ..":"..getFlowValue(info, "SIP_RTP_L4_SRC_PORT")
+     end
+     if((sip_rtp_src_addr == 1) or ((getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~=nil) and (getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~=""))) then
+       string_table = string_table.." <i class=\"fa fa-exchange fa-lg\"></i> "
+     end
+     if((getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~=nil) and (getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")~="")) then
+       sip_rtp_dst_addr = 1
+       string_table = string_table .. getFlowValue(info, "SIP_RTP_IPV4_DST_ADDR")
+     end
+     if((getFlowValue(info, "SIP_RTP_L4_DST_PORT")~=nil) and (getFlowValue(info, "SIP_RTP_L4_DST_PORT")~="") and (sip_rtp_dst_addr == 1)) then
+       string_table = string_table ..":"..getFlowValue(info, "SIP_RTP_L4_DST_PORT")
+     end
+
      string_table = string_table.."</div></td></tr>\n"
 
      string_table = string_table.."<tr><th width=30%> Failure Response Code </th><td colspan=2><div id=response_code>" .. getFlowValue(info, "SIP_RESPONSE_CODE") .. "</div></td></tr>\n"
@@ -1124,18 +1199,142 @@ function getRTPTableRows(info)
    if(rtp_found == 1) then
      string_table = string_table.."<tr><th colspan=3 class=\"info\" >RTP Protocol Information</th></tr>\n"
      string_table = string_table.."<tr><th width=30%> Sync Source ID </th><td colspan=2><div id=sync_source_id>" .. getFlowValue(info, "RTP_SSRC") .. "</div></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>First / Last Flow Timestamp</th><td><div id=first_flow_timestamp><i class='fa fa-clock-o fa-lg'></i>  "..getFlowValue(info, "RTP_FIRST_TS").."</div></td><td><div id=last_flow_timestamp><i class='fa fa-clock-o fa-lg'></i>  "..getFlowValue(info, "RTP_LAST_TS").."</div></td></tr>\n"
+     string_table = string_table .. "<tr><th width=30%>First / Last Flow Timestamp</th><td><div id=first_flow_timestamp>"
+     rtp_first_ts = getFlowValue(info, "RTP_FIRST_TS")
+     if((rtp_first_ts ~= nil) and (rtp_first_ts ~= "")) then
+       string_table = string_table .. "<i class='fa fa-clock-o fa-lg'></i>  "..rtp_first_ts
+     end
+     string_table = string_table .. "</div></td><td><div id=last_flow_timestamp>"
+     rtp_last_ts = getFlowValue(info, "RTP_LAST_TS")
+     if((rtp_last_ts ~= nil) and (rtp_last_ts ~= "")) then
+       string_table = string_table .. "<i class='fa fa-clock-o fa-lg'></i>  "..rtp_last_ts
+     end
+     string_table = string_table .. "</div></td></tr>\n"
      string_table = string_table .. "<tr><th width=30%>First / Last Flow Sequence</th><td><div id=first_flow_sequence>"..getFlowValue(info, "RTP_FIRST_SEQ").."</div></td><td><div id=last_flow_sequence>"..getFlowValue(info, "RTP_LAST_SEQ").."</div></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>Jitter IN / OUT</th><td><span id=jitter_in>"..getFlowValue(info, "RTP_IN_JITTER").." ms </span> <span id=jitter_in_trend></span></td><td><span id=jitter_out>"..getFlowValue(info, "RTP_OUT_JITTER").." ms </span> <span id=jitter_out_trend></span></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>Packet Lost in Stream IN / OUT</th><td><span id=packet_lost_in>"..formatPackets(getFlowValue(info, "RTP_IN_PKT_LOST")).."</span> <span id=packet_lost_in_trend></span></td><td><span id=packet_lost_out>"..formatPackets(getFlowValue(info, "RTP_OUT_PKT_LOST")).." </span> <span id=packet_lost_out_trend></span></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>Packet Discarded in Jitter Buffer IN / OUT</th><td><span id=packet_drop_in>"..formatPackets(getFlowValue(info, "RTP_IN_PKT_DROP")).."</span> <span id=packet_drop_in_trend></span></td><td><span id=packet_drop_out>"..formatPackets(getFlowValue(info, "RTP_OUT_PKT_DROP")).." </span> <span id=packet_drop_out_trend></span></td></tr>\n"
+     string_table = string_table .. "<tr><th width=30%>Jitter IN / OUT</th><td><span id=jitter_in>"
+     rtp_in_jitter = getFlowValue(info, "RTP_IN_JITTER")
+     if((rtp_in_jitter ~= nil) and (rtp_in_jitter ~= "")) then
+       string_table = string_table .. rtp_in_jitter.." ms "
+     end
+     string_table = string_table .. "</span> <span id=jitter_in_trend></span></td><td><span id=jitter_out>"
+     rtp_out_jitter = getFlowValue(info, "RTP_OUT_JITTER")
+     if((rtp_out_jitter ~= nil) and (rtp_out_jitter ~= "")) then
+       string_table = string_table .. rtp_out_jitter.." ms "
+     end
+     string_table = string_table .. "</span> <span id=jitter_out_trend></span></td></tr>\n"
+
+     string_table = string_table .. "<tr><th width=30%>Packet Lost in Stream IN / OUT</th><td><span id=packet_lost_in>"
+     rtp_in_pkt_lost = getFlowValue(info, "RTP_IN_PKT_LOST")
+     if((rtp_in_pkt_lost ~= nil) and (rtp_in_pkt_lost ~= "")) then
+       string_table = string_table .. formatPackets(rtp_in_pkt_lost)
+     end
+     string_table = string_table .. "</span> <span id=packet_lost_in_trend></span></td><td><span id=packet_lost_out>"
+     rtp_out_pkt_lost = getFlowValue(info, "RTP_OUT_PKT_LOST")
+     if((rtp_out_pkt_lost ~= nil) and (rtp_out_pkt_lost ~= "")) then
+       string_table = string_table .. formatPackets(rtp_out_pkt_lost)
+     end
+     string_table = string_table .. " </span> <span id=packet_lost_out_trend></span></td></tr>\n"
+
+     string_table = string_table .. "<tr><th width=30%>Packet Discarded in Jitter Buffer IN / OUT</th><td><span id=packet_drop_in>"
+     rtp_in_pkt_drop = getFlowValue(info, "RTP_IN_PKT_DROP")
+     if((rtp_in_pkt_drop ~= nil) and (rtp_in_pkt_drop ~= "")) then
+       string_table = string_table .. formatPackets(rtp_in_pkt_drop)
+     end
+     string_table = string_table .. "</span> <span id=packet_drop_in_trend></span></td><td><span id=packet_drop_out>"
+     rtp_out_pkt_drop = getFlowValue(info, "RTP_OUT_PKT_DROP")
+     if((rtp_out_pkt_drop ~= nil) and (rtp_out_pkt_drop ~= "")) then
+       string_table = string_table .. formatPackets(rtp_out_pkt_drop)
+     end
+     string_table = string_table .. " </span> <span id=packet_drop_out_trend></span></td></tr>\n"
+
      string_table = string_table .. "<tr><th width=30%>Payload Type IN / OUT</th><td><div id=payload_type_in>"..getFlowValue(info, "RTP_IN_PAYLOAD_TYPE").."</div></td><td><div id=payload_type_out>"..getFlowValue(info, "RTP_OUT_PAYLOAD_TYPE").."</div></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>Max Delta Time between Packets IN / OUT</th><td><span id=max_delta_time_in>"..getFlowValue(info, "RTP_IN_MAX_DELTA").." ms </span> <span id=max_delta_time_in_trend></span></td><td><span id=max_delta_time_out>"..getFlowValue(info, "RTP_OUT_MAX_DELTA").." ms </span> <span id=max_delta_time_out_trend></span></td></tr>\n"
+
+     string_table = string_table .. "<tr><th width=30%>Max Delta Time between Packets IN / OUT</th><td><span id=max_delta_time_in>"
+     rtp_in_max_delta = getFlowValue(info, "RTP_IN_MAX_DELTA")
+     if((rtp_in_max_delta ~= nil) and (rtp_in_max_delta ~= "")) then
+       string_table = string_table .. rtp_in_max_delta .. " ms "
+     end
+     string_table = string_table .. "</span> <span id=max_delta_time_in_trend></span></td><td><span id=max_delta_time_out>"
+     rtp_out_max_delta = getFlowValue(info, "RTP_OUT_MAX_DELTA")
+     if((rtp_out_max_delta ~= nil) and (rtp_out_max_delta ~= "")) then
+       string_table = string_table .. rtp_out_max_delta .. " ms "
+     end
+     string_table = string_table .. "</span> <span id=max_delta_time_out_trend></span></td></tr>\n"
      string_table = string_table .. "<tr><th width=30%> SIP Call Id  </th><td colspan=2><div id=rtp_sip_call_id>" .. getFlowValue(info, "RTP_SIP_CALL_ID") .. "</div></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>Quality of VoIP Communication (Average MOS/R-Factor) </th><td colspan=2><span><i class='fa fa-signal'></i>  </span><span id=mos_average>"..getFlowValue(info, "RTP_MOS").." </span> <span id=mos_average_trend></span><span> / </span><span id=r_factor_average>"..getFlowValue(info, "RTP_R_FACTOR").." </span> <span id=r_factor_average_trend></span></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>Quality of VoIP Communication (MOS/R-Factor) IN / OUT </th><td><span><i class='fa fa-signal'></i>  </span><span id=mos_in>"..getFlowValue(info, "RTP_IN_MOS").." </span> <span id=mos_in_trend></span><span> / </span><span id=r_factor_in>"..getFlowValue(info, "RTP_IN_R_FACTOR").." </span> <span id=r_factor_in_trend></span></td><td><span><i class='fa fa-signal'></i>  </span><span id=mos_out>"..getFlowValue(info, "RTP_OUT_MOS").." </span> <span id=mos_out_trend></span><span> / </span><span id=r_factor_out>"..getFlowValue(info, "RTP_OUT_R_FACTOR").." </span> <span id=r_factor_out_trend></span></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>RTP Transit IN / OUT</th><td><div id=rtop_transit_in>"..getFlowValue(info, "RTP_IN_TRANSIT").."</div></td><td><div id=rtp_transit_out>"..getFlowValue(info, "RTP_OUT_TRANSIT").."</div></td></tr>\n"
-     string_table = string_table .. "<tr><th width=30%>Round Trip Time</th><td colspan=2><span id=rtp_rtt>"..getFlowValue(info, "RTP_RTT").." ms </span> <span id=rtp_rtt_trend></span></td></tr>\n"
+
+     string_table = string_table .. "<tr><th width=30%>Quality of VoIP Communication (Average MOS/R-Factor) </th><td colspan=2><span id=mos_average_signal>"
+     rtp_mos = getFlowValue(info, "RTP_MOS")
+     rtp_r_factor = getFlowValue(info, "RTP_R_FACTOR")
+     if(((rtp_mos ~= nil) and (rtp_mos ~= "")) or ((rtp_r_factor ~= nil) and (rtp_r_factor ~= ""))) then
+       string_table = string_table .. "<i class='fa fa-signal'></i> "
+     end
+     string_table = string_table .. "</span><span id=mos_average>"
+     if((rtp_mos ~= nil) and (rtp_mos ~= "")) then
+       string_table = string_table .. rtp_mos
+     end
+     string_table = string_table .. "</span> <span id=mos_average_trend></span>"
+     string_table = string_table .. "<span id=mos_average_slash>"
+     if((rtp_r_factor ~= nil) and (rtp_r_factor ~= "")) then
+       string_table = string_table .. " / "
+     end
+     string_table = string_table .. "</span>"
+     string_table = string_table .. "<span id=r_factor_average>"
+     if((rtp_r_factor ~= nil) and (rtp_r_factor ~= "")) then
+       string_table = string_table .. rtp_r_factor
+     end
+     string_table = string_table .. " </span> <span id=r_factor_average_trend></span></td></tr>\n"
+
+
+     string_table = string_table .. "<tr><th width=30%>Quality of VoIP Communication (MOS/R-Factor) IN / OUT </th><td><span id=mos_in_signal>"
+     rtp_in_mos = getFlowValue(info, "RTP_IN_MOS")
+     rtp_in_r_factor = getFlowValue(info, "RTP_IN_R_FACTOR")
+     if(((rtp_in_mos ~= nil) and (rtp_in_mos ~= "")) or ((rtp_in_r_factor ~= nil) and (rtp_in_r_factor ~= ""))) then
+       string_table = string_table .. "<i class='fa fa-signal'></i> "
+     end
+     string_table = string_table .. "</span><span id=mos_in>"
+     if((rtp_in_mos ~= nil) and (rtp_in_mos ~= "")) then
+       string_table = string_table .. rtp_in_mos
+     end
+     string_table = string_table .. "</span> <span id=mos_in_trend></span>"
+     string_table = string_table .. "<span id=mos_in_slash>"
+     if((rtp_in_r_factor ~= nil) and (rtp_in_r_factor ~= "")) then
+       string_table = string_table .. " / "
+     end
+     string_table = string_table .. "</span>"
+     string_table = string_table .. "<span id=r_factor_in>"
+     if((rtp_in_r_factor ~= nil) and (rtp_in_r_factor ~= "")) then
+       string_table = string_table .. rtp_in_r_factor
+     end
+     string_table = string_table .. "</span> <span id=r_factor_in_trend></span></td><td><span id=mos_out_signal>"
+     rtp_out_mos = getFlowValue(info, "RTP_OUT_MOS")
+     rtp_out_r_factor = getFlowValue(info, "RTP_OUT_R_FACTOR")
+     if(((rtp_out_mos ~= nil) and (rtp_out_mos ~= "")) or ((rtp_out_r_factor ~= nil) and (rtp_out_r_factor ~= ""))) then
+       string_table = string_table .. "<i class='fa fa-signal'></i> "
+     end
+     string_table = string_table .. "</span><span id=mos_out>"
+     if((rtp_out_mos ~= nil) and (rtp_out_mos ~= "")) then
+       string_table = string_table .. rtp_out_mos
+     end
+     string_table = string_table .. "</span> <span id=mos_out_trend></span>"
+     string_table = string_table .. "<span id=mos_out_slash>"
+     if((rtp_out_r_factor ~= nil) and (rtp_out_r_factor ~= "")) then
+       string_table = string_table .. " / "
+     end
+     string_table = string_table .. "</span>"
+     string_table = string_table .. "<span id=r_factor_out>"
+     if((rtp_out_r_factor ~= nil) and (rtp_out_r_factor ~= "")) then
+       string_table = string_table .. rtp_out_r_factor
+     end
+     string_table = string_table .. " </span> <span id=r_factor_out_trend></span></td></tr>\n"
+
+     string_table = string_table .. "<tr><th width=30%>RTP Transit IN / OUT</th><td><div id=rtp_transit_in>"..getFlowValue(info, "RTP_IN_TRANSIT").."</div></td><td><div id=rtp_transit_out>"..getFlowValue(info, "RTP_OUT_TRANSIT").."</div></td></tr>\n"
+
+     string_table = string_table .. "<tr><th width=30%>Round Trip Time</th><td colspan=2><span id=rtp_rtt>"
+     rtp_rtt = getFlowValue(info, "RTP_RTT")
+     if((rtp_rtt ~= nil) and (rtp_rtt ~= "")) then
+       string_table = string_table .. rtp_rtt .. " ms "
+     end
+     string_table = string_table .. "</span> <span id=rtp_rtt_trend></span></td></tr>\n"
      string_table = string_table .. "<tr><th width=30%>DTMF tones sent during the call</th><td colspan=2><span id=dtmf_tones>"..getFlowValue(info, "RTP_DTMF_TONES").."</span></td></tr>\n"
    end
    return string_table
