@@ -1,7 +1,9 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 require "lua_utils"
+require "db_utils"
+require "historical_utils"
 
 top_rrds = {
    ["bytes.rrd"] = "Traffic",
@@ -9,7 +11,10 @@ top_rrds = {
    ["drops.rrd"] = "Packet Drops",
    ["num_flows.rrd"] = "Active Flows",
    ["num_hosts.rrd"] = "Active Hosts",
-   ["num_http_hosts.rrd"] = "Active HTTP Servers"
+   ["num_http_hosts.rrd"] = "Active HTTP Servers",
+   ["tcp_lost.rrd"] = "TCP Packets Lost",
+   ["tcp_ooo.rrd"] = "TCP Packets Out-Of-Order",
+   ["tcp_retransmissions.rrd"] = "TCP Retransmitted Packets",
 }
 
 -- ########################################################
@@ -62,7 +67,7 @@ function getProtoVolume(ifName, start_time, end_time)
 	 end
       end
    end
-   
+
    return(ret)
 end
 
@@ -92,7 +97,7 @@ function navigatedir(url, label, base, path, go_deep, print_html, ifid, host, st
 		  if(do_debug) then print(v.."<br>\n") end
 	       end
 	    end
-        else
+	else
 	    rrd = singlerrd2json(ifid, host, v, start_time, end_time, true, false)
 
 	    if((rrd.totalval ~= nil) and (rrd.totalval > 0)) then
@@ -108,15 +113,15 @@ function navigatedir(url, label, base, path, go_deep, print_html, ifid, host, st
 			shown = true
 		     end
 		  end
-		  
+
 		  what = string.sub(path.."/"..v, string.len(base)+2)
-		  
+
 		  label = string.sub(v,  1, string.len(v)-4)
 		  label = l4Label(string.gsub(label, "_", " "))
-		  
+
 		  ret[label] = what
 		  if(do_debug) then print(what.."<br>\n") end
-		  
+
 		  if(print_html) then
 		     if(not(printed)) then print('<li class="divider"></li>\n') printed = true end
 		     print("<li> <A HREF="..url..what..">"..label.."</A>  </li>\n")
@@ -136,11 +141,19 @@ end
 
 -- ########################################################
 
-function breakdownBar(sent, sentLabel, rcvd, rcvdLabel)
+function breakdownBar(sent, sentLabel, rcvd, rcvdLabel, thresholdLow, thresholdHigh)
    if((sent+rcvd) > 0) then
       sent2rcvd = round((sent * 100) / (sent+rcvd), 0)
+       --print(sent.."/"..rcvd.."/"..sent2rcvd)
+    if((thresholdLow == nil) or (thresholdLow < 0)) then thresholdLow = 0 end
+    if((thresholdHigh == nil) or (thresholdHigh > 100)) then thresholdHigh = 100 end
+
+    if(sent2rcvd < thresholdLow) then sentLabel = '<i class="fa fa-warning fa-lg"></i> '..sentLabel
+    elseif(sent2rcvd > thresholdHigh) then rcvdLabel = '<i class="fa fa-warning fa-lg""></i> '..rcvdLabel end
+
       print('<div class="progress"><div class="progress-bar progress-bar-warning" aria-valuenow="'.. sent2rcvd..'" aria-valuemin="0" aria-valuemax="100" style="width: ' .. sent2rcvd.. '%;">'..sentLabel)
       print('</div><div class="progress-bar progress-bar-info" aria-valuenow="'.. (100 -sent2rcvd)..'" aria-valuemin="0" aria-valuemax="100" style="width: ' .. (100-sent2rcvd) .. '%;">' .. rcvdLabel .. '</div></div>')
+
    else
       print('&nbsp;')
    end
@@ -169,6 +182,10 @@ function getRRDName(ifid, host_or_network, rrdFile)
    elseif host_or_network ~= nil and string.starts(host_or_network, 'profile:') then
        host_or_network = string.gsub(host_or_network, 'profile:', '')
        rrdname = fixPath(dirs.workingdir .. "/" .. ifid .. "/profilestats/")
+   elseif host_or_network ~= nil and string.starts(host_or_network, 'vlan:') then
+
+      host_or_network = string.gsub(host_or_network, 'vlan:', '')
+       rrdname = fixPath(dirs.workingdir .. "/" .. ifid .. "/vlanstats/")
    else
        rrdname = fixPath(dirs.workingdir .. "/" .. ifid .. "/rrd/")
    end
@@ -204,8 +221,8 @@ function getZoomAtPos(cur_zoom, pos_offset)
   for k,v in pairs(zoom_vals) do
     if(zoom_vals[k][1] == cur_zoom) then
       if (pos+pos_offset >= 1 and pos+pos_offset < 13) then
-        new_zoom_level = zoom_vals[pos+pos_offset][1]
-        break
+	new_zoom_level = zoom_vals[pos+pos_offset][1]
+	break
       end
     end
     pos = pos + 1
@@ -254,16 +271,16 @@ function drawPeity(ifid, host, rrdFile, zoomLevel, selectedEpoch)
 
    for k,v in ipairs(zoom_vals) do
       if(zoom_vals[k][1] == zoomLevel) then
-         if(k > 1) then
+	 if(k > 1) then
 	    nextZoomLevel = zoom_vals[k-1][1]
-         end
-         if(epoch) then
+	 end
+	 if(epoch) then
 	    start_time = epoch - zoom_vals[k][3]/2
 	    end_time = epoch + zoom_vals[k][3]/2
-         else
+	 else
 	    start_time = zoom_vals[k][2]
 	    end_time = "now"
-         end
+	 end
       end
    end
 
@@ -334,14 +351,30 @@ end
 -- ########################################################
 
 function drawRRD(ifid, host, rrdFile, zoomLevel, baseurl, show_timeseries,
-                 selectedEpoch, selected_epoch_sanitized, topArray)
+		 selectedEpoch, selected_epoch_sanitized, topArray)
    local debug_rrd = false
 
    if(zoomLevel == nil) then zoomLevel = "1h" end
 
    if((selectedEpoch == nil) or (selectedEpoch == "")) then
-      -- Refresh the page every minute unless a specific epoch has been selected
-      print("<script>setInterval(function() { window.location.reload();}, 60*1000); </script>\n");
+      -- Refresh the page every minute unless:
+      -- ** a specific epoch has been selected or
+      -- ** the user is browsing historical top talkers and protocols
+      print[[
+       <script>
+       setInterval(function() {
+	 var talkers_loaded, protocols_loaded;
+	 if($('a[href="#historical-top-talkers"]').length){
+	   talkers_loaded   = $('a[href="#historical-top-talkers"]').attr("loaded");
+	 }
+	 if($('a[href="#historical-top-apps"]').length){
+	   protocols_loaded = $('a[href="#historical-top-apps"]').attr("loaded");
+	 }
+	 if(typeof talkers_loaded == 'undefined' && typeof protocols_loaded == 'undefined'){
+	   window.location.reload();
+	 }
+       }, 60*1000);
+       </script>]]
    end
 
    if ntop.isPro() then
@@ -365,16 +398,16 @@ function drawRRD(ifid, host, rrdFile, zoomLevel, baseurl, show_timeseries,
 
    for k,v in ipairs(zoom_vals) do
       if(zoom_vals[k][1] == zoomLevel) then
-         if(k > 1) then
+	 if(k > 1) then
 	    nextZoomLevel = zoom_vals[k-1][1]
-         end
-         if(epoch) then
+	 end
+	 if(epoch) then
 	    start_time = epoch - zoom_vals[k][3]/2
 	    end_time = epoch + zoom_vals[k][3]/2
-         else
+	 else
 	    start_time = zoom_vals[k][2]
 	    end_time = "now"
-         end
+	 end
       end
    end
 
@@ -412,8 +445,29 @@ font-family: Arial, Helvetica, sans-serif;
 
 <div>
 
+<div class="container-fluid">
+  <ul class="nav nav-tabs" role="tablist" id="historical-tabs-container">
+    <li class="active"> <a href="#historical-tab-chart" role="tab" data-toggle="tab"> Chart </a> </li>
+]]
+
+if ntop.getPrefs().is_dump_flows_to_mysql_enabled
+   -- hide historical tabs for networks
+   and not string.starts(host, 'net:')
+then
+   print('<li><a href="#ipv4" role="tab" data-toggle="tab" id="tab-ipv4"> IPv4 Flows </a> </li>\n')
+  print('<li><a href="#ipv6" role="tab" data-toggle="tab" id="tab-ipv6"> IPv6 Flows </a> </li>\n')
+end
+
+print[[
+</ul>
+
+
+  <div class="tab-content">
+    <div class="tab-pane fade active in" id="historical-tab-chart">
+
+<br>
 <table border=0>
-<tr><td valign=top>
+<tr><td valign="top">
 ]]
 
 
@@ -442,7 +496,7 @@ end
 d = fixPath(p)
 
    go_deep = false
-   navigatedir(baseurl .. '&graph_zoom=' .. zoomLevel .. '&epoch=' .. (selectedEpoch or '')..'&rrd_file=', 
+   navigatedir(baseurl .. '&graph_zoom=' .. zoomLevel .. '&epoch=' .. (selectedEpoch or '')..'&rrd_file=',
 	       "*", d, d, go_deep, true, ifid, host, start_time, end_time)
 
    print [[
@@ -458,10 +512,11 @@ for k,v in ipairs(zoom_vals) do
    -- but exclude applications. Application statistics are gathered
    -- every 5 minutes
    local net_or_profile = false
+
    if host and (string.starts(host, 'net:') or string.starts(host, 'profile:')) then
        net_or_profile = true
    end
-   if zoom_vals[k][1] == '1m' and (not net_or_profile and not top_rrds[rrdFile]) then
+   if zoom_vals[k][1] == '1m' and (net_or_profile or (not net_or_profile and not top_rrds[rrdFile])) then
        goto continue
    end
    print('<label class="btn btn-link ')
@@ -504,6 +559,7 @@ print [[
 
 
 ]]
+
 if(string.contains(rrdFile, "num_")) then
    formatter_fctn = "fint"
 else
@@ -519,7 +575,7 @@ print('   <tr><th>&nbsp;</th><th>Time</th><th>Value</th></tr>\n')
 
 rrd = rrd2json(ifid, host, rrdFile, start_time, end_time, true, false) -- the latest false means: expand_interface_views
 
-if(string.contains(rrdFile, "num_") or string.contains(rrdFile, "packets")  or string.contains(rrdFile, "drops")) then
+if(string.contains(rrdFile, "num_") or string.contains(rrdFile, "tcp_") or string.contains(rrdFile, "packets")  or string.contains(rrdFile, "drops")) then
    print('   <tr><th>Min</th><td>' .. os.date("%x %X", rrd.minval_time) .. '</td><td>' .. formatValue(rrd.minval) .. '</td></tr>\n')
    print('   <tr><th>Max</th><td>' .. os.date("%x %X", rrd.maxval_time) .. '</td><td>' .. formatValue(rrd.maxval) .. '</td></tr>\n')
    print('   <tr><th>Last</th><td>' .. os.date("%x %X", rrd.lastval_time) .. '</td><td>' .. formatValue(round(rrd.lastval), 1) .. '</td></tr>\n')
@@ -535,18 +591,35 @@ else
 end
 
 print('   <tr><th>Selection Time</th><td colspan=2><div id=when></div></td></tr>\n')
-print('   <tr><th>Minute<br>Top Talkers</th><td colspan=2><div id=talkers></div></td></tr>\n')
+print('   <tr><th>Minute<br>Interface<br>Top Talkers</th><td colspan=2><div id=talkers></div></td></tr>\n')
 
 
 print [[
-
    </table>
 ]]
 end -- topArray ~= nil
 
-print[[</div></td></tr></table><p>]]
-printGraphTopFlows(ifid, (host or ''), _GET["epoch"], zoomLevel, rrdFile)
-print [[
+print[[</div></td></tr></table>
+
+    </div> <!-- closes div id "historical-tab-chart "-->
+]]
+
+if ntop.getPrefs().is_dump_flows_to_mysql_enabled
+   -- hide historical tabs for networks and profiles
+   and not string.starts(host, 'net:')
+then
+  if tonumber(start_time) ~= nil and tonumber(end_time) ~= nil then
+    -- if both start_time and end_time are vaid epoch we can print finer-grained top flows
+    printTopFlows(ifid, (host or ''), start_time, end_time, rrdFile, '', '', '', 5, 5)
+  else
+    printGraphTopFlows(ifid, (host or ''), _GET["epoch"], zoomLevel, rrdFile)
+  end
+end
+
+print[[
+  </div> <!-- closes div class "tab-content" -->
+</div> <!-- closes div class "container-fluid" -->
+
 <script>
 
 var palette = new Rickshaw.Color.Palette();
@@ -640,36 +713,36 @@ print[[
 
 infoHTML += "<ul>";
 $.ajax({
-          type: 'GET',
-          url: ']]
-          print(ntop.getHttpPrefix().."/lua/top_generic.lua?m=top_talkers&epoch='+point.value.x+'&addvlan=true")
-            print [[',
-                  data: { epoch: point.value.x },
-                  async: false,
-                  success: function(content) {
-                   var info = jQuery.parseJSON(content);
-                   $.each(info, function(i, n) {
-                     if (n.length > 0)
-                       infoHTML += "<li>"+capitaliseFirstLetter(i)+" [Avg Traffic/sec]<ol>";
-                     var items = 0;
-                     var other_traffic = 0;
-                     $.each(n, function(j, m) {
-                       if(items < 3) {
-                         infoHTML += "<li><a href='host_details.lua?host="+m.address+"'>"+abbreviateString(m.label ? m.label : m.address,24);
-                       infoHTML += "</a>";
-                       if (m.vlan != "0") infoHTML += " ("+m.vlanm+")";
-                       infoHTML += " ("+fbits((m.value*8)/60)+")</li>";
-                         items++;
-                       } else
-                         other_traffic += m.value;
-                     });
-                     if (other_traffic > 0)
-                         infoHTML += "<li>Other ("+fbits((other_traffic*8)/60)+")</li>";
-                     if (n.length > 0)
-                       infoHTML += "</ol></li>";
-                   });
-                   infoHTML += "</ul></li></li>";
-           }
+	  type: 'GET',
+	  url: ']]
+	  print(ntop.getHttpPrefix().."/lua/top_generic.lua?m=top_talkers&epoch='+point.value.x+'&addvlan=true&id="..tostring(ifid))
+	    print [[',
+		  data: { epoch: point.value.x },
+		  async: false,
+		  success: function(content) {
+		   var info = jQuery.parseJSON(content);
+		   $.each(info, function(i, n) {
+		     if (n.length > 0)
+		       infoHTML += "<li>"+capitaliseFirstLetter(i)+" [Avg Traffic/sec]<ol>";
+		     var items = 0;
+		     var other_traffic = 0;
+		     $.each(n, function(j, m) {
+		       if(items < 3) {
+			 infoHTML += "<li><a href='host_details.lua?host="+m.address+"'>"+abbreviateString(m.label ? m.label : m.address,24);
+		       infoHTML += "</a>";
+		       if (m.vlan != "0") infoHTML += " ("+m.vlanm+")";
+		       infoHTML += " ("+fbits((m.value*8)/60)+")</li>";
+			 items++;
+		       } else
+			 other_traffic += m.value;
+		     });
+		     if (other_traffic > 0)
+			 infoHTML += "<li>Other ("+fbits((other_traffic*8)/60)+")</li>";
+		     if (n.length > 0)
+		       infoHTML += "</ol></li>";
+		   });
+		   infoHTML += "</ul></li></li>";
+	   }
    });
 infoHTML += "</ul>";]]
 end -- topArray
@@ -905,11 +978,21 @@ function printGraphTopFlows(ifId, host, epoch, zoomLevel, l7proto)
    epoch_end = epoch
    epoch_begin = epoch-d
 
-   printTopFlows(ifId, host, epoch_begin, epoch_end, l7proto, '', '', '', 5, 5)
+   local tot_flows = { }
+   local versions = { ['IPv4']=4, ['IPv6']=6 }
+   for k,v in pairs(versions) do
+      local res = getNumFlows(ifId, v, (host or ''), '', '', (l7proto or ''), '', epoch_begin, epoch_end)
+
+      for _,flow in pairs(res) do
+	 tot_flows[v] = flow['TOT_FLOWS']
+      end
+   end
+
+   printTopFlows(ifId, host, epoch_begin, epoch_end, l7proto, '', '', '', tot_flows[4], tot_flows[6])
 end
 
 function printTopFlows(ifId, host, epoch_begin, epoch_end, l7proto, l4proto, port, info, limitv4, limitv6)
-   url_update = "/lua/get_db_flows.lua?ifId="..ifId.. "&host="..(host or '') .. "&epoch_begin="..epoch_begin.."&epoch_end="..epoch_end.."&l4proto="..l4proto.."&port="..port.."&info="..info
+   url_update = ntop.getHttpPrefix().."/lua/get_db_flows.lua?ifId="..ifId.. "&host="..(host or '') .. "&epoch_begin="..(epoch_begin or '').."&epoch_end="..(epoch_end or '').."&l4proto="..(l4proto or '').."&port="..(port or '').."&info="..(info or '')
 
    if(l7proto ~= "") then
       if(not(isnumber(l7proto))) then
@@ -942,12 +1025,6 @@ function printTopFlows(ifId, host, epoch_begin, epoch_end, l7proto, l4proto, por
       title = ""
    end
 
-   print [[
-   <div class="container-fluid">
-
-    <!-- Nav tabs -->
-    <ul class="nav nav-tabs" role="tablist">
-]]
 
 if(host ~= nil) then
   local chunks = {host:match("(%d+)%.(%d+)%.(%d+)%.(%d+)")}
@@ -956,41 +1033,77 @@ if(host ~= nil) then
   end
 end
 
-
-selected = false
-if(not((limitv4 == nil) or (limitv4 == "") or (limitv4 == "0"))) then
-   print('<li class="active"> <a href="#ipv4" role="tab" data-toggle="tab"> IPv4  </a> </li>\n')
-   selected = true
+-- prepare some attributes that will be attached to divs
+local div_data = ""
+if epoch_begin ~= "" and epoch_begin ~= nil then
+   div_data = div_data..' epoch_begin="'..tostring(epoch_begin)..'" '
+end
+if epoch_end ~= "" and epoch_end ~= nil then
+   div_data = div_data..' epoch_end="'..tostring(epoch_end)..'" '
+end
+if host ~= "" and host ~= nil then
+   div_data = div_data..' host="'..tostring(host)..'" '
+end
+if l7proto ~= "" and l7proto ~= nil then
+   div_data = div_data..' l7_proto_id="'..l7proto..'" '
+end
+if l4proto ~= "" and l4proto ~= nil then
+   div_data = div_data..' l4_proto_id="'..l4proto..'" '
+end
+if port ~= "" and port ~= nil then
+   div_data = div_data..' port="'..port..'" '
 end
 
-if(not((limitv6 == nil) or (limitv6 == "") or (limitv6 == "0"))) then
-   if(selected == false) then print('<li class="active">\n') else print('<li>\n') end
-   print('<a href="#ipv6" role="tab" data-toggle="tab"> IPv6 </a> </li>\n')
-end
+local ipv4_enabled = not(limitv4 == nil or limitv4 == "" or limitv4 == "0")
+local ipv6_enabled = not(limitv6 == nil or limitv6 == "" or limitv6 == "0")
 
+if ipv4_enabled == true then
 print [[
-    </ul>
-
-    <!-- Tab panes -->
-    <div class="tab-content">
-   ]]
-
-if(not((limitv4 == nil) or (limitv4 == "") or (limitv4 == "0"))) then
-print [[
-      <div class="tab-pane fade active in" id="ipv4"> <div id="table-flows4"></div> </div>
+      <div class="tab-pane fade" id="ipv4" ]] print(div_data) print[[">
+	<div id="table-flows4"></div>
+]] historicalDownloadButtonsBar('flows_v4', 'ipv4',
+				ipv4_enabled,
+				ipv6_enabled) print[[
+      </div>
+]]
+else
+print[[
+  <script>
+    $('#tab-ipv4').remove()
+  </script>
 ]]
 end
 
-if(not((limitv6 == nil) or (limitv6 == "") or (limitv6 == "0"))) then
-   if(selected == false) then print('<div class="tab-pane fade active in" id="ipv6">\n') else print('<div class="tab-pane fade" id="ipv6">\n') end
-   print('<div id="table-flows6"></div> </div>')
+if ipv6_enabled == true then
+print[[
+      <div class="tab-pane fade" id="ipv6" ]] print(div_data) print[[">
+    <div id="table-flows6"></div>
+]] historicalDownloadButtonsBar('flows_v6', 'ipv6',
+				ipv4_enabled,
+				ipv6_enabled
+			       ) print[[
+  </div>
+]]
+else
+print[[
+  <script>
+    $('#tab-ipv6').remove()
+  </script>
+]]
 end
 
 print [[
-    </div>
-</div>
 
-<script>
+      <script>
+      $('a[href="#ipv4"]').on('shown.bs.tab', function (e) {
+        if ($('a[href="#ipv4"]').attr("loaded") == 1){
+          // do nothing if the tab has already been computed and populated
+          return;
+        }
+
+        // if here, then we actually have to load the datatable
+        $('a[href="#ipv4"]').attr("loaded", 1);
+
    ]]
 
 if(not((limitv4 == nil) or (limitv4 == "") or (limitv4 == "0"))) then
@@ -1072,7 +1185,23 @@ print [[
 							  }
 						       },
 						       {
-							  title: "Bytes",
+							  title: "Traffic Sent",
+							  field: "IN_BYTES",
+							  sortable: true,
+							  css: {
+							     textAlign: 'right'
+							  }
+						       },
+						       {
+							  title: "Traffic Received",
+							  field: "OUT_BYTES",
+							  sortable: true,
+							  css: {
+							     textAlign: 'right'
+							  }
+						       },
+						       {
+							  title: "Total Traffic",
 							  field: "BYTES",
 							  sortable: true,
 							  css: {
@@ -1084,7 +1213,7 @@ print [[
 							  field: "INFO",
 							  sortable: true,
 							  css: {
-							     textAlign: 'right'
+							     textAlign: 'left'
 							  }
 						       },
 						       {
@@ -1101,7 +1230,8 @@ print [[
 
 
 
-            var table4 = $("#table-flows4").datatable(graph_options4);
+	    var table4 = $("#table-flows4").datatable(graph_options4);
+      });  // closes the event handler on shown.bs.tab
       ]]
    end
 
@@ -1109,6 +1239,16 @@ print [[
       if((limitv6 == nil) or (limitv6 == "") or (limitv6 == "0")) then print("</script>") return end
 
 print [[
+      $('a[href="#ipv6"]').on('shown.bs.tab', function (e) {
+        if ($('a[href="#ipv6"]').attr("loaded") == 1){
+          // do nothing if the tab has already been computed and populated
+          return;
+        }
+
+        // if here, then we actually have to load the datatable
+        $('a[href="#ipv6"]').attr("loaded", 1);
+
+
 	    var url_update6 = "]] print(url_update.."&limit="..limitv6) print [[&version=6";
 
 	    var graph_options6 = {
@@ -1187,7 +1327,23 @@ print [[
 							  }
 						       },
 						       {
-							  title: "Bytes",
+							  title: "Traffic Sent",
+							  field: "IN_BYTES",
+							  sortable: true,
+							  css: {
+							     textAlign: 'right'
+							  }
+						       },
+						       {
+							  title: "Traffic Received",
+							  field: "OUT_BYTES",
+							  sortable: true,
+							  css: {
+							     textAlign: 'right'
+							  }
+						       },
+						       {
+							  title: "Total Traffic",
 							  field: "BYTES",
 							  sortable: true,
 							  css: {
@@ -1208,6 +1364,7 @@ print [[
 
 
 	 var table6 = $("#table-flows6").datatable(graph_options6);
+      }); // closes the event handler on shown.bs.tab
 	 </script>
 ]]
 
@@ -1229,16 +1386,15 @@ function singlerrd2json(ifid, host, rrdFile, start_time, end_time, rickshaw_json
    local scaling_factor = 8
 
    --io.write(prefixLabel.."\n")
-   if(prefixLabel == "Bytes") then
+   if(prefixLabel == "Bytes" or string.starts(rrdFile, 'categories/')) then
       prefixLabel = "Traffic"
    end
 
-   if(string.contains(rrdFile, "num_") or string.contains(rrdFile, "packets")  or string.contains(rrdFile, "drops"))
-    then
-        -- do not scale number, packets, and drops
-        scaling_factor = 1
+   if(string.contains(rrdFile, "num_") or string.contains(rrdFile, "tcp_") or string.contains(rrdFile, "packets") or string.contains(rrdFile, "drops")) then
+      -- do not scale number, packets, and drops
+      scaling_factor = 1
    end
-   
+
    if(not ntop.notEmptyFile(rrdname)) then return '{}' end
 
    local fstart, fstep, fnames, fdata = ntop.rrd_fetch(rrdname, 'AVERAGE', start_time, end_time)
@@ -1254,27 +1410,27 @@ function singlerrd2json(ifid, host, rrdFile, start_time, end_time, rickshaw_json
    local sample_rate = round(num_points_found / max_num_points)
 
    if(sample_rate < 1) then sample_rate = 1 end
-   
+
    -- prepare rrd labels
    for i, n in ipairs(fnames) do
       -- handle duplicates
       if (names_cache[n] == nil) then
-         local extra_info = ''
+	 local extra_info = ''
 	 names_cache[n] = true
-         if append_ifname_to_labels then
-             extra_info = getInterfaceName(ifid)
-         end
-	 if host ~= nil and not string.starts(host, 'profile:') then
-             extra_info = extra_info.." ".. firstToUpper(n)
-         end
-         if extra_info ~= "" then
-             names[#names+1] = prefixLabel.." ("..trimSpace(extra_info)..")"
-         else
-             names[#names+1] = prefixLabel
-         end
+	 if append_ifname_to_labels then
+	     extra_info = getInterfaceName(ifid)
+	 end
+	 if host ~= nil and not string.starts(host, 'profile:') and not string.starts(rrdFile, 'categories/') then
+	     extra_info = extra_info.." ".. firstToUpper(n)
+	 end
+	 if extra_info ~= "" then
+	     names[#names+1] = prefixLabel.." ("..trimSpace(extra_info)..")"
+	 else
+	     names[#names+1] = prefixLabel
+	 end
       end
     end
-   
+
    local minval, maxval, lastval = 0, 0, 0
    local maxval_time, minval_time, lastval_time  = nil, nil, nil
    local sampling = 1
@@ -1295,47 +1451,47 @@ function singlerrd2json(ifid, host, rrdFile, start_time, end_time, rickshaw_json
 	 else
 	    --io.write(w.."\n")
 	    w = tonumber(w)
-            if(w < 0) then
+	    if(w < 0) then
 	       w = 0
 	    end
 	 end
 
-         -- update the total value counter, which is the non-scaled integral over time
-         totalval[instant] = totalval[instant] + w * fstep
-         -- also update the average val (do not multiply by fstep, this is not the integral)
-         avgval[instant] = avgval[instant] + w
-         -- and the scaled current value (remember that these are derivatives)
-         w = w * scaling_factor
-         -- the scaled current value w goes into its own element elemId
+	 -- update the total value counter, which is the non-scaled integral over time
+	 totalval[instant] = totalval[instant] + w * fstep
+	 -- also update the average val (do not multiply by fstep, this is not the integral)
+	 avgval[instant] = avgval[instant] + w
+	 -- and the scaled current value (remember that these are derivatives)
+	 w = w * scaling_factor
+	 -- the scaled current value w goes into its own element elemId
 	 if (s[elemId] == nil) then s[elemId] = 0 end
 	 s[elemId] = s[elemId] + w
 	 --if(s[elemId] > 0) then io.write("[".. elemId .. "]=" .. s[elemId] .."\n") end
 	 elemId = elemId + 1
       end
-      
+
       -- stops every sample_rate samples, or when there are no more points
       if(sampling == sample_rate or num_points_found == i) then
-         local sample_sum = 0
-         for elemId=1,#s do
-            -- calculate the average in the sampling period
+	 local sample_sum = 0
+	 for elemId=1,#s do
+	    -- calculate the average in the sampling period
 	    s[elemId] = s[elemId] / sampling
-            sample_sum = sample_sum + s[elemId]
+	    sample_sum = sample_sum + s[elemId]
+	 end
+	 -- update last instant
+	 if lastval_time == nil or instant > lastval_time then
+	    lastval = sample_sum
+	    lastval_time = instant
+	 end
+	 -- possibly update maximum value (grab the most recent in case of a tie)
+	 if maxval_time == nil or (sample_sum >= maxval and instant > maxval_time) then
+	    maxval = sample_sum
+	    maxval_time = instant
+	 end
+	 -- possibly update the minimum value (grab the most recent in case of a tie)
+	 if minval_time == nil or (sample_sum <= minval and instant > minval_time) then
+	    minval = sample_sum
+	    minval_time = instant
 	 end
-         -- update last instant
-         if lastval_time == nil or instant > lastval_time then
-            lastval = sample_sum
-            lastval_time = instant
-         end
-         -- possibly update maximum value (grab the most recent in case of a tie)
-         if maxval_time == nil or (sample_sum >= maxval and instant > maxval_time) then
-            maxval = sample_sum
-            maxval_time = instant
-         end
-         -- possibly update the minimum value (grab the most recent in case of a tie)
-         if minval_time == nil or (sample_sum <= minval and instant > minval_time) then
-            minval = sample_sum
-            minval_time = instant
-         end
 	 series[#series+1] = s
 	 sampling = 1
 	 s = {}
@@ -1502,9 +1658,9 @@ function rrd2json(ifid, host, rrdFile, start_time, end_time, rickshaw_json, expa
    local ret = {}
    local num = 0
    local debug_metric = false
-   
+
    interface.select(getInterfaceName(ifid))
-   local ifstats = interface.getStats()   
+   local ifstats = interface.getStats()
    local rrd_if_ids = {}  -- read rrds for interfaces listed here
    rrd_if_ids[1] = ifid -- the default submitted interface
    -- interface.select(getInterfaceName(ifid))
@@ -1515,14 +1671,14 @@ function rrd2json(ifid, host, rrdFile, start_time, end_time, rickshaw_json, expa
        io.write('ifstats.isView: '..tostring(ifstats.isView)..'\n')
    end
    if expand_interface_views and ifstats.isView then
-        -- expand rrds for views and read each physical interface separately
-        for iface,_ in pairs(ifstats.interfaces) do
-            if(debug_metric) then io.write('iface: '..iface..' id: '..getInterfaceId(iface)..'\n') end
-            rrd_if_ids[#rrd_if_ids+1] = getInterfaceId(iface)
-        end       
+	-- expand rrds for views and read each physical interface separately
+	for iface,_ in pairs(ifstats.interfaces) do
+	    if(debug_metric) then io.write('iface: '..iface..' id: '..getInterfaceId(iface)..'\n') end
+	    rrd_if_ids[#rrd_if_ids+1] = getInterfaceId(iface)
+	end
    end
-   
-   
+
+
    if(debug_metric) then io.write("RRD File: "..rrdFile.."\n") end
 
    if(rrdFile == "all") then
@@ -1531,45 +1687,45 @@ function rrd2json(ifid, host, rrdFile, start_time, end_time, rickshaw_json, expa
        local dirs = ntop.getDirs()
        local p = dirs.workingdir .. "/" .. ifid .. "/rrd/"
        if(debug_metric) then io.write("Navigating: "..p.."\n") end
-       
+
        if(host ~= nil) then
-           p = p .. getPathFromKey(host)
-           go_deep = true
+	   p = p .. getPathFromKey(host)
+	   go_deep = true
        else
-           go_deep = false
+	   go_deep = false
        end
-       
+
        d = fixPath(p)
        rrds = navigatedir("", "*", d, d, go_deep, false, ifid, host, start_time, end_time)
-       
+
        local traffic_array = {}
        for key, value in pairs(rrds) do
-           rsp = singlerrd2json(ifid, host, value, start_time, end_time, rickshaw_json, expand_interface_views)
-           if(rsp.totalval ~= nil) then total = rsp.totalval else total = 0 end
-           
-           if(total > 0) then
-               traffic_array[total] = rsp
-               if(debug_metric) then io.write("Analyzing: "..value.." [total "..total.."]\n") end
-           end
+	   rsp = singlerrd2json(ifid, host, value, start_time, end_time, rickshaw_json, expand_interface_views)
+	   if(rsp.totalval ~= nil) then total = rsp.totalval else total = 0 end
+
+	   if(total > 0) then
+	       traffic_array[total] = rsp
+	       if(debug_metric) then io.write("Analyzing: "..value.." [total "..total.."]\n") end
+	   end
        end
-       
+
        for key, value in pairsByKeys(traffic_array, rev) do
-           ret[#ret+1] = value
-           if(ret[#ret].json ~= nil) then
-               if(debug_metric) then io.write(key.."\n") end
-               num = num + 1
-               if(num >= 10) then break end
-           end
+	   ret[#ret+1] = value
+	   if(ret[#ret].json ~= nil) then
+	       if(debug_metric) then io.write(key.."\n") end
+	       num = num + 1
+	       if(num >= 10) then break end
+	   end
        end
    else
        num = 0
        for _,iface in pairs(rrd_if_ids) do
-           if(debug_metric) then io.write('iface: '..iface..'\n') end
-            for i,rrd in pairs(split(rrdFile, ",")) do
-                if(debug_metric) then io.write("["..i.."] "..rrd..' iface: '..iface.."\n") end
-                ret[#ret + 1] = singlerrd2json(iface, host, rrd, start_time, end_time, rickshaw_json, expand_interface_views)
-                if(ret[#ret].json ~= nil) then num = num + 1 end
-            end
+	   if(debug_metric) then io.write('iface: '..iface..'\n') end
+	    for i,rrd in pairs(split(rrdFile, ",")) do
+		if(debug_metric) then io.write("["..i.."] "..rrd..' iface: '..iface.."\n") end
+		ret[#ret + 1] = singlerrd2json(iface, host, rrd, start_time, end_time, rickshaw_json, expand_interface_views)
+		if(ret[#ret].json ~= nil) then num = num + 1 end
+	    end
        end
 
    end
@@ -1585,13 +1741,13 @@ function rrd2json(ifid, host, rrdFile, start_time, end_time, rickshaw_json, expa
    -- jsons for single interfaces, and not for the view. Since view statistics
    -- are in ret[1], it suffices to aggregate jsons from index i >= 2
    if expand_interface_views and ifstats.isView then
-       i = 2 
+       i = 2
    end
    local json = "["
    local first = true  -- used to decide where to append commas
    while i <= num do
       if(debug_metric) then io.write("->"..i.."\n") end
-      if not first then json = json.."," end     
+      if not first then json = json.."," end
       json = json..ret[i].json
       i = i + 1
       first = false
