@@ -1,5 +1,5 @@
 --
--- (C) 2014-15 - ntop.org
+-- (C) 2014-16 - ntop.org
 --
 require "lua_trace"
 
@@ -8,16 +8,15 @@ require "lua_trace"
 
 function aggregateInterfaceStats(ifstats)
    if(ifstats == nil) then return(ifstats) end
-   
+
    local tot = {}
 
    for ifname,_v in pairs(ifstats["interfaces"]) do
-
       for k,v in pairs(ifstats["interfaces"][ifname]) do
-	 if(type(v) ~= "table") then	    
+	 if(type(v) ~= "table") then
 	    if((tot[k] == nil) and (k ~= "id") and (k ~= "name")) then
 	       --io.write(k.."\n")
-	       tot[k] = v 
+	       tot[k] = v
 	    end
 	 end
       end
@@ -33,20 +32,42 @@ function aggregateInterfaceStats(ifstats)
 
       keys = { "profiles" }
       for _,key in pairs(keys) do
-      	 if(_v[key] ~= nil) then
+	 if(_v[key] ~= nil) then
 	   tot[key] = {}
-  	   for k,v in pairs(_v[key]) do
+	   for k,v in pairs(_v[key]) do
 	      --io.write(k.."\n")
 	      if(tot[key][k] == nil) then tot[key][k] = 0 end
 	      tot[key][k] = tot[key][k] + v
- 	   end
+	   end
 	 end
       end
 
       keys = { "pktSizeDistribution" }
       for _,key in pairs(keys) do
 	 if(tot[key] == nil) then tot[key] = { } end
-	 
+
+	 for k,v in pairs(_v[key]) do
+	    --io.write(k.."\n")
+	    if(tot[key][k] == nil) then tot[key][k] = 0 end
+	    tot[key][k] = tot[key][k] + v
+	 end
+      end
+
+      keys = { "tcpFlowStats" }
+      for _,key in pairs(keys) do
+	 if(tot[key] == nil) then tot[key] = { } end
+
+	 for k,v in pairs(_v[key]) do
+	    --io.write(k.."\n")
+	    if(tot[key][k] == nil) then tot[key][k] = 0 end
+	    tot[key][k] = tot[key][k] + v
+	 end
+      end
+
+      keys = { "tcpPacketStats" }
+      for _,key in pairs(keys) do
+	 if(tot[key] == nil) then tot[key] = { } end
+
 	 for k,v in pairs(_v[key]) do
 	    --io.write(k.."\n")
 	    if(tot[key][k] == nil) then tot[key][k] = 0 end
@@ -56,17 +77,20 @@ function aggregateInterfaceStats(ifstats)
 
       keys = { "localstats", "ndpi" }
       for _,key in pairs(keys) do
+	 if(tot[key] == nil) then tot[key] = { } end
 	 -- io.write(key.."\n")
 
-	 tot[key] = { }
 	 for k,v in pairs(_v[key]) do
-	    if(tot[key][k] == nil) then tot[key][k] = { } end	 
+	    if(tot[key][k] == nil) then tot[key][k] = { } end
 	    for k1,v1 in pairs(_v[key][k]) do
-	       --io.write(k1.."="..type(v1).."\n")
+	       -- io.write(k1.."="..type(v1).."\n")
 
 	       if(type(v1) == "number") then
-		  if(tot[key][k][k1] == nil) then tot[key][k][k1] = 0 end	    
+		  if(tot[key][k][k1] == nil) then
+		     tot[key][k][k1] = 0
+		  end
 		  tot[key][k][k1] = tot[key][k][k1] + v1
+		  -- io.write("tot["..key.."]["..k.."]["..k1.."]="..tot[key][k][k1].."\n")
 	       else
 		  tot[key][k][k1] = v1
 	       end
@@ -74,19 +98,88 @@ function aggregateInterfaceStats(ifstats)
 	 end
       end
    end
-   
+
    for k,v in pairs(tot) do
       ifstats[k] = v
    end
-   
+
    return(ifstats)
 end
 
+-- ##########################################
+
+function aggregateGroupStats(groupStats)
+   if(groupStats == nil) then return(groupStats) end
+
+   local tot = 0
+   local res = { }
+   for ifname,_v in pairs(groupStats) do
+      for k,v in pairs(_v["groups"]) do
+	 if k == nil or v == nil or type(v) ~= 'table' then
+	    goto continue
+	 end
+	 if res[k] == nil then
+	    res[k] = {}
+	 end
+	 for prop_name, prop_value in pairs(v) do
+	    if res[k][prop_name] == nil then
+	       res[k][prop_name] = prop_value
+	    elseif type(prop_value) == 'number' then
+	       res[k][prop_name] = res[k][prop_name] + prop_value
+	    end
+	 end
+	 ::continue::
+      end
+      tot = tot + _v["numGroupedHosts"]
+   end
+
+   return res,tot
+end
+
+-- ##########################################
+
+function aggregateHostsStats(hostStats)
+   if(hostStats == nil) then return(hostStats) end
+
+   local tot = 0
+   local res = { }
+   for ifname,_v in pairs(hostStats) do
+      for k,v in pairs(_v["hosts"]) do
+	    --io.write(k.."\n")
+	    res[k] = v
+      end
+
+      tot = tot + _v["numHosts"]
+   end
+
+   return res,tot
+end
+
+-- ##########################################
+
+function aggregateFlowsStats(flowstats)
+   -- TODO: prevent possible flow overlap when using interface views
+   if(flowstats == nil) then return(flowstats) end
+
+   local tot = 0
+   local res = { }
+   for ifname,_v in pairs(flowstats) do
+      for k,v in ipairs(_v["flows"]) do
+	    --io.write(k.."\n")
+	    res[k] = v
+      end
+
+      tot = tot + _v["numFlows"]
+   end
+
+   return res,tot
+end
+
 -- ##############################################
 
 function getInterfaceName(interface_id)
    local ifnames = interface.getIfNames()
-   
+
    interface_id = tonumber(interface_id)
    for _,if_name in pairs(ifnames) do
       --io.write(if_name.."\n")
@@ -94,10 +187,10 @@ function getInterfaceName(interface_id)
       _ifstats = interface.getStats()
       _ifstats = aggregateInterfaceStats(_ifstats)
       if(_ifstats.id == interface_id) then
-	 return(_ifstats.name) 
+	 return(_ifstats.name)
       end
    end
-   
+
    return("")
 end
 
@@ -132,10 +225,10 @@ end
 
 --print("(((("..ifname.."))))")
 l4_keys = {
-  { "TCP", "tcp" },
-  { "UDP", "udp" },
-  { "ICMP", "icmp" },
-  { "Other IP", "other ip" }
+  { "TCP", "tcp", 6 },
+  { "UDP", "udp", 17 },
+  { "ICMP", "icmp", 1 },
+  { "Other IP", "other ip", -1 }
 }
 
 function __FILE__() return debug.getinfo(2,'S').source end
@@ -143,8 +236,8 @@ function __LINE__() return debug.getinfo(2, 'l').currentline end
 
 -- ##############################################
 
-function sendHTTPHeaderIfName(mime, ifname, maxage)
-  info = ntop.getInfo()
+function sendHTTPHeaderIfName(mime, ifname, maxage, content_disposition)
+  info = ntop.getInfo(false)
 
   print('HTTP/1.1 200 OK\r\n')
   print('Cache-Control: max-age=0, no-cache, no-store\r\n')
@@ -152,23 +245,24 @@ function sendHTTPHeaderIfName(mime, ifname, maxage)
   print('Pragma: no-cache\r\n')
   print('X-Frame-Options: DENY\r\n')
   print('X-Content-Type-Options: nosniff\r\n')
-  if(_SESSION ~= nil) then print('Set-Cookie: session='.._SESSION["session"]..'; max-age=' .. maxage .. '; path=/; HttpOnly\r\n') end  
+  if(_SESSION ~= nil) then print('Set-Cookie: session='.._SESSION["session"]..'; max-age=' .. maxage .. '; path=/; HttpOnly\r\n') end
   if(ifname ~= nil) then print('Set-Cookie: ifname=' .. ifname .. '; path=/\r\n') end
   print('Content-Type: '.. mime ..'\r\n')
+  if(content_disposition ~= nil) then print('Content-Disposition: '..content_disposition..'\r\n') end
   print('Last-Modified: '..os.date("!%a, %m %B %Y %X %Z").."\r\n")
   print('\r\n')
 end
 
 -- ##############################################
 
-function sendHTTPHeaderLogout(mime)
-  sendHTTPHeaderIfName(mime, nil, 0)
+function sendHTTPHeaderLogout(mime, content_disposition)
+  sendHTTPHeaderIfName(mime, nil, 0, content_disposition)
 end
 
 -- ##############################################
 
-function sendHTTPHeader(mime)
-  sendHTTPHeaderIfName(mime, nil, 3600)
+function sendHTTPHeader(mime, content_disposition)
+  sendHTTPHeaderIfName(mime, nil, 3600, content_disposition)
 end
 
 -- ##############################################
@@ -192,35 +286,25 @@ end
 
 -- ##############################################
 
+-- Simplified checker
+function isIPv6String(ip)
+  if(string.find(ip, ":") ~= nil) then
+     return true
+  end
+
+    return false
+end
+
+-- ##############################################
+
 function findString(str, tofind)
-  local upper_lower = true
   if(str == nil) then return(nil) end
   if(tofind == nil) then return(nil) end
 
-  str1    = string.gsub(str, "-", "_")
-  tofind1 = string.gsub(tofind, "-", "_")
-  rsp     = string.find(str1, tofind1, 1)
+  str1    = string.lower(string.gsub(str, "-", "_"))
+  tofind1 = string.lower(string.gsub(tofind, "-", "_"))
 
-  if(upper_lower) then
-    if(rsp == nil) then
-      -- Lowercase
-      str1 = string.lower(str1)
-      tofind1 = string.lower(tofind1)
-      rsp = string.find(str1, tofind1, 1)
-    end
-
-    if(rsp == nil) then
-      -- Uppercase
-      str1 = string.upper(str1)
-      tofind1 = string.upper(tofind1)
-      rsp = string.find(str1, tofind1, 1)
-    end
-  end
-  --print(str1 .. "/" .. tofind1.."\n")
-  --print(rsp)
-  --print("\n")
-
-  return(rsp)
+  return(string.find(str1, tofind1, 1))
 end
 
 -- ##############################################
@@ -245,24 +329,34 @@ end
 -- ##############################################
 
 function string.contains(String,Start)
-  return(string.find(String,Start,1) ~= nil)
+   if type(String) ~= 'string' or type(Start) ~= 'string' then
+      return false
+   end
+   return(string.find(String,Start,1) ~= nil)
 end
 
 -- ##############################################
 
 function string.starts(String,Start)
-  return string.sub(String,1,string.len(Start))==Start
+   if type(String) ~= 'string' or type(Start) ~= 'string' then
+      return false
+   end
+   return string.sub(String,1,string.len(Start))==Start
 end
 
 -- ##############################################
 
 function string.ends(String,End)
-  return End=='' or string.sub(String,-string.len(End))==End
+   if type(String) ~= 'string' or type(End) ~= 'string' then
+      return false
+   end
+   return End=='' or string.sub(String,-string.len(End))==End
 end
 
 -- ##############################################
 
 function printASN(asn, asname)
+  asname = asname:gsub('"','')
   if(asn > 0) then
     return("<A HREF='http://as.robtex.com/as"..asn..".html' title='"..asname.."'>"..asname.."</A> <i class='fa fa-external-link fa-lg'></i>")
   else
@@ -304,12 +398,12 @@ function shortHostName(name)
       ret = ""
 
       for token in string.gmatch(name, "([%w-]+).") do
-        if(tot < max_len) then
-          if(n > 0) then ret = ret .. "." end
-          ret = ret .. token
-          tot = tot+string.len(token)
-          n = n + 1
-        end
+	if(tot < max_len) then
+	  if(n > 0) then ret = ret .. "." end
+	  ret = ret .. token
+	  tot = tot+string.len(token)
+	  n = n + 1
+	end
       end
 
       return(ret .. "...")
@@ -357,7 +451,11 @@ alert_type_keys = {
   { "<i class='fa fa-frown-o'></i> Blacklist Host",  3 },
   { "<i class='fa fa-clock-o'></i> Periodic Activity",  4 },
   { "<i class='fa fa-sort-asc'></i> Quota Exceeded",  5 },
-  { "<i class='fa fa-ban'></i> Malware Detected",  6 }
+  { "<i class='fa fa-ban'></i> Malware Detected",  6 },
+  { "<i class='fa fa-bomb'></i> Ongoing Attacker",  7 },
+  { "<i class='fa fa-bomb'></i> Under Attack",  8 },
+  { "<i class='fa fa-exclamation'></i> Misconfigured App",  9 },
+  { "<i class='fa fa-exclamation'></i> Suspicious Activity",  10 },
 }
 
 function alertSeverityLabel(v)
@@ -413,9 +511,8 @@ end
 --   print(_key .. "=" .. _value .. "\n")
 --end
 
-function round(num, idp)
-  return tonumber(string.format("%." .. (idp or 0) .. "f", num))
-end
+function round(num, idp) return tonumber(string.format("%." .. (idp or 0) .. "f", num)) end
+--function round(num) return math.floor(num+.5) end
 
 -- Note that the function below returns a string as returnong a number
 -- would not help as a new float would be returned
@@ -510,7 +607,7 @@ function isnumber(str)
       return(false)
    end
 end
- 
+
 function split(pString, pPattern)
   local Table = {}  -- NOTE: use {n = 0} in Lua-5.0
   local fpat = "(.-)" .. pPattern
@@ -536,16 +633,16 @@ function tmax(t)
     local argmx, mx = nil, nil
     if (type(t) ~= "table") then return nil, nil end
     for k, v in pairs(t) do
-        -- first iteration
-        if mx == nil and argmx == nil then
-            mx = v
-            argmx = k
-        elseif (v == mx and k > argmx) or v > mx then
-        -- if there is a tie, prefer the greatest argument
-        -- otherwise grab the maximum
-            argmx = k
-            mx = v
-        end
+	-- first iteration
+	if mx == nil and argmx == nil then
+	    mx = v
+	    argmx = k
+	elseif (v == mx and k > argmx) or v > mx then
+	-- if there is a tie, prefer the greatest argument
+	-- otherwise grab the maximum
+	    argmx = k
+	    mx = v
+	end
     end
     return argmx, mx
 end
@@ -556,16 +653,16 @@ function tmin(t)
     local argmn, mn = nil, nil
     if (type(t) ~= "table") then return nil, nil end
     for k, v in pairs(t) do
-        -- first iteration
-        if mn == nil and argmn == nil then
-            mn = v
-            argmn = k
-        elseif (v == mn and k > argmn) or v < mn then
-        -- if there is a tie, prefer the greatest argument
-        -- otherwise grab the minimum
-            argmn = k
-            mn = v
-        end
+	-- first iteration
+	if mn == nil and argmn == nil then
+	    mn = v
+	    argmn = k
+	elseif (v == mn and k > argmn) or v < mn then
+	-- if there is a tie, prefer the greatest argument
+	-- otherwise grab the minimum
+	    argmn = k
+	    mn = v
+	end
     end
     return argmn, mn
 end
@@ -585,9 +682,9 @@ string.split = function(s, p)
       index = next_index
     else
       if index > 0 and index <= last_index then
-        table.insert(temp, string.sub(s, index, last_index))
+	table.insert(temp, string.sub(s, index, last_index))
       elseif index == 0 then
-        temp = nil
+	temp = nil
       end
       break
     end
@@ -654,6 +751,18 @@ function secondsToTime(seconds)
   return msg
 end
 
+function msToTime(ms)
+  if(ms > 1000) then
+    return secondsToTime(ms/1000)
+  else
+    if(ms < 1) then
+      return("< 1 ms")
+    else
+      return(round(ms, 4).." ms")
+    end
+  end
+end
+
 function starts(String,Start)
   return string.sub(String,1,string.len(Start))==Start
 end
@@ -662,21 +771,101 @@ function ends(String,End)
   return End=='' or string.sub(String,-string.len(End))==End
 end
 
+-- #################################################################
+
+-- NOTE keep in sync with Flashstart::initMapping()
+
+host_categories =	 {
+      ["freetime"] = "FreeTime",
+      ["chat"] = "Chat",
+      ["onlineauctions"] = "Auctions",
+      ["onlinegames"] = "Online Games",
+      ["pets"] = "Animals",
+      ["porn"] = "Porn",
+      ["religion"] = "Religion",
+      ["phishing"] = "Phishing",
+      ["sexuality"] = "Sex",
+      ["games"] = "Games",
+      ["socialnetworking"] = "SocialNetwork",
+      ["jobsearch"] = "JobSearch",
+      ["mail"] = "Webmail",
+      ["news"] = "News",
+      ["proxy"] = "AnonymousProxy",
+      ["publicite"] = "Advertisement",
+      ["sports"] = "Sport",
+      ["vacation"] = "Travel",
+      ["ecommerce"] = "E-commerce",
+      ["instantmessaging"] = "InstantMessaging",
+      ["kidstimewasting"] = "KidGames",
+      ["audio-video"] = "AudioVideo",
+      ["books"] = "Books",
+      ["government"] = "Gouvernment",
+      ["malware"] = "Malware",
+      ["medical"] = "Medicine",
+      ["ann"] = "Ads",
+      ["drugs"] = "Drugs",
+      ["dating"] = "OnlineDating",
+      ["desktopsillies"] = "DesktopImages",
+      ["filehosting"] = "FileHosting",
+      ["filesharing"] = "FileSharing",
+      ["gambling"] = "Gambling",
+      ["warez"] = "CracksWarez",
+      ["radio"] = "Radio",
+      ["updatesites"] = "Updates",
+      ["financial"] = "FinanceBanking",
+      ["adult"] = "Adults",
+      ["fashion"] = "Fashion",
+      ["showbiz"] = "Showbiz",
+      ["ict"] = "ICT",
+      ["company"] = "Business",
+      ["education"] = "EducationSchool",
+      ["searchengines"] = "SearchEngines",
+      ["blog"] = "Blog",
+      ["association"] = "Associations",
+      ["music"] = "Musica",
+      ["legal"] = "Legal",
+      ["photo"] = "Photo",
+      ["stats"] = "Webstat",
+      ["content"] = "ContentServer",
+      ["domainforsale"] = "DomainForSale",
+      ["weapons"] = "Guns",
+      ["generic"] = "Generic"
+}
 
 -- #################################################################
 
-function getCategoryIcon(what, cat)
-   if((cat == "") or (cat == nil)) then
+function getCategoryLabel(cat)
+   if((cat == "") or (cat == nil) or (cat == "???")) then
       return("")
-   elseif(cat == "safe") then
-   return("<A HREF=http://google.com/safebrowsing/diagnostic?site="..what.."&hl=en-us><font color=green><i class=\'fa fa-check\'></i></font></A>")
-   elseif(cat == "malware") then
-      return("<A HREF=http://google.com/safebrowsing/diagnostic?site="..what.."&hl=en-us><font color=red><i class=\'fa fa-ban\'></i></font></A>")
-   else
-      return(cat)
    end
+
+  for c,v in pairs(host_categories) do
+   if(c == cat) then
+     return(v)
+   end
+  end
+
+  return(cat)
 end
 
+function getCategoryIcon(what, cat)
+   if((cat == "") or (cat == nil) or (cat == "???")) then
+      return("")
+   end
+
+  ret = ""
+  for c,_ in pairs(cat) do
+   if(host_categories[c] ~= nil) then
+     ret = ret .. " <span class='label label-info'>"..host_categories[c].."</span>"
+   else
+     ret = ret .. " <span class='label label-info'>"..c.."</span>"
+   end
+  end
+
+  return(ret)
+end
+
+
 function abbreviateString(str, len)
   if(str == nil) then
     return("")
@@ -706,8 +895,8 @@ function clearbit(x, p)
   return hasbit(x, p) and x - p or x
 end
 
-function isBroadMulticast(ip)   
-   if(ip == "0.0.0.0") then 
+function isBroadMulticast(ip)
+   if(ip == "0.0.0.0") then
       return true
    end
    -- print(ip)
@@ -716,33 +905,49 @@ function isBroadMulticast(ip)
    if(t == nil) then
       return false  -- Might be an IPv6 address
    else
-      if(tonumber(t[1]) >= 224)  then 
+      if(tonumber(t[1]) >= 224)  then
 	 return true
       end
    end
-   
+
    return false
 end
 
 function isBroadcastMulticast(ip)
    -- check NoIP
-   if(ip == "0.0.0.0") then 
+   if(ip == "0.0.0.0") then
       return true
    end
-   
+
    -- check IPv6
    t = string.split(ip, "%.")
-   
+
    if(t ~= nil) then
       -- check Multicast / Broadcast
-      if(tonumber(t[1]) >= 224) then 
+      if(tonumber(t[1]) >= 224) then
 	 return true
       end
    end
-   
+
    return false
 end
 
+function isIPv4(ip)
+  if string.find(ip, "%.") then
+    return true
+  end
+  return false
+end
+
+
+function isIPv6(ip)
+   if string.find(ip, "%.") then
+     return false
+  end
+  return true
+end
+
+
 function addGauge(name, url, maxValue, width, height)
   if(url ~= nil) then print('<A HREF="'..url..'">') end
   print('<canvas id="'..name..'" height="'..height..'" width="'..width..'"></canvas>\n')
@@ -797,7 +1002,7 @@ end
 
 -- Return the first 'howmany' hosts
 function getTopInterfaceHosts(howmany, localHostsOnly)
-  hosts_stats = interface.getHostsInfo()
+  hosts_stats,total = aggregateHostsStats(interface.getHostsInfo())
   ret = {}
   sortTable = {}
   n = 0
@@ -846,6 +1051,7 @@ function purifyInterfaceName(interface_name)
   -- io.write(debug.traceback().."\n")
   interface_name = string.gsub(interface_name, "@", "_")
   interface_name = string.gsub(interface_name, ":", "_")
+  interface_name = string.gsub(interface_name, "/", "_")
   return(interface_name)
 end
 
@@ -857,7 +1063,7 @@ function fixPath(path)
       path = string.sub(path, 1, 2) .. string.gsub(string.sub(path, 3), ":", "_")
       -- io.write("->"..path.."\n")
    end
-   
+
   return(path)
 end
 
@@ -942,7 +1148,7 @@ resolved_host_labels_cache = {}
 
 function getHostAltName(host_ip)
    local alt_name = resolved_host_labels_cache[host_ip]
-   
+
    if(alt_name ~= nil) then
       return(alt_name)
    end
@@ -954,7 +1160,7 @@ function getHostAltName(host_ip)
    end
 
    resolved_host_labels_cache[host_ip] = alt_name
-   
+
    return(alt_name)
 end
 
@@ -970,19 +1176,19 @@ function host2name(name, vlan)
    vlan = tonumber(vlan or "0")
 
    name = getHostAltName(name)
-   
+
    if(name == orig_name) then
       rname = ntop.getResolvedAddress(name)
-      
+
       if((rname ~= nil) and (rname ~= "")) then
 	 name = rname
       end
    end
- 
+
    if(vlan > 0) then
       name = name .. '@' .. vlan
    end
-   
+
    return name
 end
 
@@ -991,8 +1197,13 @@ function flowinfo2hostname(flow_info, host_type, vlan)
    local name
    local orig_name
 
+   if(host_type == "srv") then
+      if(flow_info["host_server_name"] ~= nil) then return(flow_info["host_server_name"]) end
+      if(flow_info["protos.ssl.certificate"] ~= nil)  then return(flow_info["protos.ssl.certificate"]) end
+   end
+
    name = flow_info[host_type..".host"]
-   
+
    if((name == "") or (name == nil)) then
       name = flow_info[host_type..".ip"]
    end
@@ -1055,7 +1266,7 @@ function hostinfo2hostkey(host_info,host_type,show_vlan)
     end
   end
 
-  if(((host_info["vlan"] ~= nil) and (host_info["vlan"] ~= 0)) 
+  if(((host_info["vlan"] ~= nil) and (host_info["vlan"] ~= 0))
      or ((show_vlan ~= nil) and show_vlan))  then
     rsp = rsp..'@'..tostring(host_info["vlan"])
   end
@@ -1070,7 +1281,7 @@ end
 --
 function url2hostinfo(get_info)
   local host = {}
-  
+
   -- Catch when the host key is using as host url parameter
   if((get_info["host"] ~= nil) and (string.find(get_info["host"],"@"))) then
     get_info = hostkey2hostinfo(get_info["host"])
@@ -1223,7 +1434,7 @@ function hostinfo2jqueryid(host_info,host_type)
 
 
   if((host_info["vlan"] ~= nil) and (host_info["vlan"] ~= 0)) then
-    rsp = rsp..'_'..tostring(host_info["vlan"])
+    rsp = rsp..'@'..tostring(host_info["vlan"])
   end
 
   rsp = string.gsub(rsp, "%.", "__")
@@ -1244,7 +1455,7 @@ function version2int(v)
     major = e[1]
     minor = e[2]
     veryminor = e[3]
-    
+
     if(major == nil or tonumber(major) == nil or type(major) ~= "string")     then major = 0 end
     if(minor == nil or tonumber(minor) == nil or type(minor) ~= "string")     then minor = 0 end
     if(veryminor == nil or tonumber(veryminor) == nil or type(veryminor) ~= "string") then veryminor = 0 end
@@ -1268,7 +1479,7 @@ function ntop_version_check()
    end
 
    if(rsp ~= nil) then
-      info = ntop.getInfo()
+      info = ntop.getInfo(false)
       new_version = version2int(rsp)
 
       version_elems  = split(info["version"], " ");
@@ -1282,7 +1493,7 @@ end
 
 
 -- Print contents of `tbl`, with indentation.
--- You can call it as tprint(mytable) 
+-- You can call it as tprint(mytable)
 -- The other two parameters should not be set
 function tprint(s, l, i)
    l = (l) or 1000; i = i or "";-- default item limit, indent string
@@ -1292,12 +1503,12 @@ function tprint(s, l, i)
    io.write(i..' '..ts..'\n');
    for k,v in pairs(s) do
       local indent = ""
-      
+
       if(i ~= "") then
 	 indent = i .. "."
       end
       indent = indent .. tostring(k)
-      
+
       l = tprint(v, l, indent);
       if (l < 0) then break end
    end
@@ -1317,7 +1528,7 @@ function table.len(table)
  local count = 0
 
   if(table == nil) then return(0) end
-  for k,v in pairs(table) do 
+  for k,v in pairs(table) do
     count = count + 1
   end
 
@@ -1353,79 +1564,6 @@ end
 -----  End of Redis Utils  ------
 
 
--- ############################################
--- Runtime preference
-
-function prefsInputField(label, comment, key, value)
-  if(_GET[key] ~= nil) then
-    k = "ntopng.prefs."..key
-    v_s = _GET[key]
-    v = tonumber(v_s)
-    if(v ~= nil and (v > 0) and (v < 86400)) then
-      -- print(k.."="..v)
-      ntop.setCache(k, tostring(v))
-      value = v
-    elseif (v_s ~= nil) then
-      ntop.setCache(k, v_s)
-      value = v_s
-    end
-  end
-
-  print('<tr><td width=50%><strong>'..label..'</strong><p><small>'..comment..'</small></td>')
-
-  print [[
-	   <td class="input-group col-lg-3" align=right><form class="navbar-form navbar-right">]]
-print('<input id="csrf" name="csrf" type="hidden" value="'..ntop.getRandomCSRFValue()..'" />\n')
-print [[
- <div class="input-group" >
-      <div >
-        <input type="text" class="form-control" name="]] print(key) print [[" value="]] print(value.."") print [[">
-      <span class="input-group-btn">
-        <button class="btn btn-default" type="submit">Save</button>
-      </span>
-      </div>
-    </div><!-- /input-group -->
-</form></td></tr>
-]]
-
-end
-
-function toggleTableButton(label, comment, on_label, on_value, on_color , off_label, off_value, off_color, submit_field, redis_key, disabled)
-  if(_GET[submit_field] ~= nil) then
-    ntop.setCache(redis_key, _GET[submit_field])
-    value = _GET[submit_field]
-  else
-    value = ntop.getCache(redis_key)
-  end
-  if (disabled == true) then
-    disabled = 'disabled = ""'
-  else
-    disabled = ""
-  end
-
-  -- Read it anyway to
-  if(value == off_value) then
-    rev_value  = on_value
-    on_active  = "btn-default"
-    off_active = "btn-"..off_color.." active"
-  else
-    rev_value  = off_value
-    on_active  = "btn-"..on_color.." active"
-    off_active = "btn-default"
-  end
-
-  if(label ~= "") then print('<tr><td width=50%><strong>'..label..'</strong><p><small>'..comment..'</small></td><td align=right>\n') end
-  print('<form>\n<div class="btn-group btn-toggle">')
-  print('<input id="csrf" name="csrf" type="hidden" value="'..ntop.getRandomCSRFValue()..'" />\n')
-  print('<input type=hidden name='..submit_field..' value='..rev_value..'>\n')
-  print('<button type="submit" '..disabled..' class="btn btn-sm  '..on_active..'">'..on_label..'</button>')
-  print('<button '..disabled..' class="btn btn-sm '..off_active..'">'..off_label..'</button></div>\n')
-  print('</form>\n')
-  if(label ~= "") then print('</td></tr>') end
-
-  return(value)
-end
-
 function isPausedInterface(current_ifname)
   state = ntop.getCache("ntopng.prefs."..current_ifname.."_not_idle")
   if(state == "0") then return true else return false end
@@ -1448,6 +1586,16 @@ function isLoopback(name)
   end
 end
 
+function isLocalPacketdumpEnabled()
+   local nbox_integration = ntop.getCache("ntopng.prefs.nbox_integration")
+   if nbox_integration == nil or nbox_integration ~= "1" then
+      nbox_integration = false
+   else
+      nbox_integration = true
+   end
+   return isAdministrator() and not nbox_integration and not interface.isView() and interface.isPacketInterface()
+end
+
 function processColor(proc)
   if(proc == nil) then
     return("")
@@ -1463,9 +1611,9 @@ end
  -- Table preferences
 
 function getDefaultTableSort(table_type)
-  table_key = getRedisPrefix("ntopng.prefs.table")
+   table_key = getRedisPrefix("ntopng.prefs.table")
   if(table_type ~= nil) then
-    value = ntop.getHashCache(table_key, "sort_"..table_type)
+     value = ntop.getHashCache(table_key, "sort_"..table_type)
   end
   if((value == nil) or (value == "")) then value = 'column_' end
   return(value)
@@ -1501,6 +1649,7 @@ end
 
 
 function getInterfaceNameAlias(interface_name)
+   -- io.write(debug.traceback().."\n")
    label = ntop.getCache('ntopng.prefs.'..interface_name..'.name')
    if((label == nil) or (label == "")) then
       return(interface_name)
@@ -1512,13 +1661,13 @@ end
 function getHumanReadableInterfaceName(interface_name)
    key = 'ntopng.prefs.'..interface_name..'.name'
    custom_name = ntop.getCache(key)
-   
+
    if((custom_name ~= nil) and (custom_name ~= "")) then
       return(custom_name)
    else
       interface.select(interface_name)
       _ifstats = aggregateInterfaceStats(interface.getStats())
-      
+
       -- print(interface_name.."=".._ifstats.name)
       return(_ifstats.name)
    end
@@ -1547,7 +1696,7 @@ end
 -- ##############################################
 
 function harvestUnusedDir(path, min_epoch)
-   local files = ntop.readdir(path)   
+   local files = ntop.readdir(path)
 
    -- print("Reading "..path.."<br>\n")
 
@@ -1567,7 +1716,7 @@ function harvestUnusedDir(path, min_epoch)
    end
 end
 
- -- ############################################## 
+ -- ##############################################
 
 function harvestJSONTopTalkers(days)
    local when = os.time() - 86400 * days
@@ -1584,19 +1733,19 @@ function harvestJSONTopTalkers(days)
    end
 end
 
- -- ############################################## 
+ -- ##############################################
 
 function isAdministrator()
    local user_group = ntop.getUserGroup()
-   
-   if(user_group == "administrator") then 
+
+   if(user_group == "administrator") then
       return(true)
    else
       return(false)
    end
 end
 
- -- ############################################## 
+ -- ##############################################
 
 function haveAdminPrivileges()
    if(isAdministrator) then
@@ -1609,7 +1758,7 @@ function haveAdminPrivileges()
    end
 end
 
- -- ############################################## 
+ -- ##############################################
 
 function getKeysSortedByValue(tbl, sortFunction)
   local keys = {}
@@ -1630,7 +1779,7 @@ function getKeys(t, col)
   return keys
 end
 
- -- ##############################################                                                                                                                                                  
+ -- ##############################################
 
 function formatBreed(breed)
    if(breed == "Safe") then
@@ -1638,7 +1787,7 @@ function formatBreed(breed)
    elseif(breed == "Acceptable") then
       return("<i class='fa fa-thumbs-o-up' alt='Acceptable Protocol'></i>")
    elseif(breed == "Fun") then
-      return("<i class='fa fa-smile' alt='Fun Protocol'></i>")
+      return("<i class='fa fa-smile-o' alt='Fun Protocol'></i>")
    elseif(breed == "Unsafe") then
       return("<i class='fa fa-thumbs-o-down'></i>")
    elseif(breed == "Dangerous") then
@@ -1718,11 +1867,11 @@ end
 function maxRateToString(max_rate)
    if((max_rate == nil) or (max_rate == "")) then max_rate = -1 end
    max_rate = tonumber(max_rate)
-   if(max_rate == -1) then 
-      return("No Limit") 
+   if(max_rate == -1) then
+      return("No Limit")
    else
-      if(max_rate == 0) then 
-	 return("Drop All Traffic") 
+      if(max_rate == 0) then
+	 return("Drop All Traffic")
       else
 	 if(max_rate < 1000) then
 	    return(max_rate.." Kbit/s")
@@ -1770,7 +1919,7 @@ end
 
 local mac_cache = { }
 -- get_mac_classification
-function get_mac_classification(m)   
+function get_mac_classification(m)
    local path = fixPath(dirs.installdir.."/httpdocs/other/EtherOUI.txt")
    local file_mac
 
@@ -1790,11 +1939,11 @@ function get_mac_classification(m)
 	 b = string.sub(mac_line, 1, 8)
 	 if (m == b) then
 	   t = split(mac_line, "\t")
-           file_mac.close()
+	   file_mac.close()
 	   rsp = split(t[2], " ")[1]
 	   mac_cache[m] = rsp
-           return rsp
-        end
+	   return rsp
+	end
       end
       mac_line = file_mac:read("*l")
    end
@@ -1831,9 +1980,9 @@ function get_symbolic_mac(mac_address)
 
       if(magic_short_macs[m] ~= nil) then
 	 return(magic_short_macs[m].."_"..t.." ("..mac_address..")")
-      else	 
+      else
 	 local s = get_mac_classification(m)
-	 
+
 	 if(m == s) then
 	    return(get_mac_classification(m)..":"..t)
 	 else
@@ -1856,7 +2005,7 @@ function getservbyport(port_num, proto)
    if(proto == nil) then proto = "TCP" end
 
    port_num = tonumber(port_num)
-   
+
    proto = string.lower(proto)
 
    -- io.write(port_num.."@"..proto.."\n")
@@ -1865,7 +2014,7 @@ end
 
 -- getSpeedMax
 function getSpeedMax(ifname)
-   
+
    if(ifname == nil) then
       return -1
    end
@@ -1903,11 +2052,194 @@ end
 -- ###############################################
 
 function tolongint(what)
-   return(string.format("%u", what))
+   if(what == nil) then
+      return(0)
+   else
+      return(string.format("%u", what))
+   end
 end
 
 -- ###############################################
 
 function trimSpace(what)
+   if(what == nil) then return("") end
    return(string.gsub(what, "%s+", ""))
 end
+
+-- ###############################################
+
+function formatWebSite(site)
+   return("<A target=\"_blank\" HREF=http://"..site..">"..site.."</A> <i class=\"fa fa-external-link\"></i></th>")
+end
+
+-- Update Utils::flowstatus2str
+function getFlowStatus(status)
+  if(status == 0) then return("<font color=green>Normal</font>")
+  elseif(status == 1) then return("<font color=orange>Slow TCP Connection</font>")
+  elseif(status == 2) then return("<font color=orange>Slow Application Header</font>")
+  elseif(status == 3) then return("<font color=orange>Slow Data Exchange (Slowloris?)</font>")
+  elseif(status == 4) then return("<font color=orange>Low Goodput</font>")
+  elseif(status == 5) then return("<font color=orange>Suspicious TCP SYN Probing (or server port down)</font>")
+  elseif(status == 6) then return("<font color=orange>TCP Connection Reset</font>")
+  elseif(status == 7) then return("<font color=orange>Suspicious TCP Probing</font>")
+  else return("<font color=orange>Unknown status ("..status..")</font>")
+  end
+end
+
+-- prints purged information for hosts / flows
+function purgedErrorString()
+    return 'Very likely it is expired and ntopng has purged it from memory. You can set purge idle timeout settings from the <A HREF="'..ntop.getHttpPrefix()..'/lua/admin/prefs.lua?subpage_active=data_purge"><i class="fa fa-flask"></i> Preferences</A>.'
+end
+
+-- print TCP flags
+function printTCPFlags(flags)
+      if(hasbit(flags,0x01)) then print('<span class="label label-info">FIN</span> ') end
+      if(hasbit(flags,0x02)) then print('<span class="label label-info">SYN</span> ')  end
+      if(hasbit(flags,0x04)) then print('<span class="label label-danger">RST</span> ') end
+      if(hasbit(flags,0x08)) then print('<span class="label label-info">PUSH</span> ') end
+      if(hasbit(flags,0x10)) then print('<span class="label label-info">ACK</span> ')  end
+      if(hasbit(flags,0x20)) then print('<span class="label label-info">URG</span> ')  end
+end
+
+-- ##########################################
+
+function historicalProtoHostHref(ifId, host, l4_proto, ndpi_proto_id, info)
+   if ntop.isPro() and ntop.getPrefs().is_dump_flows_to_mysql_enabled == true then
+      local hist_url = ntop.getHttpPrefix().."/lua/pro/db_explorer.lua?search=true&ifId="..ifId
+      local now    = os.time()
+      local ago1h  = now - 3600
+
+      hist_url = hist_url.."&epoch_end="..tostring(now)
+      if((host ~= nil) and (host ~= "")) then hist_url = hist_url.."&"..hostinfo2url(host) end
+      if((l4_proto ~= nil) and (l4_proto ~= "")) then
+	 hist_url = hist_url.."&l4proto="..l4_proto
+      end
+      if((ndpi_proto_id ~= nil) and (ndpi_proto_id ~= "")) then hist_url = hist_url.."&protocol="..ndpi_proto_id end
+      if((info ~= nil) and (info ~= "")) then hist_url = hist_url.."&info="..info end
+      print('&nbsp;')
+      -- print('<span class="label label-info">')
+      print('<a href="'..hist_url..'&epoch_begin='..tostring(ago1h)..'" title="Flows seen in the last hour"><i class="fa fa-history fa-lg"></i></a>')
+      -- print('</span>')
+   end
+end
+
+-- ##########################################
+
+_icmp_types = {
+	 { 0, 0, "Echo Reply" },
+	 { 3, 0, "Network Unreachable" },
+	 { 3, 1, "Host Unreachable" },
+	 { 3, 2, "Protocol Unreachable" },
+	 { 3, 3, "Port Unreachable" },
+	 { 3, 4, "Fragmentation needed but no fragment bit set" },
+	 { 3, 5, "Source routing failed" },
+	 { 3, 6, "Destination network unknown" },
+	 { 3, 7, "Destination host unknown" },
+	 { 3, 8, "Source host isolated (obsolete)" },
+	 { 3, 9, "Destination network administratively prohibited" },
+	 { 3, 10, "Destination host administratively prohibited" },
+	 { 3, 11, "Network unreachable for TOS" },
+	 { 3, 12, "Host unreachable for TOS" },
+	 { 3, 13, "Communication administratively prohibited by filtering" },
+	 { 3, 14, "Host precedence violation" },
+	 { 3, 15, "Precedence cutoff in effect" },
+	 { 4, 0, "Source quench" },
+	 { 5, 0, "Redirect for network" },
+	 { 5, 1, "Redirect for host" },
+	 { 5, 2, "Redirect for TOS and network" },
+	 { 5, 3, "Redirect for TOS and host" },
+	 { 8, 0, "Echo request x" },
+	 { 9, 0, "Router advertisement" },
+	 { 10, 0, "Route solicitation" },
+	 { 11, 0, "TTL equals 0 during transit" },
+	 { 11, 1, "TTL equals 0 during reassembly" },
+	 { 12, 0, "IP header bad (catchall error)" },
+	 { 12, 1, "Required options missing" },
+	 { 13, 0, "Timestamp request (obsolete)" },
+	 { 14, 0, "Timestamp reply (obsolete)" },
+	 { 15, 0, "Information request (obsolete)" },
+	 { 16, 0, "Information reply (obsolete)" },
+	 { 17, 0, "Address mask request" },
+	 { 18, 0, "Address mask reply" }
+}
+
+-- Code is currently ignored on IVMPv6
+_icmpv6_types = {
+        { 0, "Reserved" },
+	{ 1, "Destination Unreachable" },
+	{ 2, "Packet Too Big" },
+	{ 3, "Time Exceeded" },
+	{ 4, "Parameter Problem" },
+	{ 100, "Private experimentation" },
+	{ 101, "Private experimentation" },
+	-- { 102-126, "Unassigned" },
+	{ 127, "Reserved for expansion of ICMPv6 error messages" },
+	{ 128, "Echo Request" },
+	{ 129, "Echo Reply" },
+	{ 130, "Multicast Listener Query" },
+	{ 131, "Multicast Listener Report" },
+	{ 132, "Multicast Listener Done" },
+	{ 133, "Router Solicitation" },
+	{ 134, "Router Advertisement" },
+	{ 135, "Neighbor Solicitation" },
+	{ 136, "Neighbor Advertisement" },
+	{ 137, "Redirect Message" },
+	{ 138, "Router Renumbering" },
+	{ 139, "ICMP Node Information Query" },
+	{ 140, "ICMP Node Information Response" },
+	{ 141, "Inverse Neighbor Discovery Solicitation Message" },
+	{ 142, "Inverse Neighbor Discovery Advertisement Message" },
+	{ 143, "Version 2 Multicast Listener Report" },
+	{ 144, "Home Agent Address Discovery Request Message" },
+	{ 145, "Home Agent Address Discovery Reply Message" },
+	{ 146, "Mobile Prefix Solicitation" },
+	{ 147, "Mobile Prefix Advertisement" },
+	{ 148, "Certification Path Solicitation Message" },
+	{ 149, "Certification Path Advertisement Message" },
+	{ 150, "ICMP messages utilized by experimental mobility protocols" },
+	{ 151, "Multicast Router Advertisement" },
+	{ 152, "Multicast Router Solicitation" },
+	{ 153, "Multicast Router Termination" },
+	{ 154, "FMIPv6 Messages" },
+	{ 155, "RPL Control Message" },
+	{ 156, "ILNPv6 Locator Update Message" },
+	{ 157, "Duplicate Address Request" },
+	{ 158, "Duplicate Address Confirmation" },
+	{ 159, "MPL Control Message" },
+	-- { 160-199, "Unassigned" },
+	{ 200, "Private experimentation" },
+	{ 201, "Private experimentation" },
+	{ 255, "Reserved for expansion of ICMPv6 informational messages" }
+}
+
+-- #############################################
+
+function getICMPV6TypeCode(icmp)
+  local t = icmp.type
+  local c = icmp.code
+
+  for _, _e in ipairs(_icmpv6_types) do
+    if(_e[1] == t) then
+    	return(_e[2])
+    end
+  end
+
+ return(t.."/"..c)
+end
+
+-- #############################################
+
+function getICMPTypeCode(icmp)
+  local t = icmp.type
+  local c = icmp.code
+
+  for _, _e in ipairs(_icmp_types) do
+    if((_e[1] == t) and (_e[2] == c)) then
+    	return(_e[3])
+    end
+  end
+
+ return(getICMPV6TypeCode(icmp))
+end
+
+
