@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -13,7 +13,9 @@ sendHTTPHeader('text/html; charset=iso-8859-1')
 
 function dumpInterfaceStats(interface_name)
    interface.select(interface_name)
-   ifstats = aggregateInterfaceStats(interface.getStats())
+
+   ifstats = interface.getStats()
+   ifstats = aggregateInterfaceStats(ifstats)
 
    stats = interface.getFlowsStats()
 
@@ -25,10 +27,12 @@ function dumpInterfaceStats(interface_name)
       hosts_pctg = math.floor(1+((ifstats.hosts*100)/prefs.max_num_hosts))
       flows_pctg = math.floor(1+((ifstats.flows*100)/prefs.max_num_flows))
 
-      print('\t{ "ifname": "'.. interface_name..'", "packets": '.. ifstats.packets .. ', "bytes": ' .. ifstats.bytes .. ', "drops": ' .. ifstats.drops .. ', "alerts": '.. ntop.getNumQueuedAlerts() ..', "num_flows": '.. ifstats.flows .. ', "num_hosts": ' .. ifstats.hosts .. ', "epoch": ' .. os.time()..' , "uptime": " ' .. secondsToTime(uptime) .. '", "hosts_pctg": ' .. hosts_pctg .. ', "flows_pctg": ' .. flows_pctg)
+      print('\t{ "ifname": "'.. interface_name..'", "packets": '.. ifstats.packets .. ', "bytes": ' .. ifstats.bytes .. ', "drops": ' .. ifstats.drops .. ', "alerts": '.. ntop.getNumQueuedAlerts() ..', "num_flows": '.. ifstats.flows .. ', "num_hosts": ' .. ifstats.hosts .. ', "epoch": ' .. os.time()..' , "uptime": " ' .. secondsToTime(uptime) .. '", "hosts_pctg": ' .. hosts_pctg .. ', "flows_pctg": ' .. flows_pctg .. ', "remote_pps": ' .. ifstats.remote_pps ..', "remote_bps": ' .. ifstats.remote_bps)
       print(', "is_view": '..tostring(ifstats.isView))
       print(', "local2remote": '.. ifstats["localstats"]["bytes"]["local2remote"]..', "remote2local": '..ifstats["localstats"]["bytes"]["remote2local"])
 
+      print(', "tcpPacketStats": { "retransmissions": '..tostring(ifstats.tcpPacketStats.retransmissions)..', "out_of_order": '..tostring(ifstats.tcpPacketStats.out_of_order)..', "lost":'..tostring(ifstats.tcpPacketStats.lost)..' }')
+
       if(ifstats["bridge.device_a"] ~= nil) then
 	 print(', "a_to_b_in_pkts": '.. ifstats["bridge.a_to_b.in_pkts"])
 	 print(', "a_to_b_in_bytes": '.. ifstats["bridge.a_to_b.in_bytes"])
@@ -94,4 +98,4 @@ if(_GET["ifname"] == "all") then
    print("\n]\n")
 else
    dumpInterfaceStats(ifname)
-end
\ No newline at end of file
+end
