@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -111,7 +111,7 @@ end
 -----------------------------------
 
 mode = _GET["mode"] -- memory(actual-memory),bytes,latency
-type = _GET["type"] -- user,process(proc_name)
+type = _GET["distr"] -- user,process(proc_name)
 host = _GET["host"]
 filter = _GET["filter"] -- all,client,server
 
@@ -122,7 +122,7 @@ if(host == nil) then
    print("<div class=\"alert alert-danger\"><img src=".. ntop.getHttpPrefix() .. "/img/warning.png> This flow cannot be found (expired ?)</div>")
 else
 
-  flows_stats = interface.getFlowsInfo()
+  flows_stats,total = aggregateFlowsStats(interface.getFlowsInfo())
 
   -- Default values
   aggregated_flows = {}
