@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -10,7 +10,7 @@ require "lua_utils"
 sendHTTPHeader('application/json')
 
 interface.select(ifname)
-flows_stats = interface.getFlowsInfo()
+flows_stats,total = aggregateFlowsStats(interface.getFlowsInfo())
 
 links = {}
 hosts = {}
