@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -15,8 +15,9 @@ active_page = "hosts"
 dofile(dirs.installdir .. "/scripts/lua/inc/menu.lua")
 
 interface.select(ifname)
-hosts_stats = interface.getHostsInfo()
+hosts_stats,total = aggregateHostsStats(interface.getLocalHostsInfo())
 
+--io.write(ifname.."/"..total.."\n")
 max_num = 25
 localhosts = {}
 found = false
