@@ -1,5 +1,5 @@
 --
--- (C) 2013-15 - ntop.org
+-- (C) 2013-16 - ntop.org
 --
 
 dirs = ntop.getDirs()
@@ -15,7 +15,7 @@ user = _GET["user"]
 
 
 interface.select(ifname)
-flows = interface.getFlowsInfo()
+flows_stats,total = aggregateFlowsStats(interface.getFlowsInfo())
 -- flows = interface.findUserFlows(user)
 
 local debug = false
