@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -57,6 +57,13 @@ Host::~Host() {
   if(num_uses > 0)
     ntop->getTrace()->traceEvent(TRACE_WARNING, "Internal error: num_uses=%u", num_uses);
 
+  if(topSitesKey) {
+    char oldk[64];
+
+    snprintf(oldk, sizeof(oldk), "%s.old", topSitesKey);
+    ntop->getRedis()->rename(topSitesKey, oldk);
+  }
+
   if(ip && (!ip->isEmpty())) dumpStats(false);
 
   // ntop->getTrace()->traceEvent(TRACE_NORMAL, "Deleting %s (%s)", k, localHost ? "local": "remote");
@@ -86,11 +93,12 @@ Host::~Host() {
   if(country)        free(country);
   if(city)           free(city);
   if(asname)         free(asname);
-
+  if(categoryStats)  delete categoryStats;
   if(syn_flood_attacker_alert) delete syn_flood_attacker_alert;
   if(syn_flood_victim_alert)   delete syn_flood_victim_alert;
   if(ip) delete ip;
   if(m) delete m;
+  if(topSitesKey) free(topSitesKey);
 }
 
 /* *************************************** */
@@ -124,16 +132,17 @@ void Host::computeHostSerial() {
 void Host::initialize(u_int8_t mac[6], u_int16_t _vlanId, bool init_all) {
   char key[64], redis_key[128], *k;
   u_char* ant_mac =  iface->getAntennaMac();
+  char buf[64], host[96];
 
 #ifdef NTOPNG_PRO
   sent_to_sketch = rcvd_from_sketch = NULL;
 #endif
 
   if(ant_mac)
-    memcpy(antenna_mac_address, ant_mac, 6); 
+    memcpy(antenna_mac_address, ant_mac, 6);
   else
     memset(antenna_mac_address, 0, sizeof(antenna_mac_address));
-    
+
   if(mac) memcpy(mac_address, mac, 6); else memset(mac_address, 0, 6);
 
   //if(_vlanId > 0) ntop->getTrace()->traceEvent(TRACE_NORMAL, "VLAN => %d", _vlanId);
@@ -142,16 +151,17 @@ void Host::initialize(u_int8_t mac[6], u_int16_t _vlanId, bool init_all) {
 
   max_new_flows_sec_threshold = CONST_MAX_NEW_FLOWS_SECOND;
   max_num_syn_sec_threshold = CONST_MAX_NUM_SYN_PER_SECOND;
-  max_num_active_flows = CONST_MAX_NUM_HOST_ACTIVE_FLOWS;
-  networkStats = NULL, local_network_id = -1;
+  max_num_active_flows = CONST_MAX_NUM_HOST_ACTIVE_FLOWS, good_low_flow_detected = false;
+  networkStats = NULL, local_network_id = -1, nextResolveAttempt = 0;
   syn_flood_attacker_alert = new AlertCounter(max_num_syn_sec_threshold, CONST_MAX_THRESHOLD_CROSS_DURATION);
   syn_flood_victim_alert = new AlertCounter(max_num_syn_sec_threshold, CONST_MAX_THRESHOLD_CROSS_DURATION);
-  category[0] = '\0', os[0] = '\0', httpbl[0] = '\0', blacklisted_host = false;
+  os[0] = '\0', trafficCategory[0] = '\0', blacklisted_host = false;
   num_uses = 0, symbolic_name = NULL, vlan_id = _vlanId,
     ingress_shaper_id = egress_shaper_id = DEFAULT_SHAPER_ID,
     total_num_flows_as_client = total_num_flows_as_server = 0,
     num_active_flows_as_client = num_active_flows_as_server = 0;
   first_seen = last_seen = iface->getTimeLastPktRcvd();
+  nextSitesUpdate = 0;
   if((m = new(std::nothrow) Mutex()) == NULL)
     ntop->getTrace()->traceEvent(TRACE_WARNING, "Internal error: NULL mutex. Are you running out of memory?");
 
@@ -160,7 +170,7 @@ void Host::initialize(u_int8_t mac[6], u_int16_t _vlanId, bool init_all) {
   longitude = 0, latitude = 0, host_quota_mb = 0;
   k = get_string_key(key, sizeof(key));
   snprintf(redis_key, sizeof(redis_key), "%s.%d.json", k, vlan_id);
-  dns = NULL, http = NULL;
+  dns = NULL, http = NULL, categoryStats = NULL, topSitesKey = NULL;
 
 #ifdef NTOPNG_PRO
   l7Policy = NULL;
@@ -168,49 +178,66 @@ void Host::initialize(u_int8_t mac[6], u_int16_t _vlanId, bool init_all) {
 
   if(init_all) {
     if(ip) {
-      char buf[64], host[96], json[8192];
+      char sitesBuf[64], *strIP = ip->print(buf, sizeof(buf));
 
-      snprintf(host, sizeof(host), "%s@%u", ip->print(buf, sizeof(buf)), vlan_id);
+      snprintf(host, sizeof(host), "%s@%u", strIP, vlan_id);
 
       updateLocal();
       updateHostTrafficPolicy(host);
       systemHost = ip->isLocalInterfaceAddress();
 
+      if(localHost) {
+	char oldk[64];
+
+	snprintf(sitesBuf, sizeof(sitesBuf), "sites.%s", strIP);
+	topSitesKey = strdup(sitesBuf);
+
+	snprintf(oldk, sizeof(oldk), "%s.old", topSitesKey);
+	ntop->getRedis()->rename(topSitesKey, oldk);
+      }
+
       // ntop->getTrace()->traceEvent(TRACE_NORMAL, "Loading %s (%s)", k, localHost ? "local": "remote");
 
       if(localHost || systemHost) {
 	dns = new DnsStats();
-	http = new HTTPStats(iface->get_hosts_hash());
+	http = new HTTPstats(iface->get_hosts_hash());
       }
 
-      if(((localHost || systemHost)
-	  && ntop->getPrefs()->is_host_persistency_enabled())
-	 && (!ntop->getRedis()->get(redis_key, json, sizeof(json)))) {
-	/* Found saved copy of the host so let's start from the previous state */
-        // ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s => %s", redis_key, json);
-	ntop->getTrace()->traceEvent(TRACE_INFO, "Deserializing %s", redis_key);
-	deserialize(json, redis_key);
+      if((localHost || systemHost)
+	 && ntop->getPrefs()->is_host_persistency_enabled()){
+	char *json;
+	if((json = (char*)malloc(HOST_MAX_SERIALIZED_LEN * sizeof(char))) == NULL)
+	  ntop->getTrace()->traceEvent(TRACE_ERROR,
+				       "Unable to allocate memory to deserialize %s", redis_key);
+	else if (!ntop->getRedis()->get(redis_key, json, HOST_MAX_SERIALIZED_LEN)){
+	  /* Found saved copy of the host so let's start from the previous state */
+	  // ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s => %s", redis_key, json);
+	  ntop->getTrace()->traceEvent(TRACE_INFO, "Deserializing %s", redis_key);
+	  deserialize(json, redis_key);
+	}
+	if(json) free(json);
       }
 
       if(localHost || systemHost
 	 || ntop->getPrefs()->is_dns_resolution_enabled_for_all_hosts()) {
 	char rsp[256];
 
-	if(ntop->getRedis()->getAddress(host, rsp, sizeof(rsp), true) == 0) {
-	  if(symbolic_name) free(symbolic_name);
-	  symbolic_name = strdup(rsp);
-	}
+	if(ntop->getRedis()->getAddress(host, rsp, sizeof(rsp), true) == 0)
+	  setName(rsp);
 	// else ntop->getRedis()->pushHostToResolve(host, false, localHost);
       }
 
-      if((!localHost || systemHost) && ntop->getPrefs()->is_httpbl_enabled() && ip->isIPv4()) {
+      if((!localHost || systemHost)
+	 && ntop->getPrefs()->is_httpbl_enabled()
+	 && ip->isIPv4()) {
 	// http:bl only works for IPv4 addresses
-	if(ntop->getRedis()->getAddressHTTPBL(host, iface, httpbl, sizeof(httpbl), true) == 0) {
-          if(strcmp(httpbl, NULL_BL)) {
+	if(ntop->getRedis()->getAddressTrafficFiltering(host, iface, trafficCategory,
+							sizeof(trafficCategory), true) == 0) {
+          if(strcmp(trafficCategory, NULL_BL)) {
 	    blacklisted_host = true;
 	    ntop->getTrace()->traceEvent(TRACE_INFO,
 					 "Blacklisted host found %s [%s]",
-					 host, httpbl);
+					 host, trafficCategory);
 	  }
 	}
       }
@@ -228,8 +255,7 @@ void Host::initialize(u_int8_t mac[6], u_int16_t _vlanId, bool init_all) {
 	       mac_address[0], mac_address[1], mac_address[2],
 	       mac_address[3], mac_address[4], mac_address[5]);
 
-      if(symbolic_name) free(symbolic_name);
-      symbolic_name = strdup(buf);
+      setName(buf);
       localHost = true;
     }
 
@@ -362,7 +388,7 @@ void Host::updateLocal() {
   if(ip)
     localHost = ip->isLocalHost(&local_network_id);
 
-  if(local_network_id > 0)
+  if(local_network_id >= 0)
     networkStats = getNetworkStats(local_network_id);
 
   if(0) {
@@ -391,10 +417,10 @@ void Host::set_mac(char *m) {
   u_int32_t mac[6] = { 0 };
 
   if(!strcmp(m, "00:00:00:00:00:00")) return;
-  
+
   sscanf(m, "%02X:%02X:%02X:%02X:%02X:%02X",
 	 &mac[0], &mac[1], &mac[2], &mac[3], &mac[4], &mac[5]);
-  
+
   mac_address[0] = mac[0], mac_address[1] = mac[1],
     mac_address[2] = mac[2], mac_address[3] = mac[3],
     mac_address[4] = mac[4], mac_address[5] = mac[5];
@@ -402,36 +428,72 @@ void Host::set_mac(char *m) {
 
 /* *************************************** */
 
+void Host::getSites(lua_State* vm, char *k, const char *label) {
+  int rc;
+  char **sites;
+
+  lua_newtable(vm);
+
+  if((rc = ntop->getRedis()->zRevRange(k, &sites)) > 0) {
+
+    for(int i = 0; i < rc; i++) {
+      if((sites[i] == NULL) || (sites[i+1] == NULL))
+	continue; /* safety check */
+
+      lua_push_int_table_entry(vm, sites[i], atoi(sites[i+1]));
+
+      free(sites[i]), free(sites[i+1]);
+      i++;
+    }
+
+    free(sites);
+  }
+
+  lua_pushstring(vm, label);
+  lua_insert(vm, -2);
+  lua_settable(vm, -3);
+}
+
+/* *************************************** */
+
 void Host::lua(lua_State* vm, patricia_tree_t *ptree,
-	       bool host_details, bool verbose, bool returnHost) {
+	       bool host_details, bool verbose,
+	       bool returnHost, bool asListElement,
+	       bool exclude_deserialized_bytes) {
   char buf[64];
   char buf_id[64];
   char *ipaddr = NULL;
 
-  if(!match(ptree)) return;
+  if(ptree && (!match(ptree)))
+    return;
 
   lua_newtable(vm);
   if(ip) {
     char ip_buf[64];
 
     lua_push_str_table_entry(vm, "ip", (ipaddr = ip->print(ip_buf, sizeof(ip_buf))));
-  } else
+    lua_push_int_table_entry(vm, "ipkey", ip->key());
+  } else {
     lua_push_nil_table_entry(vm, "ip");
-  
+    lua_push_nil_table_entry(vm, "ipkey");
+  }
+
   if((antenna_mac_address[0] != 0)
      && (antenna_mac_address[1] != 0)
      && (antenna_mac_address[2] != 0)
      && (antenna_mac_address[3] != 0)
      && (antenna_mac_address[4] != 0)
-     && (antenna_mac_address[5] != 0))  
+     && (antenna_mac_address[5] != 0))
     lua_push_str_table_entry(vm, "antenna_mac", get_mac(buf, sizeof(buf), antenna_mac_address));
 
   lua_push_str_table_entry(vm, "mac", get_mac(buf, sizeof(buf), mac_address));
   lua_push_int_table_entry(vm, "vlan", vlan_id);
   lua_push_bool_table_entry(vm, "localhost", localHost);
 
-  lua_push_int_table_entry(vm, "bytes.sent", sent.getNumBytes());
-  lua_push_int_table_entry(vm, "bytes.rcvd", rcvd.getNumBytes());
+  lua_push_int_table_entry(vm, "bytes.sent",
+			   sent.getNumBytes() - (exclude_deserialized_bytes ? sent.getNumDeserializedBytes() : 0));
+  lua_push_int_table_entry(vm, "bytes.rcvd",
+			   rcvd.getNumBytes() - (exclude_deserialized_bytes ? rcvd.getNumDeserializedBytes() : 0));
 
   if(ip) {
     char *local_net;
@@ -451,7 +513,7 @@ void Host::lua(lua_State* vm, patricia_tree_t *ptree,
     }
 
     lua_push_str_table_entry(vm, "name",
-			     host_details ? get_name(buf, sizeof(buf), false) :  symbolic_name);
+			     get_name(buf, sizeof(buf), false));
     lua_push_int32_table_entry(vm, "local_network_id", local_network_id);
 
     local_net = ntop->getLocalNetworkName(local_network_id);
@@ -524,6 +586,19 @@ void Host::lua(lua_State* vm, patricia_tree_t *ptree,
 	lua_push_int_table_entry(vm, "bridge.egress_shaper_id", egress_shaper_id);
 	lua_push_int_table_entry(vm, "bridge.host_quota_mb", host_quota_mb);
       }
+
+      lua_push_int_table_entry(vm, "low_goodput_flows.as_client", low_goodput_client_flows);
+      lua_push_int_table_entry(vm, "low_goodput_flows.as_server", low_goodput_server_flows);
+
+
+      if(topSitesKey) {
+	char oldk[64];
+
+	snprintf(oldk, sizeof(oldk), "%s.old", topSitesKey);
+
+	getSites(vm, topSitesKey, "sites");
+	getSites(vm, oldk, "sites.old");
+      }
     }
 
     lua_push_int_table_entry(vm, "seen.first", first_seen);
@@ -541,11 +616,11 @@ void Host::lua(lua_State* vm, patricia_tree_t *ptree,
     lua_push_int_table_entry(vm, "throughput_trend_pps", pkts_thpt_trend);
 
     // ntop->getTrace()->traceEvent(TRACE_NORMAL, "[pkts_thpt: %.2f] [pkts_thpt_trend: %d]", pkts_thpt,pkts_thpt_trend);
-    lua_push_int_table_entry(vm, "num_alerts", getNumAlerts());
+    lua_push_int_table_entry(vm, "num_alerts", triggerAlerts() ? getNumAlerts() : 0);
 
     if(ip) {
-      if(ntop->get_categorization()) lua_push_str_table_entry(vm, "category", get_category());
-      if(ntop->getPrefs()->is_httpbl_enabled()) lua_push_str_table_entry(vm, "httpbl", get_httpbl());
+      if(ntop->getPrefs()->is_httpbl_enabled())
+	lua_push_str_table_entry(vm, "httpbl", get_httpbl());
     }
 
     lua_push_bool_table_entry(vm, "dump_host_traffic", dump_host_traffic);
@@ -553,6 +628,7 @@ void Host::lua(lua_State* vm, patricia_tree_t *ptree,
     if(verbose) {
       char *rsp = serialize();
 
+      if(categoryStats) categoryStats->lua(vm);
       if(ndpiStats) ndpiStats->lua(iface, vm);
       lua_push_str_table_entry(vm, "json", rsp);
       free(rsp);
@@ -587,16 +663,11 @@ void Host::lua(lua_State* vm, patricia_tree_t *ptree,
 
     if(!returnHost) {
       /* Use the ip@vlan_id as a key only in case of multi vlan_id, otherwise use only the ip as a key */
-
       if(vlan_id == 0) {
         sprintf(buf_id, "%s",(ip != NULL) ? ip->print(buf, sizeof(buf)) : get_mac(buf, sizeof(buf), mac_address));
       } else {
         sprintf(buf_id, "%s@%d",(ip != NULL) ? ip->print(buf, sizeof(buf)) : get_mac(buf, sizeof(buf), mac_address), vlan_id );
       }
-
-      lua_pushstring(vm,buf_id);
-      lua_insert(vm, -2);
-      lua_settable(vm, -3);
     }
   } else {
     /* Use the ip@vlan_id as a key only in case of multi vlan_id, otherwise use only the ip as a key */
@@ -605,6 +676,9 @@ void Host::lua(lua_State* vm, patricia_tree_t *ptree,
     } else {
       sprintf(buf_id, "%s@%d",(ip != NULL) ? ip->print(buf, sizeof(buf)) : get_mac(buf, sizeof(buf), mac_address),vlan_id );
     }
+  }
+
+  if(asListElement) {
     lua_pushstring(vm, buf_id);
     lua_insert(vm, -2);
     lua_settable(vm, -3);
@@ -617,47 +691,28 @@ void Host::lua(lua_State* vm, patricia_tree_t *ptree,
   As this method can be called from Lua, in order to avoid concurency issues
   we need to lock/unlock
 */
-void Host::setName(char *name, bool update_categorization) {
+void Host::setName(char *name) {
   if(m) m->lock(__FILE__, __LINE__);
   if((symbolic_name == NULL) || (symbolic_name && strcmp(symbolic_name, name))) {
     if(symbolic_name) free(symbolic_name);
     symbolic_name = strdup(name);
   }
+
   if(m) m->unlock(__FILE__, __LINE__);
-
-  refreshCategory();
-}
-
-/* ***************************************** */
-
-void Host::refreshCategory() {
-  char buf[128] =  { 0 };
-  char* ip_addr = ip->print(buf, sizeof(buf));
-
-  if((symbolic_name != NULL)
-     && strcmp(ip_addr, symbolic_name)
-     && (category[0] == '\0')
-     && ip
-     && ip->isIPv4()
-     && (!ip->isMulticastAddress())
-     && (!ip->isBroadcastAddress())
-     && ntop->get_categorization()
-     ) {
-    ntop->get_categorization()->findCategory(symbolic_name, category, sizeof(category), false);
-  }
-
-  if(category[0] == '\0')
-    snprintf(category, sizeof(category), "%s", CATEGORIZATION_SAFE_SITE);
 }
 
 /* ***************************************** */
 
 void Host::refreshHTTPBL() {
-  if(ip && ip->isIPv4() && (!localHost) && (httpbl[0] == '\0') && ntop->getPrefs()->is_httpbl_enabled()) {
+  if(ip
+     && ip->isIPv4()
+     && (!localHost)
+     && (trafficCategory[0] == '\0')
+     && ntop->get_httpbl()) {
     char buf[128] =  { 0 };
     char* ip_addr = ip->print(buf, sizeof(buf));
 
-    ntop->get_httpbl()->findHTTPBL(ip_addr, httpbl, sizeof(httpbl), false);
+    ntop->get_httpbl()->findCategory(ip_addr, trafficCategory, sizeof(trafficCategory), false);
   }
 }
 
@@ -669,6 +724,13 @@ char* Host::get_name(char *buf, u_int buf_len, bool force_resolution_if_not_foun
   } else {
     char *addr, redis_buf[64];
     int rc;
+    time_t now = time(NULL);
+
+    if(nextResolveAttempt
+       && ((nextResolveAttempt > now) || (nextResolveAttempt == (time_t)-1))) {
+      return(symbolic_name);
+    } else
+      nextResolveAttempt = ntop->getPrefs()->is_dns_resolution_enabled() ? now + MIN_HOST_RESOLUTION_FREQUENCY : (time_t)-1;
 
     addr = ip->print(buf, buf_len);
 
@@ -679,12 +741,9 @@ char* Host::get_name(char *buf, u_int buf_len, bool force_resolution_if_not_foun
 				      force_resolution_if_not_found);
 
     if(rc == 0)
-      setName(redis_buf, false);
-    else {
-      if(symbolic_name != NULL) free(symbolic_name);
-
-      symbolic_name = strdup(addr);
-    }
+      setName(redis_buf);
+    else
+      setName(addr);
 
     return(symbolic_name);
   }
@@ -743,12 +802,13 @@ u_int32_t Host::key() {
 /* *************************************** */
 
 void Host::incStats(u_int8_t l4_proto, u_int ndpi_proto,
-		    u_int64_t sent_packets, u_int64_t sent_bytes,
-		    u_int64_t rcvd_packets, u_int64_t rcvd_bytes) {
+		    struct site_categories *category,
+		    u_int64_t sent_packets, u_int64_t sent_bytes, u_int64_t sent_goodput_bytes,
+		    u_int64_t rcvd_packets, u_int64_t rcvd_bytes, u_int64_t rcvd_goodput_bytes) {
 
   if(sent_packets || rcvd_packets) {
-    ((GenericHost*)this)->incStats(l4_proto, ndpi_proto, sent_packets,
-				   sent_bytes, rcvd_packets, rcvd_bytes);
+    ((GenericHost*)this)->incStats(l4_proto, ndpi_proto, sent_packets, sent_bytes, sent_goodput_bytes,
+				   rcvd_packets, rcvd_bytes, rcvd_goodput_bytes);
 
     if(sent_packets == 1) sent_stats.incStats((u_int)sent_bytes);
     if(rcvd_packets == 1) recv_stats.incStats((u_int)rcvd_bytes);
@@ -774,6 +834,20 @@ void Host::incStats(u_int8_t l4_proto, u_int ndpi_proto,
 	other_ip_sent.incStats(sent_packets, sent_bytes);
       break;
     }
+
+    if(category && localHost && ntop->get_flashstart()) {
+      if(categoryStats == NULL)
+	categoryStats = new CategoryStats();
+
+      if(categoryStats) {
+	for(int i=0; i <MAX_NUM_CATEGORIES; i++)
+	  if(category->categories[i] == NTOP_UNKNOWN_CATEGORY_ID)
+	    break;
+	  else
+	    categoryStats->incStats(category->categories[i],
+				    sent_bytes+rcvd_bytes);
+      }
+    }
   }
 }
 
@@ -789,20 +863,31 @@ char* Host::get_string_key(char *buf, u_int buf_len) {
 /* *************************************** */
 
 char* Host::serialize() {
+  json_object *my_object = getJSONObject();
+  char *rsp = strdup(json_object_to_json_string(my_object));
+
+  /* Free memory */
+  json_object_put(my_object);
+
+  return(rsp);
+}
+
+/* *************************************** */
+
+json_object* Host::getJSONObject() {
   json_object *my_object;
-  char *rsp, buf[32];
+  char buf[32];
 
   if((my_object = json_object_new_object()) == NULL) return(NULL);
 
   json_object_object_add(my_object, "mac_address", json_object_new_string(get_mac(buf, sizeof(buf), mac_address)));
-
+  json_object_object_add(my_object, "seen.last", json_object_new_int(last_seen));
   json_object_object_add(my_object, "asn", json_object_new_int(asn));
   if(symbolic_name)       json_object_object_add(my_object, "symbolic_name", json_object_new_string(symbolic_name));
   if(country)             json_object_object_add(my_object, "country",   json_object_new_string(country));
   if(city)                json_object_object_add(my_object, "city",      json_object_new_string(city));
   if(asname)              json_object_object_add(my_object, "asname",    json_object_new_string(asname));
-  if(category[0] != '\0') json_object_object_add(my_object, "category",  json_object_new_string(category));
-  if(httpbl[0] != '\0')   json_object_object_add(my_object, "httpbl",    json_object_new_string(httpbl));
+  if(trafficCategory[0] != '\0')   json_object_object_add(my_object, "trafficCategory",    json_object_new_string(trafficCategory));
   if(vlan_id != 0)        json_object_object_add(my_object, "vlan_id",   json_object_new_int(vlan_id));
   if(latitude)            json_object_object_add(my_object, "latitude",  json_object_new_double(latitude));
   if(longitude)           json_object_object_add(my_object, "longitude", json_object_new_double(longitude));
@@ -835,16 +920,11 @@ char* Host::serialize() {
   /* The value below is handled by reading dumps on disk as otherwise the string will be too long */
   //json_object_object_add(my_object, "activityStats", activityStats.getJSONObject());
 
+  if(categoryStats)  json_object_object_add(my_object, "categories", categoryStats->getJSONObject());
   if(dns)  json_object_object_add(my_object, "dns", dns->getJSONObject());
   if(http) json_object_object_add(my_object, "http", http->getJSONObject());
 
-  //ntop->getTrace()->traceEvent(TRACE_WARNING, "%s()", __FUNCTION__);
-  rsp = strdup(json_object_to_json_string(my_object));
-
-  /* Free memory */
-  json_object_put(my_object);
-
-  return(rsp);
+  return(my_object);
 }
 
 /* *************************************** */
@@ -881,9 +961,13 @@ bool Host::addIfMatching(lua_State* vm, patricia_tree_t *ptree, char *key) {
 
 bool Host::deserialize(char *json_str, char *key) {
   json_object *o, *obj;
+  enum json_tokener_error jerr = json_tokener_success;
 
-  if((o = json_tokener_parse(json_str)) == NULL) {
-    ntop->getTrace()->traceEvent(TRACE_WARNING, "JSON Parse error [key: %s]: %s", key, json_str);
+  if((o = json_tokener_parse_verbose(json_str, &jerr)) == NULL) {
+    ntop->getTrace()->traceEvent(TRACE_WARNING, "JSON Parse error [%s] key: %s: %s",
+				 json_tokener_error_desc(jerr),
+				 key,
+				 json_str);
     return(false);
   }
 
@@ -895,8 +979,7 @@ bool Host::deserialize(char *json_str, char *key) {
   if(json_object_object_get_ex(o, "country", &obj))        { if(country) free(country); country = strdup(json_object_get_string(obj)); }
   if(json_object_object_get_ex(o, "city", &obj))           { if(city) free(city); city = strdup(json_object_get_string(obj)); }
   if(json_object_object_get_ex(o, "asname", &obj))         { if(asname) free(asname); asname = strdup(json_object_get_string(obj)); }
-  if(json_object_object_get_ex(o, "category", &obj))       { snprintf(category, sizeof(category), "%s", json_object_get_string(obj)); }
-  if(json_object_object_get_ex(o, "httpbl", &obj))         { snprintf(httpbl, sizeof(httpbl), "%s", json_object_get_string(obj)); }
+  if(json_object_object_get_ex(o, "trafficCategory", &obj))         { snprintf(trafficCategory, sizeof(trafficCategory), "%s", json_object_get_string(obj)); }
   if(json_object_object_get_ex(o, "vlan_id", &obj))   vlan_id = json_object_get_int(obj);
   if(json_object_object_get_ex(o, "latitude", &obj))  latitude  = (float)json_object_get_double(obj);
   if(json_object_object_get_ex(o, "longitude", &obj)) longitude = (float)json_object_get_double(obj);
@@ -926,6 +1009,17 @@ bool Host::deserialize(char *json_str, char *key) {
     if(http) http->deserialize(obj);
   }
 
+  if(categoryStats) {
+    delete categoryStats;
+    categoryStats = NULL;
+  }
+
+  // deserialize categories only if flashstart is enabled for the current instance
+  if(json_object_object_get_ex(o, "categories", &obj) && ntop->get_flashstart()) {
+    categoryStats = new CategoryStats();
+    if(categoryStats) categoryStats->deserialize(obj);
+  }
+
   if(ndpiStats) {
     delete ndpiStats;
     ndpiStats = NULL;
@@ -980,7 +1074,7 @@ void Host::updateSynFlags(time_t when, u_int8_t flags, Flow *f, bool syn_sent) {
 #endif
 
     h = ip->print(ip_buf, sizeof(ip_buf));
-
+    
     if(syn_sent) {
       error_msg = "Host <A HREF=%s/lua/host_details.lua?host=%s&ifname=%s>%s</A> is a SYN flooder [%u SYNs sent in the last %u sec] %s";
       snprintf(msg, sizeof(msg),
@@ -990,21 +1084,24 @@ void Host::updateSynFlags(time_t when, u_int8_t flags, Flow *f, bool syn_sent) {
 	       counter->getOverThresholdDuration(),
 	       f->print(flow_buf, sizeof(flow_buf)));
     } else {
+      char attacker_buf[64], *attacker_str;
       Host *attacker = f->get_srv_host();
       IpAddress *aip = attacker->get_ip();
       char aip_buf[48], *aip_ptr;
+
+      attacker_str = attacker->get_ip()->print(attacker_buf, sizeof(attacker_buf));
       aip_ptr = aip->print(aip_buf, sizeof(aip_buf));
       error_msg = "Host <A HREF=%s/lua/host_details.lua?host=%s&ifname=%s>%s</A> is under SYN flood attack by host %s [%u SYNs received in the last %u sec] %s";
       snprintf(msg, sizeof(msg),
 	       error_msg, ntop->getPrefs()->get_http_prefix(),
-	       h, iface->get_name(), h, aip_ptr,
+	       h, iface->get_name(), attacker_str, aip_ptr,
 	       counter->getCurrentHits(),
 	       counter->getOverThresholdDuration(),
 	       f->print(flow_buf, sizeof(flow_buf)));
     }
 
     ntop->getTrace()->traceEvent(TRACE_INFO, "SYN Flood: %s", msg);
-    ntop->getRedis()->queueAlert(alert_level_error, alert_syn_flood, msg);
+    ntop->getRedis()->queueAlert(alert_level_error, alert_on, alert_syn_flood, msg);
     incNumAlerts();
   }
 }
@@ -1012,12 +1109,10 @@ void Host::updateSynFlags(time_t when, u_int8_t flags, Flow *f, bool syn_sent) {
 /* *************************************** */
 
 void Host::incNumFlows(bool as_client) {
-  if(!localHost || !triggerAlerts()) return;
-
   if(as_client) {
     total_num_flows_as_client++, num_active_flows_as_client++;
 
-    if(num_active_flows_as_client == max_num_active_flows) {
+    if(num_active_flows_as_client == max_num_active_flows && localHost && triggerAlerts()) {
       const char* error_msg = "Host <A HREF=%s/lua/host_details.lua?host=%s&ifname=%s>%s</A> is a possible scanner [%u active flows]";
       char ip_buf[48], *h, msg[512];
 
@@ -1028,13 +1123,13 @@ void Host::incNumFlows(bool as_client) {
 	       h, iface->get_name(), h, num_active_flows_as_client);
 
       ntop->getTrace()->traceEvent(TRACE_INFO, "Begin scan attack: %s", msg);
-      ntop->getRedis()->queueAlert(alert_level_error, alert_flow_flood, msg);
+      ntop->getRedis()->queueAlert(alert_level_error, alert_on, alert_flow_flood, msg);
       incNumAlerts();
     }
   } else {
     total_num_flows_as_server++, num_active_flows_as_server++;
 
-    if(num_active_flows_as_server == max_num_active_flows) {
+    if(num_active_flows_as_server == max_num_active_flows && localHost && triggerAlerts()) {
       const char* error_msg = "Host <A HREF=%s/lua/host_details.lua?host=%s&ifname=%s>%s</A> is possibly under scan attack [%u active flows]";
       char ip_buf[48], *h, msg[512];
 
@@ -1045,7 +1140,7 @@ void Host::incNumFlows(bool as_client) {
 	       h, iface->get_name(), h, num_active_flows_as_server);
 
       ntop->getTrace()->traceEvent(TRACE_INFO, "Begin scan attack: %s", msg);
-      ntop->getRedis()->queueAlert(alert_level_error, alert_flow_flood, msg);
+      ntop->getRedis()->queueAlert(alert_level_error, alert_on, alert_flow_flood, msg);
       incNumAlerts();
     }
   }
@@ -1054,13 +1149,11 @@ void Host::incNumFlows(bool as_client) {
 /* *************************************** */
 
 void Host::decNumFlows(bool as_client) {
-  if(!localHost || !triggerAlerts()) return;
-
   if(as_client) {
     if(num_active_flows_as_client) {
       num_active_flows_as_client--;
 
-      if(num_active_flows_as_client == max_num_active_flows) {
+      if(num_active_flows_as_client == max_num_active_flows && localHost && triggerAlerts()) {
 	const char* error_msg = "Host <A HREF=%s/lua/host_details.lua?host=%s&ifname=%s>%s</A> is no longer a possible scanner [%u active flows]";
 	char ip_buf[48], *h, msg[512];
 
@@ -1071,7 +1164,7 @@ void Host::decNumFlows(bool as_client) {
 		 h, iface->get_name(), h, num_active_flows_as_client);
 
 	ntop->getTrace()->traceEvent(TRACE_INFO, "End scan attack: %s", msg);
-	ntop->getRedis()->queueAlert(alert_level_info, alert_flow_flood, msg);
+	ntop->getRedis()->queueAlert(alert_level_info, alert_off, alert_flow_flood, msg);
 	incNumAlerts();
       }
     } else
@@ -1080,7 +1173,7 @@ void Host::decNumFlows(bool as_client) {
     if(num_active_flows_as_server) {
       num_active_flows_as_server--;
 
-      if(num_active_flows_as_server == max_num_active_flows) {
+      if(num_active_flows_as_server == max_num_active_flows && localHost && triggerAlerts()) {
 	const char* error_msg = "Host <A HREF=%s/lua/host_details.lua?host=%s&ifname=%s>%s</A> is no longer under scan attack [%u active flows]";
 	char ip_buf[48], *h, msg[512];
 
@@ -1091,7 +1184,7 @@ void Host::decNumFlows(bool as_client) {
 		 h, iface->get_name(), h, num_active_flows_as_server);
 
 	ntop->getTrace()->traceEvent(TRACE_INFO, "End scan attack: %s", msg);
-	ntop->getRedis()->queueAlert(alert_level_info, alert_flow_flood, msg);
+	ntop->getRedis()->queueAlert(alert_level_info, alert_off, alert_flow_flood, msg);
 	incNumAlerts();
       }
     } else
@@ -1128,6 +1221,19 @@ void Host::updateStats(struct timeval *tv) {
 
   if(!localHost || !triggerAlerts()) return;
 
+  if(tv->tv_sec >= nextSitesUpdate) {
+    if(nextSitesUpdate > 0) {
+      char oldk[64];
+      
+      snprintf(oldk, sizeof(oldk), "%s.old", topSitesKey);
+      ntop->getRedis()->rename(topSitesKey, oldk);
+      
+      ntop->getRedis()->zTrim(oldk, 10);
+    }
+
+    nextSitesUpdate = tv->tv_sec + HOST_SITES_REFRESH;
+  }
+
   if(isAboveQuota()) {
     const char *error_msg = "Host <A HREF=%s/lua/host_details.lua?host=%s&ifname=%s>%s</A> is above quota [%u])";
     char ip_buf[48], *h, msg[512];
@@ -1136,7 +1242,7 @@ void Host::updateStats(struct timeval *tv) {
     snprintf(msg, sizeof(msg),
              error_msg, ntop->getPrefs()->get_http_prefix(),
              h, iface->get_name(), h, host_quota_mb);
-    ntop->getRedis()->queueAlert(alert_level_warning, alert_quota, msg);
+    ntop->getRedis()->queueAlert(alert_level_warning, alert_permanent, alert_quota, msg);
   }
 }
 
@@ -1253,17 +1359,20 @@ void Host::readAlertPrefs() {
 /* *************************************** */
 
 void Host::incHitter(Host *peer, u_int64_t sent_bytes, u_int64_t rcvd_bytes) {
+#ifdef NOTUSED // check for memory corruptions here!
 #ifdef NTOPNG_PRO
   if(sent_bytes) sent_to_sketch->update(peer->key(), sent_bytes);
   if(rcvd_bytes) rcvd_from_sketch->update(peer->key(), rcvd_bytes);
 #endif
+#endif
 }
 
 /* *************************************** */
 
 void Host::getPeerBytes(lua_State* vm, u_int32_t peer_key) {
-  lua_newtable(vm);
-  
+#ifdef NOTUSED
+ lua_newtable(vm);
+
 #ifdef NTOPNG_PRO
   if(sent_to_sketch && rcvd_from_sketch) {
     lua_push_int_table_entry(vm, "sent", sent_to_sketch->estimate(peer_key));
@@ -1274,8 +1383,74 @@ void Host::getPeerBytes(lua_State* vm, u_int32_t peer_key) {
 
   lua_push_int_table_entry(vm, "sent", 0);
   lua_push_int_table_entry(vm, "rcvd", 0);
+#endif
 }
 
 /* *************************************** */
 
+void Host::incLowGoodputFlows(bool asClient) {
+  bool alert = false;
+  
+  if(asClient) {
+    if(++low_goodput_client_flows > HOST_LOW_GOODPUT_THRESHOLD) alert = true;
+  } else {
+    if(++low_goodput_server_flows > HOST_LOW_GOODPUT_THRESHOLD) alert = true;
+  }
 
+  /* TODO: decide if an alert should be sent in a future version */
+  if(alert && (!good_low_flow_detected)) {
+#if 0
+    char alert_msg[1024], *c, c_buf[64];
+
+    c = get_ip()->print(c_buf, sizeof(c_buf));
+
+    snprintf(alert_msg, sizeof(alert_msg),
+	     "Host <A HREF='/lua/host_details.lua?host=%s&ifname=%s'>%s</A> has %d low goodput active %s flows",
+	     c, iface->get_name(), get_name() ? get_name() : c,
+	     HOST_LOW_GOODPUT_THRESHOLD, asClient ? "client" : "server");
+
+    ntop->getRedis()->queueAlert(alert_level_error, alert_on, asClient ? alert_host_under_attack : alert_host_attacker, alert_msg);
+#endif
+    good_low_flow_detected = true;
+  }
+}
+
+/* *************************************** */
+
+void Host::decLowGoodputFlows(bool asClient) {
+  bool alert = false;
+
+  if(asClient) {
+    if(--low_goodput_client_flows < HOST_LOW_GOODPUT_THRESHOLD) alert = true;
+  } else {
+    if(--low_goodput_server_flows < HOST_LOW_GOODPUT_THRESHOLD) alert = true;
+  }
+
+  if(alert && good_low_flow_detected) {
+    /* TODO: send end of alert */
+    good_low_flow_detected = false;
+  }
+}
+
+/* *************************************** */
+
+void Host::incrVisitedWebSite(char *hostname) {
+  u_int ip4_0 = 0, ip4_1 = 0, ip4_2 = 0, ip4_3 = 0;
+  
+  if(topSitesKey 
+    && (strstr(hostname, "in-addr.arpa") == NULL)
+    && (sscanf(hostname, "%u.%u.%u.%u", &ip4_0, &ip4_1, &ip4_2, &ip4_3) != 4)
+    ) {
+#if 0
+    char *firstdot = strchr(hostname, '.');
+
+    if(firstdot) {
+      char *nextdot = strchr(&firstdot[1], '.');
+
+      ntop->getRedis()->zIncr(topSitesKey, nextdot ? &firstdot[1] : hostname);
+    }
+#else
+    ntop->getRedis()->zIncr(topSitesKey, hostname);
+#endif
+  }
+}
