@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -37,6 +37,7 @@ static const char* dirs[] = {
   NULL,
 #endif
   CONST_ALT_INSTALL_DIR,
+  CONST_ALT2_INSTALL_DIR,
   NULL
 };
 
@@ -47,10 +48,10 @@ Ntop::Ntop(char *appName) {
   globals = new NtopGlobals();
   pa = new PeriodicActivities();
   address = new AddressResolution();
-  categorization = NULL;
-  httpbl = NULL;
+  httpbl = NULL, flashstart = NULL;
   custom_ndpi_protos = NULL;
   prefs = NULL, redis = NULL;
+  elastic_search = NULL;
   num_cpus = -1;
   num_defined_interfaces = num_defined_interface_views = 0;
   local_interface_addresses = New_Patricia(128);
@@ -87,7 +88,8 @@ Ntop::Ntop(char *appName) {
   umask(0);
 
   if(getcwd(startup_dir, sizeof(startup_dir)) == NULL)
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Occurred while checking the current directory (errno=%d)", errno);
+    ntop->getTrace()->traceEvent(TRACE_ERROR,
+				 "Occurred while checking the current directory (errno=%d)", errno);
 
   dirs[0] = startup_dir;
 
@@ -101,11 +103,7 @@ Ntop::Ntop(char *appName) {
     fixPath(path);
 
     if(stat(path, &statbuf) == 0) {
-#ifdef __OpenBSD__
-      strlcpy(install_dir, dirs[i], sizeof(install_dir));
-#else
-      strcpy(install_dir, dirs[i]);
-#endif
+      snprintf(install_dir, sizeof(install_dir), "%s", dirs[i]);
       break;
     }
   }
@@ -164,9 +162,11 @@ Ntop::~Ntop() {
 
   if(udp_socket != -1) closesocket(udp_socket);
 
-  if(httpbl) delete httpbl;
-  if(httpd)  delete httpd;
+  if(httpbl)     delete httpbl;
+  if(flashstart) delete flashstart;
+  if(httpd)      delete httpd;
   if(custom_ndpi_protos) delete(custom_ndpi_protos);
+  if(elastic_search) delete(elastic_search);
 
   Destroy_Patricia(local_interface_addresses, NULL);
   delete address;
@@ -197,7 +197,7 @@ void Ntop::registerPrefs(Prefs *_prefs, bool quick_registration) {
        || (!(statbuf.st_mode & S_IWRITE)) /* It's not writable    */) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "Invalid directory %s specified",
 				   prefs->get_data_dir());
-      _exit(-1);
+      exit(-1);
     }
 
     if(stat(prefs->get_callbacks_dir(), &statbuf)
@@ -205,7 +205,7 @@ void Ntop::registerPrefs(Prefs *_prefs, bool quick_registration) {
        || (!(statbuf.st_mode & S_IREAD)) /* It's not readable    */) {
       ntop->getTrace()->traceEvent(TRACE_ERROR, "Invalid directory %s specified",
 				   prefs->get_callbacks_dir());
-      _exit(-1);
+      exit(-1);
     }
 
     if(prefs->get_local_networks()) {
@@ -221,6 +221,8 @@ void Ntop::registerPrefs(Prefs *_prefs, bool quick_registration) {
 
   initRedis();
 
+  initElasticSearch();
+
 #ifdef NTOPNG_PRO
   pro->check_license(true, false);
 #endif
@@ -230,6 +232,7 @@ void Ntop::registerPrefs(Prefs *_prefs, bool quick_registration) {
 
 #ifdef NTOPNG_PRO
 void Ntop::registerNagios(void) {
+  if(nagios_manager) { delete nagios_manager; nagios_manager = NULL; }
   nagios_manager = new NagiosManager();
 }
 #endif
@@ -244,6 +247,14 @@ void Ntop::initRedis() {
 
 /* ******************************************* */
 
+void Ntop::initElasticSearch() {
+  if(elastic_search) delete(elastic_search);
+
+  elastic_search = new ElasticSearch();
+}
+
+/* ******************************************* */
+
 void Ntop::createExportInterface() {
   if(prefs->get_export_endpoint())
     export_interface = new ExportInterface(prefs->get_export_endpoint());
@@ -258,7 +269,7 @@ void Ntop::start() {
   time_t when = time(NULL);
 
   getTrace()->traceEvent(TRACE_NORMAL,
-			 "Welcome to ntopng %s v.%s - (C) 1998-15 ntop.org",
+			 "Welcome to ntopng %s v.%s - (C) 1998-2016 ntop.org",
 			 PACKAGE_MACHINE, PACKAGE_VERSION);
 
   if(PACKAGE_OS[0] != '\0')
@@ -270,30 +281,34 @@ void Ntop::start() {
   snprintf(buf, sizeof(buf), "ntopng.%s.hostkeys", daybuf);
 
   pa->startPeriodicActivitiesLoop();
-  if(categorization) categorization->startCategorizeCategorizationLoop();
-  if(httpbl) httpbl->startHTTPBLLoop();
+  if(httpbl) httpbl->startLoop();
+  else if(flashstart) flashstart->startLoop();
 
   runtimeprefs = new RuntimePrefs();
 
   prefs->loadIdleDefaults();
 #ifdef NTOPNG_PRO
   pro->printLicenseInfo();
-  prefs->loadNagiosDefaults();
 #endif
+  prefs->loadInstanceNameDefaults();
   loadLocalInterfaceAddress();
 
-  for(int i=0; i<num_defined_interfaces; i++){
-    iface[i]->startPacketPolling();
+  for(int i=0; i<num_defined_interfaces; i++) {
     iface[i]->allocateNetworkStats();
+    iface[i]->startPacketPolling();
   }
 
   sleep(2);
   address->startResolveAddressLoop();
 
   while(!globals->isShutdown()) {
-    sleep(HOUSEKEEPING_FREQUENCY);
+    u_int16_t nap = ntop->getPrefs()->get_housekeeping_frequency();
+    ntop->getTrace()->traceEvent(TRACE_DEBUG,
+				 "Sleeping %i seconds before doing the chores.",
+				 nap);
+    sleep(nap);
+    ntop->getTrace()->traceEvent(TRACE_DEBUG, "Going to do the chores.");
     runHousekeepingTasks();
-    // break;
   }
 }
 
@@ -473,7 +488,7 @@ void Ntop::loadLocalInterfaceAddress() {
 	   && (ifa->ifa_addr->sa_family != AF_INET6))
        || ((ifa->ifa_flags & IFF_UP) == 0))
       continue;
-    
+
     for(int i=0; i<num_defined_interfaces; i++) {
       if(strstr(iface[i]->get_name(), ifa->ifa_name)) {
 	ifId = i;
@@ -538,7 +553,7 @@ void Ntop::loadLocalInterfaceAddress() {
 	iface[ifId]->addInterfaceAddress(buf_orig);
 #endif
 
-	for(u_int32_t i = cidr, j = 0; i > 0; i -= 8, ++j)
+	for(int i = cidr, j = 0; i > 0; i -= 8, ++j)
 	  s6->sin6_addr.s6_addr[j] &= i >= 8 ? 0xff : (u_int32_t)(( 0xffU << ( 8 - i ) ) & 0xffU );
 
 	inet_ntop(ifa->ifa_addr->sa_family,(void *)&(s6->sin6_addr), buf, sizeof(buf));
@@ -634,6 +649,13 @@ void Ntop::getUsers(lua_State* vm) {
     else
       lua_push_str_table_entry(vm, CONST_ALLOWED_NETS, (char*)"");
 
+    snprintf(key, sizeof(key), CONST_STR_USER_ALLOWED_IFNAME, username);
+    if((ntop->getRedis()->get(key, val, sizeof(val)) >= 0)
+       && val[0] != '\0')
+      lua_push_str_table_entry(vm, CONST_ALLOWED_IFNAME, val);
+    else
+      lua_push_str_table_entry(vm, CONST_ALLOWED_IFNAME, (char*)"");
+
     lua_pushstring(vm, username);
     lua_insert(vm, -2);
     lua_settable(vm, -3);
@@ -665,6 +687,18 @@ void Ntop::getUserGroup(lua_State* vm) {
     return;
   }
 
+#if defined(NTOPNG_PRO) && defined(HAVE_LDAP)
+  snprintf(key, sizeof(key), PREF_USER_TYPE_LOG, username);
+  if (ntop->getRedis()->get(key, val, sizeof(val)) >= 0){
+    if( !strcmp(val,"ldap") ) {
+      snprintf(key, sizeof(key), PREF_LDAP_GROUP_OF_USER, username);
+      lua_pushstring(vm,
+    		 (ntop->getRedis()->get(key, val, sizeof(val)) >= 0) ? val : (char*)"unknown");
+      return;
+    }
+  }
+#endif
+
   snprintf(key, sizeof(key), CONST_STR_USER_GROUP, username);
   lua_pushstring(vm,
 		 (ntop->getRedis()->get(key, val, sizeof(val)) >= 0) ? val : (char*)"unknown");
@@ -689,24 +723,135 @@ void Ntop::getAllowedNetworks(lua_State* vm) {
   snprintf(key, sizeof(key), CONST_STR_USER_NETS, username);
   lua_pushstring(vm, (ntop->getRedis()->get(key, val, sizeof(val)) >= 0) ? val : (char*)"");
 }
+
+/* ******************************************* */
+
+bool Ntop::getInterfaceAllowed(lua_State* vm, char *ifname) const {
+  char *allowed_ifname;
+
+  if(ifname == NULL) {
+    return false;
+  }
+
+  lua_getglobal(vm, CONST_ALLOWED_IFNAME);
+  if((allowed_ifname = (char*)lua_touserdata(vm, lua_gettop(vm))) == NULL
+     || allowed_ifname[0] == '\0') {
+    ifname = NULL;
+    return false;
+  }
+  strncpy(ifname, allowed_ifname, strlen(allowed_ifname));
+  return true;
+}
+
+/* ******************************************* */
+
+bool Ntop::isInterfaceAllowed(lua_State* vm, const char *ifname) const {
+  char *allowed_ifname;
+  bool ret;
+
+  if(vm == NULL || ifname == NULL) {
+    return true; /* always return true when no lua state is passed */
+  }
+
+  lua_getglobal(vm, CONST_ALLOWED_IFNAME);
+  if((allowed_ifname = (char*)lua_touserdata(vm, lua_gettop(vm))) == NULL
+     || allowed_ifname[0] == '\0') {
+    ntop->getTrace()->traceEvent(TRACE_DEBUG,
+				 "No allowed interface found for %s", ifname);
+    // this is a lua script called within nopng (no HTTP UI and user interaction)
+    ret = true;
+  } else {
+    ntop->getTrace()->traceEvent(TRACE_DEBUG,
+				 "Allowed interface %s, requested %s",
+				 allowed_ifname, ifname);
+    ret = !strncmp(allowed_ifname, ifname, strlen(allowed_ifname));
+  }
+
+  lua_pop(vm, 1);
+  return ret;
+}
+
+/* ******************************************* */
+
+bool Ntop::isInterfaceAllowed(lua_State* vm, int ifid) const {
+  return isInterfaceAllowed(vm, prefs->get_if_name(ifid));
+}
+
 /* ******************************************* */
 
 // Return 1 if username/password is allowed, 0 otherwise.
 bool Ntop::checkUserPassword(const char *user, const char *password) {
-  char key[64], val[64];
+  char key[64], val[64], password_hash[33];
+#if defined(NTOPNG_PRO) && defined(HAVE_LDAP)
+  bool localAuth = true;
+#endif
 
   if((user == NULL) || (user[0] == '\0'))
     return(false);
 
+#if defined(NTOPNG_PRO) && defined(HAVE_LDAP)
+  if(ntop->getRedis()->get((char*)PREF_NTOP_AUTHENTICATION_TYPE, val, sizeof(val)) >= 0) {
+    if(!strcmp(val, "ldap") /* LDAP only */) localAuth = false;
+
+    if(strncmp(val, "ldap", 4) == 0) {
+      bool is_admin;
+      char ldapServer[64] = { 0 }, ldapAccountType[64] = { 0 }, ldapAnonymousBind[32] = { 0 },
+           bind_dn[128] = { 0 }, bind_pwd[64] = { 0 }, group[64] = { 0 },
+           search_path[128] = { 0 }, admin_group[64] = { 0 };
+
+      if(!password || !password[0])
+        return false;
+
+      ntop->getRedis()->get((char*)PREF_LDAP_SERVER, ldapServer, sizeof(ldapServer));
+      ntop->getRedis()->get((char*)PREF_LDAP_ACCOUNT_TYPE, ldapAccountType, sizeof(ldapAccountType));
+      ntop->getRedis()->get((char*)PREF_LDAP_BIND_ANONYMOUS, ldapAnonymousBind, sizeof(ldapAnonymousBind));
+      ntop->getRedis()->get((char*)PREF_LDAP_BIND_DN, bind_dn, sizeof(bind_dn));
+      ntop->getRedis()->get((char*)PREF_LDAP_BIND_PWD, bind_pwd, sizeof(bind_pwd));
+      ntop->getRedis()->get((char*)PREF_LDAP_SEARCH_PATH, search_path, sizeof(search_path));
+      ntop->getRedis()->get((char*)PREF_LDAP_USER_GROUP, group, sizeof(group));
+      ntop->getRedis()->get((char*)PREF_LDAP_ADMIN_GROUP, admin_group, sizeof(admin_group));
+
+      if(ldapServer[0]) {
+	bool ret = LdapAuthenticator::validUserLogin(ldapServer, ldapAccountType,
+						     (atoi(ldapAnonymousBind) == 0) ? false : true,
+						     bind_dn[0] != '\0' ? bind_dn : NULL,
+						     bind_pwd[0] != '\0' ? bind_pwd : NULL,
+						     search_path[0] != '\0' ? search_path : NULL,
+						     user,
+						     password[0] != '\0' ? password : NULL,
+						     group[0] != '\0' ? group : NULL,
+						     admin_group[0] != '\0' ? admin_group : NULL,
+						     &is_admin);
+
+	if(ret) {
+	  snprintf(key, sizeof(key), PREF_LDAP_GROUP_OF_USER, user);
+	  ntop->getRedis()->set(key, is_admin ?  (char*)"administrator" : (char*)"unprivileged", 0);
+          snprintf(key, sizeof(key), PREF_USER_TYPE_LOG, user);
+	  ntop->getRedis()->set(key, (char*)"ldap", 0);
+	  return(true);
+	}
+      }
+    }
+  }
+
+  if(!localAuth) return(false);
+#endif
+
   snprintf(key, sizeof(key), CONST_STR_USER_PASSWORD, user);
 
   if(ntop->getRedis()->get(key, val, sizeof(val)) < 0) {
     return(false);
   } else {
-    char password_hash[33];
-
     mg_md5(password_hash, password, NULL);
-    return((strcmp(password_hash, val) == 0) ? true : false);
+    if(strcmp(password_hash, val) == 0) {
+#if defined(NTOPNG_PRO) && defined(HAVE_LDAP)
+      snprintf(key, sizeof(key), PREF_USER_TYPE_LOG, user);
+      ntop->getRedis()->set(key, (char*)"local", 0);
+#endif
+      return(true);
+    } else {
+      return(false);
+    }
   }
 }
 
@@ -762,7 +907,36 @@ bool Ntop::changeAllowedNets(char *username, char *allowed_nets) const {
 
 /* ******************************************* */
 
-bool Ntop::addUser(char *username, char *full_name, char *password, char *host_role, char *allowed_networks) {
+bool Ntop::changeAllowedIfname(char *username, char *allowed_ifname) const {
+  /* Add as exception :// */
+  char *column_slash = strstr(allowed_ifname, ":__");
+  
+  if (username == NULL || username[0] == '\0')
+    return false;
+  
+  if(column_slash)
+    column_slash[1] = column_slash[2] = '/';  
+
+  ntop->getTrace()->traceEvent(TRACE_DEBUG,
+			       "Changing allowed ifname to %s for %s",
+			       allowed_ifname, username);
+
+  char key[64];
+  snprintf(key, sizeof(key), CONST_STR_USER_ALLOWED_IFNAME, username);
+
+  if(allowed_ifname != NULL and allowed_ifname[0] != '\0') {
+    return (ntop->getRedis()->set(key, allowed_ifname, 0) >= 0);
+  } else {
+    ntop->getRedis()->delKey(key);
+  }
+
+  return(true);
+}
+
+/* ******************************************* */
+
+bool Ntop::addUser(char *username, char *full_name, char *password, char *host_role,
+		   char *allowed_networks, char *allowed_ifname) {
   char key[64], val[64];
   char password_hash[33];
 
@@ -782,6 +956,12 @@ bool Ntop::addUser(char *username, char *full_name, char *password, char *host_r
   snprintf(key, sizeof(key), CONST_STR_USER_NETS, username);
   ntop->getRedis()->set(key, allowed_networks, 0);
 
+  if(allowed_ifname && allowed_ifname[0] != '\0'){
+    ntop->getTrace()->traceEvent(TRACE_DEBUG, "Setting allowed ifname: %s", allowed_ifname);
+    snprintf(key, sizeof(key), CONST_STR_USER_ALLOWED_IFNAME, username);
+    ntop->getRedis()->set(key, allowed_ifname, 0);
+  }
+
   return(true);
 }
 
@@ -791,13 +971,21 @@ bool Ntop::deleteUser(char *username) {
   char key[64];
 
   snprintf(key, sizeof(key), CONST_STR_USER_FULL_NAME, username);
-  ntop->getRedis()->del(key);
+  ntop->getRedis()->delKey(key);
 
   snprintf(key, sizeof(key), CONST_STR_USER_GROUP, username);
-  ntop->getRedis()->del(key);
+  ntop->getRedis()->delKey(key);
 
   snprintf(key, sizeof(key), CONST_STR_USER_PASSWORD, username);
-  return((ntop->getRedis()->del(key) >= 0) ? true : false);
+  ntop->getRedis()->delKey(key);
+
+  snprintf(key, sizeof(key), CONST_STR_USER_NETS, username);
+  ntop->getRedis()->delKey(key);
+
+  snprintf(key, sizeof(key), CONST_STR_USER_ALLOWED_IFNAME, username);
+  ntop->getRedis()->delKey(key);
+
+  return true;
 }
 
 /* ******************************************* */
@@ -906,7 +1094,7 @@ void Ntop::daemonize() {
     } else { /* father */
       ntop->getTrace()->traceEvent(TRACE_NORMAL,
 				   "Parent process is exiting (this is normal)");
-      _exit(0);
+      exit(0);
     }
   }
 #endif
@@ -937,7 +1125,10 @@ void Ntop::setLocalNetworks(char *_nets) {
 
 /* ******************************************* */
 
-NetworkInterface* Ntop::getNetworkInterface(const char *name) {
+NetworkInterface* Ntop::getNetworkInterface(lua_State* vm, const char *name) {
+  if(name == NULL)
+    return NULL;
+
   /* This method accepts both interface names or Ids */
   int if_id = atoi(name);
   char str[8];
@@ -948,17 +1139,15 @@ NetworkInterface* Ntop::getNetworkInterface(const char *name) {
 
     for(int i=0; i<num_defined_interfaces; i++) {
       if(iface[i]->get_id() == if_id)
-	return(iface[i]);
+	return isInterfaceAllowed(vm, iface[i]->get_name()) ? iface[i] : NULL;
     }
-
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to find interface Id %d", if_id);
-
     return(NULL);
   }
 
-  for(int i=0; i<num_defined_interfaces; i++) {
+  /* if here, name is a string */
+  for(int i=0; i<num_defined_interfaces; i++) {    
     if(strstr(name, iface[i]->get_name()))
-      return(iface[i]);
+      return isInterfaceAllowed(vm, iface[i]->get_name()) ? iface[i] : NULL;
   }
 
   /* FIX: remove this for at some point, when endpoint is passed */
@@ -977,8 +1166,11 @@ NetworkInterface* Ntop::getNetworkInterface(const char *name) {
 
 /* ******************************************* */
 
-NetworkInterfaceView* Ntop::getNetworkInterfaceView(const char *name) {
+NetworkInterfaceView* Ntop::getNetworkInterfaceView(lua_State* vm, const char *name) {
   /* This method accepts both interface names or Ids */
+  ntop->getTrace()->traceEvent(TRACE_DEBUG, "Getting interface view at name %s", name ? name : "NULL");
+  if(name == NULL)
+    return NULL;
   int if_id = atoi(name);
   char str[8];
 
@@ -988,102 +1180,24 @@ NetworkInterfaceView* Ntop::getNetworkInterfaceView(const char *name) {
 
     for(int i=0; i<num_defined_interface_views; i++) {
       if(ifaceViews[i]->get_id() == if_id)
-	return(ifaceViews[i]);
+	return isInterfaceAllowed(vm, ifaceViews[i]->get_name()) ? ifaceViews[i] : NULL;
     }
 
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to find interface view Id %d", if_id);
-
-    return(NULL);
+    return NULL;
   }
 
+  /* if here, interface view name is a string */
   for(int i=0; i<num_defined_interface_views; i++) {
-    if(strcmp(ifaceViews[i]->get_name(), name) == 0)
-      return(ifaceViews[i]);
+    if(!strcmp(ifaceViews[i]->get_name(), name))
+      return isInterfaceAllowed(vm, ifaceViews[i]->get_name()) ? ifaceViews[i] : NULL;
   }
 
-  return(ifaceViews[0]); /* FIX: remove at some point */
+  return NULL;
 };
 
 /* ******************************************* */
 
-NetworkInterface* Ntop::getInterfaceById(int id){
-  for(int i=0; i<num_defined_interfaces; i++) {
-    if(iface[i]->get_id() == id) {
-      return(iface[i]);
-    }
-  }
-
-  return(NULL);
-}
-
-/* ******************************************* */
-
-NetworkInterfaceView* Ntop::getInterfaceViewById(int id){
-  for(int i=0; i<num_defined_interface_views; i++) {
-    if(ifaceViews[i]->get_id() == id)
-      return(ifaceViews[i]);
-  }
-
-  return(NULL);
-}
-
-/* ******************************************* */
-
-NetworkInterface* Ntop::getInterface(char *name) {
-  /* This method accepts both interface names or Ids */
-  int if_id;
-  char str[8];
-
-  if(name == NULL) return(NULL);
-
-  if_id = atoi(name);
-
-  snprintf(str, sizeof(str), "%d", if_id);
-  if(strcmp(name, str) == 0) {
-    /* name is a number */
-
-    return(getInterfaceById(if_id));
-  }
-
-  for(int i=0; i<num_defined_interfaces; i++) {
-    if(strcmp(iface[i]->get_name(), name) == 0) {
-      return(iface[i]);
-    }
-  }
-
-  return(NULL);
-}
-
-/* ******************************************* */
-
-NetworkInterfaceView* Ntop::getInterfaceView(char *name) {
-  /* This method accepts both interface names or Ids */
-  int if_id;
-  char str[8];
-
-  if(name == NULL) return(NULL);
-
-  if_id = atoi(name);
-
-  snprintf(str, sizeof(str), "%d", if_id);
-  if(strcmp(name, str) == 0) {
-    /* name is a number */
-
-    return(getInterfaceViewById(if_id));
-  }
-
-  for(int i=0; i<num_defined_interface_views; i++) {
-    if(strcmp(ifaceViews[i]->get_name(), name) == 0) {
-      return(ifaceViews[i]);
-    }
-  }
-
-  return(NULL);
-}
-
-/* ******************************************* */
-
-/* NOTUSED */
+#ifdef NOTUSED
 int Ntop::getInterfaceIdByName(char *name) {
   /* This method accepts both interface names or Ids */
   int if_id = atoi(name);
@@ -1092,7 +1206,7 @@ int Ntop::getInterfaceIdByName(char *name) {
   snprintf(str, sizeof(str), "%d", if_id);
   if(strcmp(name, str) == 0) {
     /* name is a number */
-    NetworkInterface *iface = getInterfaceById(if_id);
+    NetworkInterface *iface = getNetworkInterface(if_id);
 
     if(iface != NULL)
       return(iface->get_id());
@@ -1108,6 +1222,7 @@ int Ntop::getInterfaceIdByName(char *name) {
 
   return(-1);
 }
+#endif
 
 /* ******************************************* */
 
@@ -1119,7 +1234,7 @@ int Ntop::getInterfaceViewIdByName(char *name) {
   snprintf(str, sizeof(str), "%d", if_id);
   if(strcmp(name, str) == 0) {
     /* name is a number */
-    NetworkInterfaceView *iface = getInterfaceViewById(if_id);
+    NetworkInterfaceView *iface = getNetworkInterfaceView(if_id);
 
     if(iface != NULL)
       return(iface->get_id());
@@ -1152,10 +1267,9 @@ void Ntop::registerInterface(NetworkInterface *_if) {
     ntop->getTrace()->traceEvent(TRACE_NORMAL, "Registered interface %s [id: %d]",
 				 _if->get_name(), _if->get_id());
     iface[num_defined_interfaces++] = _if;
-    return;
-  }
-
-  ntop->getTrace()->traceEvent(TRACE_ERROR, "Too many interfaces defined");
+    _if->startDBLoop();
+  } else
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Too many interfaces defined");
 };
 
 /* ******************************************* */
