@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -32,22 +32,21 @@ PcapInterface::PcapInterface(const char *name) : NetworkInterface(name) {
   struct stat buf;
 
   pcap_handle = NULL, pcap_list = NULL;
-
   
-  if((stat(name, &buf) == 0) || (name[0] == '-')) {
+  if((stat(name, &buf) == 0) || (name[0] == '-') || !strncmp(name, "stdin", 5)) {
     /*
       The file exists so we need to check if it's a 
       text file or a pcap file
     */
 
-    if(strcmp(name, "-") == 0) {
+    if(strcmp(name, "-") == 0 || !strncmp(name, "stdin", 5)) {
       /* stdin */
       pcap_handle = pcap_fopen_offline(stdin, pcap_error_buffer);
       pcap_datalink_type = DLT_EN10MB;
     } else if((pcap_handle = pcap_open_offline(ifname, pcap_error_buffer)) == NULL) {
       if((pcap_list = fopen(name, "r")) == NULL) {
 	ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to open file %s", name);
-	_exit(0);
+	exit(0);
       }	else
 	read_pkts_from_pcap_dump = true;
     } else {
@@ -86,7 +85,11 @@ PcapInterface::PcapInterface(const char *name) : NetworkInterface(name) {
       ntop->getTrace()->traceEvent(TRACE_NORMAL, "Reading packets from interface %s...", ifname);
       read_pkts_from_pcap_dump = false;
       pcap_datalink_type = pcap_datalink(pcap_handle);
-    }
+
+      if(pcap_setdirection(pcap_handle, ntop->getPrefs()->getCaptureDirection()) != 0)
+	ntop->getTrace()->traceEvent(TRACE_WARNING, "Unable to set packet capture direction");
+    } else
+      throw 1;
   }
   
   if(ntop->getPrefs()->are_ixia_timestamps_enabled())
@@ -160,8 +163,9 @@ static void* packetPollLoop(void* ptr) {
 
       if((rc = pcap_next_ex(pd, &hdr, &pkt)) > 0) {
 	if((rc > 0) && (pkt != NULL) && (hdr->caplen > 0)) {
-	  int a, b;
-	  
+	  u_int16_t p;
+	  bool shaped;
+
 #ifdef WIN32
 	  /*
 	    For some unknown reason, on Windows winpcap
@@ -177,10 +181,10 @@ static void* packetPollLoop(void* ptr) {
 	  hdr_copy.len = min(hdr->len, sizeof(pkt_copy) - 1);
 	  hdr_copy.caplen = min(hdr_copy.len, hdr_copy.caplen);
 	  memcpy(pkt_copy, pkt, hdr_copy.len);
-	  iface->packet_dissector(&hdr_copy, (const u_char*)pkt_copy, &a, &b);
+	  iface->dissectPacket(&hdr_copy, (const u_char*)pkt_copy, &shaped, &p);
 #else
 	  hdr->caplen = min_val(hdr->caplen, iface->getMTU());
-	  iface->packet_dissector(hdr, pkt, &a, &b);
+	  iface->dissectPacket(hdr, pkt, &shaped, &p);
 #endif
 	}
       } else if(rc < 0) {
