@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,29 +24,33 @@
 /* ******************************************* */
 
 Prefs::Prefs(Ntop *_ntop) {
-  num_deferred_interfaces_to_register = 0;
+  num_deferred_interfaces_to_register = 0, cli = NULL;
   memset(deferred_interfaces_to_register, 0, sizeof(deferred_interfaces_to_register));
   ntop = _ntop, sticky_hosts = location_none;
   local_networks = strdup(CONST_DEFAULT_HOME_NET","CONST_DEFAULT_LOCAL_NETS);
   local_networks_set = false, shutdown_when_done = false;
   enable_users_login = true, disable_localhost_login = false;
   enable_dns_resolution = sniff_dns_responses = true, use_promiscuous_mode = true;
-  categorization_enabled = false, httpbl_enabled = false, resolve_all_host_ip = false;
+  resolve_all_host_ip = false, online_license_check = false;
   max_num_hosts = MAX_NUM_INTERFACE_HOSTS, max_num_flows = MAX_NUM_INTERFACE_HOSTS;
   data_dir = strdup(CONST_DEFAULT_DATA_DIR);
-  install_dir = NULL;
+  install_dir = NULL, captureDirection = PCAP_D_INOUT;
   docs_dir = strdup(CONST_DEFAULT_DOCS_DIR);
   scripts_dir = strdup(CONST_DEFAULT_SCRIPTS_DIR);
   callbacks_dir = strdup(CONST_DEFAULT_CALLBACKS_DIR);
   config_file_path = ndpi_proto_path = NULL;
   http_port = CONST_DEFAULT_NTOP_PORT;
-  http_prefix = strdup("");
+  http_prefix = strdup(""), zmq_encryption_pwd = NULL;
+  instance_name = NULL;
+  categorization_enabled = false, enable_users_login = true;
+  categorization_key = NULL, zmq_encryption_pwd = NULL;
+  es_index = es_url = es_user = es_pwd = NULL;
   https_port = 0; // CONST_DEFAULT_NTOP_PORT+1;
   change_user = true, daemonize = false;
   user = strdup(CONST_DEFAULT_NTOP_USER);
-  http_binding_address = https_binding_address = CONST_ANY_ADDRESS;
-  categorization_key = NULL;
-  httpbl_key = NULL;
+  http_binding_address = NULL;
+  https_binding_address = NULL; // CONST_ANY_ADDRESS;
+  httpbl_key = NULL, flashstart = NULL;
   cpu_affinity = NULL;
   redis_host = strdup("127.0.0.1");
   redis_port = 6379;
@@ -56,7 +60,7 @@ Prefs::Prefs(Ntop *_ntop) {
   disable_alerts = false;
   pid_path = strdup(DEFAULT_PID_PATH);
   packet_filter = NULL;
-  disable_host_persistency = false;
+  disable_host_persistency = zmq_collector_mode = false;
   num_interfaces = 0, num_interface_views = 0, enable_auto_logout = true;
   dump_flows_on_es = dump_flows_on_mysql = false;
   enable_taps = false;
@@ -83,6 +87,7 @@ Prefs::Prefs(Ntop *_ntop) {
   other_rrd_1min_days     = OTHER_RRD_1MIN_DAYS;
   other_rrd_1h_days       = OTHER_RRD_1H_DAYS;
   other_rrd_1d_days       = OTHER_RRD_1D_DAYS;
+  housekeeping_frequency  = HOUSEKEEPING_FREQUENCY;
 
   es_type = strdup((char*)"flows"), es_index = strdup((char*)"ntopng-%Y.%m.%d"),
     es_url = strdup((char*)"http://localhost:9200/_bulk"),
@@ -90,9 +95,6 @@ Prefs::Prefs(Ntop *_ntop) {
 
   mysql_host = mysql_dbname = mysql_tablename = mysql_user = mysql_pw = NULL;
 
-#ifdef NTOPNG_PRO
-  nagios_host = nagios_port = nagios_config = NULL;
-#endif
 }
 
 /* ******************************************* */
@@ -119,33 +121,37 @@ Prefs::~Prefs() {
   if(es_url)           free(es_url);
   if(es_user)          free(es_user);
   if(es_pwd)           free(es_pwd);
+  if(instance_name)    free(instance_name);
+
   free(http_prefix);
   free(redis_host);
   free(local_networks);
-
+  if(cli)             free(cli);
   if(mysql_host)      free(mysql_host);
   if(mysql_dbname)    free(mysql_dbname);
   if(mysql_tablename) free(mysql_tablename);
   if(mysql_user)      free(mysql_user);
   if(mysql_pw)        free(mysql_pw);
 
-#ifdef NTOPNG_PRO
-  if(nagios_host)   free(nagios_host);
-  if(nagios_port)   free(nagios_port);
-  if(nagios_config) free(nagios_config);
-#endif
+  if(http_binding_address)  free(http_binding_address);
+  if(https_binding_address) free(https_binding_address);
+  /* NOTE: flashstart is deleted by the Ntop class */
 }
 
 /* ******************************************* */
 
-static void* localmalloc(unsigned long size) { return(malloc(size)); }
+/* C-binding needed by Win32 service call */
+void nDPIusage() {
+  printf("\nnDPI detected protocols:\n");
 
-/* ******************************************* */
+  struct ndpi_detection_module_struct *ndpi_struct = ndpi_init_detection_module();
+  ndpi_dump_protocols(ndpi_struct);
+
+  exit(0);
+}
 
 /* C-binding needed by Win32 service call */
 void usage() {
-  NetworkInterface n("dummy");
-
   printf("ntopng %s v.%s - "NTOP_COPYRIGHT"\n\n"
 	 "Usage:\n"
 	 "  ntopng <configuration file path>\n"
@@ -166,7 +172,9 @@ void usage() {
 #ifndef WIN32
 	 "[--data-dir|-d] <path>              | Data directory (must be writable).\n"
 	 "                                    | Default: %s\n"
-	 "[--install-dir|-t] <path>           | Set the installation directory to <dir>. Testing only.\n"
+	 "[--install-dir|-t] <path>           | Set the installation directory to <dir>.\n"
+	 "                                    | Should be set when installing ntopng \n"
+	 "                                    | under custom directories\n"
 	 "[--daemon|-e]                       | Daemonize ntopng\n"
 #endif
 	 "[--httpdocs-dir|-1] <path>          | HTTP documents root directory.\n"
@@ -176,18 +184,21 @@ void usage() {
 	 "[--callbacks-dir|-3] <path>         | Callbacks directory.\n"
 	 "                                    | Default: %s\n"
 	 "[--no-promisc|-u]                   | Don't set the interface in promiscuous mode.\n"
-	 "[--categorization-key|-c] <key>     | Key used to access host categorization\n"
-	 "                                    | services (default: disabled). \n"
-	 "                                    | Please read README.categorization for\n"
-	 "                                    | more info.\n"
-	 "[--httpbl-key|-k] <key>             | Key used to access httpbl\n"
-	 "                                    | services (default: disabled). \n"
-	 "                                    | Please read README.httpbl for\n"
-	 "                                    | more info.\n"
-	 "[--http-port|-w] <[:]http port>     | HTTP port. Set to 0 to disable http server.\n"
-	 "                                    | Prepend a : before the port to listen to the\n"
-	 "                                    | loopback address. Default: %u\n"
-	 "[--https-port|-W] <[:]https port>   | HTTPS port. See usage of -w above. Default: %u\n"
+	 "[--traffic-filtering|-k] <param>    | Filter traffic using cloud services.\n"
+	 "                                    | (default: disabled). Available options:\n"
+	 "                                    | httpbl:<api_key>        See README.httpbl\n"
+	 "[--http-port|-w] <[addr:]port>      | HTTP. Set to 0 to disable http server.\n"
+	 "                                    | Addr can be any valid ipv4 (e.g., 192.168.1.1)\n"
+	 "                                    | or ipv6 (e.g., [3ffe:2a00:100:7031::1]) address.\n"
+	 "                                    | Surround ipv6 addresses with square brackets.\n"
+	 "                                    | Prepend a ':' without addr before the port\n"
+	 "                                    | to listen on the loopback address.\n"
+	 "                                    | Default port: %u\n"
+	 "                                    | Examples:\n"
+	 "                                    | -w :3000\n"
+	 "                                    | -w 192.168.1.1:3001\n"
+	 "                                    | -w [3ffe:2a00:100:7031::1]:3002\n"
+	 "[--https-port|-W] <[:]https port>   | HTTPS. See usage of -w above. Default: %u\n"
 	 "[--local-networks|-m] <local nets>  | Local nets list (default: 192.168.1.0/24)\n"
 	 "                                    | (e.g. -m \"192.168.0.0/24,172.16.0.0/16\")\n"
 	 "[--ndpi-protocols|-p] <file>.protos | Specify a nDPI protocol file\n"
@@ -203,6 +214,10 @@ void usage() {
 	 "                                    | instead of %s\n"
 	 "[--dont-change-user|-s]             | Do not change user (debug only)\n"
 	 "[--shutdown-when-done]              | Terminate when a pcap has been read (debug only)\n"
+	 "[--zmq-collector-mode]              | Force ZMQ sockets to operate in collector mode. If\n"
+	 "                                    | used nprobe must use --zmq-probe-mode so that it can\n"
+	 "                                    | behave as a probe.\n"
+	 "--zmq-encrypt-pwd <pwd>             | Encrypt the ZMQ data using the specified password\n"
 	 "[--disable-autologout|-q]           | Disable web interface logout for inactivity\n"
 	 "[--disable-login|-l] <mode>         | Disable user login authentication:\n"
 	 "                                    | 0 - Disable login only for localhost\n"
@@ -236,6 +251,7 @@ void usage() {
 	 "                                    | all    - Dump all hosts\n"
 	 "                                    | local  - Dump only local hosts\n"
 	 "                                    | remote - Dump only remote hosts\n"
+	 "                                    | none   - Do not dump any host\n"
 	 "[--sticky-hosts|-S] <mode>          | Don't flush hosts (default: none).\n"
 	 "                                    | Values:\n"
 	 "                                    | all    - Keep all hosts in memory\n"
@@ -250,9 +266,13 @@ void usage() {
 	 "                                    |         hardware devices\n"
 	 "                                    | vss   - Timestamped packets by vssmonitoring.com\n"
 	 "                                    |         hardware devices\n"
+	 "--capture-direction                 | Specify packet capture direction\n"
+	 "                                    | 0=RX+TX (default), 1=RX only, 2=TX only\n"
+	 "--online-license-check              | Check license online\n"
 	 "[--enable-taps|-T]                  | Enable tap interfaces used to dump traffic\n"
 	 "[--http-prefix|-Z] <prefix>         | HTTP prefix to be prepended to URLs. This is\n"
 	 "                                    | useful when using ntopng behind a proxy.\n"
+	 "[--instance-name|-N] <name>         | Assign an identifier to the ntopng instance.\n"
 #ifdef NTOPNG_PRO
 	 "[--community]                       | Start ntopng in community edition (debug only).\n"
 	 "[--check-license]                   | Check if the license is valid.\n"
@@ -260,6 +280,7 @@ void usage() {
 #endif
 	 "[--verbose|-v]                      | Verbose tracing\n"
 	 "[--version|-V]                      | Print version and quit\n"
+	 "--print-ndpi-protocols              | Print the nDPI protocols recognized di ntopng\n"
 	 "[--help|-h]                         | Help\n"
 	 , PACKAGE_MACHINE, PACKAGE_VERSION,
 #ifndef WIN32
@@ -272,14 +293,11 @@ void usage() {
 	 CONST_DEFAULT_USERS_FILE);
 
   printf("\n");
+
+  NetworkInterface n("dummy");
   n.printAvailableInterfaces(true, 0, NULL, 0);
 
-  printf("\nnDPI detected protocols:\n");
-
-  struct ndpi_detection_module_struct *ndpi_struct = ndpi_init_detection_module(ntop->getGlobals()->get_detection_tick_resolution(), localmalloc, free, NULL);
-  ndpi_dump_protocols(ndpi_struct);
- 
-  _exit(0);
+  exit(0);
 }
 
 /* ******************************************* */
@@ -299,7 +317,7 @@ u_int32_t Prefs::getDefaultPrefsValue(const char *pref_key, u_int32_t default_va
 /* ******************************************* */
 
 void Prefs::getDefaultStringPrefsValue(const char *pref_key, char **buffer, const char *default_value) {
-  char rsp[32];
+  char rsp[MAX_PATH];
 
   if(ntop->getRedis()->get((char*)pref_key, rsp, sizeof(rsp)) == 0)
     *buffer = strdup(rsp);
@@ -314,36 +332,41 @@ void Prefs::loadIdleDefaults() {
   non_local_host_max_idle = getDefaultPrefsValue(CONST_REMOTE_HOST_IDLE_PREFS, MAX_REMOTE_HOST_IDLE);
   flow_max_idle = getDefaultPrefsValue(CONST_FLOW_MAX_IDLE_PREFS, MAX_FLOW_IDLE);
 
-  intf_rrd_raw_days   = getDefaultPrefsValue(CONST_INTF_RRD_RAW_DAYS, INTF_RRD_RAW_DAYS);
-  intf_rrd_1min_days  = getDefaultPrefsValue(CONST_INTF_RRD_1MIN_DAYS, INTF_RRD_1MIN_DAYS);
-  intf_rrd_1h_days    = getDefaultPrefsValue(CONST_INTF_RRD_1H_DAYS, INTF_RRD_1H_DAYS);
-  intf_rrd_1d_days    = getDefaultPrefsValue(CONST_INTF_RRD_1D_DAYS, INTF_RRD_1D_DAYS);
-  other_rrd_raw_days  = getDefaultPrefsValue(CONST_OTHER_RRD_RAW_DAYS, OTHER_RRD_RAW_DAYS);
-  other_rrd_1min_days = getDefaultPrefsValue(CONST_OTHER_RRD_1MIN_DAYS, OTHER_RRD_1MIN_DAYS);
-  other_rrd_1h_days   = getDefaultPrefsValue(CONST_OTHER_RRD_1H_DAYS, OTHER_RRD_1H_DAYS);
-  other_rrd_1d_days   = getDefaultPrefsValue(CONST_OTHER_RRD_1D_DAYS, OTHER_RRD_1D_DAYS);
+  intf_rrd_raw_days      = getDefaultPrefsValue(CONST_INTF_RRD_RAW_DAYS, INTF_RRD_RAW_DAYS);
+  intf_rrd_1min_days     = getDefaultPrefsValue(CONST_INTF_RRD_1MIN_DAYS, INTF_RRD_1MIN_DAYS);
+  intf_rrd_1h_days       = getDefaultPrefsValue(CONST_INTF_RRD_1H_DAYS, INTF_RRD_1H_DAYS);
+  intf_rrd_1d_days       = getDefaultPrefsValue(CONST_INTF_RRD_1D_DAYS, INTF_RRD_1D_DAYS);
+  other_rrd_raw_days     = getDefaultPrefsValue(CONST_OTHER_RRD_RAW_DAYS, OTHER_RRD_RAW_DAYS);
+  other_rrd_1min_days    = getDefaultPrefsValue(CONST_OTHER_RRD_1MIN_DAYS, OTHER_RRD_1MIN_DAYS);
+  other_rrd_1h_days      = getDefaultPrefsValue(CONST_OTHER_RRD_1H_DAYS, OTHER_RRD_1H_DAYS);
+  other_rrd_1d_days      = getDefaultPrefsValue(CONST_OTHER_RRD_1D_DAYS, OTHER_RRD_1D_DAYS);
+  housekeeping_frequency = getDefaultPrefsValue(CONST_RUNTIME_PREFS_HOUSEKEEPING_FREQUENCY,
+						HOUSEKEEPING_FREQUENCY);
+
+  // sets to the default value in redis if no key is found
+  getDefaultPrefsValue(CONST_RUNTIME_IS_AUTOLOGOUT_ENABLED, CONST_DEFAULT_IS_AUTOLOGOUT_ENABLED);
 }
 
 /* ******************************************* */
 
-#ifdef NTOPNG_PRO
+void Prefs::loadInstanceNameDefaults() {
+  // Do not re-set the interface name if it has already been set via command line
+  if(instance_name || !ntop)
+    return;
+  else {
+    char tmp[256];
 
-void Prefs::loadNagiosDefaults() {
-  if(nagios_host)  {  free(nagios_host); nagios_host = NULL; }
-  if(nagios_port)  {  free(nagios_port); nagios_port = NULL; }
-  if(nagios_config){  free(nagios_config); nagios_config = NULL; }
-
-  getDefaultStringPrefsValue(CONST_NAGIOS_HOST_PREFS, &nagios_host, CONST_DEFAULT_NAGIOS_HOST);
-  getDefaultStringPrefsValue(CONST_NAGIOS_PORT_PREFS, &nagios_port, CONST_DEFAULT_NAGIOS_PORT);
-  getDefaultStringPrefsValue(CONST_NAGIOS_CONF_PREFS, &nagios_config, CONST_DEFAULT_NAGIOS_CONF);
+    if(gethostname(tmp, sizeof(tmp)))
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to read hostname [%s]",
+				   strerror(errno));
+    else
+      instance_name = strdup(tmp);
+  }
 }
 
-#endif
-
 /* ******************************************* */
 
 static const struct option long_options[] = {
-  { "categorization-key",                required_argument, NULL, 'c' },
 #ifndef WIN32
   { "data-dir",                          required_argument, NULL, 'd' },
   { "install-dir",                       required_argument, NULL, 't' },
@@ -354,7 +377,7 @@ static const struct option long_options[] = {
   { "interface",                         required_argument, NULL, 'i' },
   { "local-networks",                    required_argument, NULL, 'm' },
   { "dns-mode",                          required_argument, NULL, 'n' },
-  { "httpbl-key",                        required_argument, NULL, 'k' },
+  { "traffic-filtering",                 required_argument, NULL, 'k' },
   { "disable-login",                     required_argument, NULL, 'l' },
   { "ndpi-protocols",                    required_argument, NULL, 'p' },
   { "disable-autologout",                no_argument,       NULL, 'q' },
@@ -373,6 +396,7 @@ static const struct option long_options[] = {
   { "disable-alerts",                    no_argument,       NULL, 'H' },
   { "export-flows",                      required_argument, NULL, 'I' },
   { "disable-host-persistency",          no_argument,       NULL, 'P' },
+  { "capture-direction",                 required_argument, NULL, 'Q' },
   { "sticky-hosts",                      required_argument, NULL, 'S' },
   { "enable-taps",                       no_argument,       NULL, 'T' },
   { "user",                              required_argument, NULL, 'U' },
@@ -380,17 +404,21 @@ static const struct option long_options[] = {
   { "max-num-flows",                     required_argument, NULL, 'X' },
   { "https-port",                        required_argument, NULL, 'W' },
   { "http-prefix",                       required_argument, NULL, 'Z' },
+  { "instance-name",                     required_argument, NULL, 'N' },
   { "httpdocs-dir",                      required_argument, NULL, '1' },
   { "scripts-dir",                       required_argument, NULL, '2' },
   { "callbacks-dir",                     required_argument, NULL, '3' },
+  { "print-ndpi-protocols",              no_argument,       NULL, 210 },
+  { "online-license-check",              no_argument,       NULL, 211 },
   { "hw-timestamp-mode",                 required_argument, NULL, 212 },
   { "shutdown-when-done",                no_argument,       NULL, 213 },
+  { "zmq-collector-mode",                no_argument,       NULL, 214 },
+  { "zmq-encrypt-pwd",                   required_argument, NULL, 215 },
 #ifdef NTOPNG_PRO
   { "check-maintenance",                 no_argument,       NULL, 252 },
   { "check-license",                     no_argument,       NULL, 253 },
   { "community",                         no_argument,       NULL, 254 },
 #endif
-
   /* End of options */
   { NULL,                                no_argument,       NULL,  0 }
 };
@@ -398,7 +426,42 @@ static const struct option long_options[] = {
 /* ******************************************* */
 
 int Prefs::setOption(int optkey, char *optarg) {
-  char *double_dot;
+  char *double_dot, *p, *opt = NULL;
+  int len = (optarg ? strlen(optarg) : 0)+6;
+
+  for(int i=0; ; i++) {
+    if(long_options[i].val == optkey) {
+      opt = (char*)long_options[i].name;
+      len += strlen(opt)+2;
+      break;
+    }
+  }
+
+  if((p = (char*)malloc(len)) != NULL) {
+    if(opt) {
+      if(optarg)
+	snprintf(p, len-1, "--%s \"%s\" ", opt, optarg);
+      else
+	snprintf(p, len-1, "--%s ", opt);      
+    } else {
+      if(optarg)
+	snprintf(p, len-1, "-%c %s ", optkey, optarg);
+      else
+	snprintf(p, len-1, "-%c ", optkey);
+    }
+
+    if(cli == NULL)
+      cli = p;
+    else {
+      int l = strlen(cli);
+      char *backup = cli;
+
+      if((cli = (char*)realloc(cli, l+len)) != NULL)
+	strcpy(&cli[l], p);
+      else
+	cli = backup;
+    }
+  }
 
   switch(optkey) {
   case 'B':
@@ -409,12 +472,22 @@ int Prefs::setOption(int optkey, char *optarg) {
       packet_filter = strdup(optarg);
     break;
 
-  case 'c':
-    categorization_key = optarg;
-    break;
-
   case 'k':
-    httpbl_key = optarg;
+    if(strncmp(optarg, HTTPBL_STRING, strlen(HTTPBL_STRING)) == 0)
+      httpbl_key = &optarg[strlen(HTTPBL_STRING)];
+    else if(strncmp(optarg, FLASHSTART_STRING, strlen(FLASHSTART_STRING)) == 0) {
+      char *user = strtok(&optarg[strlen(FLASHSTART_STRING)], ":"), *pwd = NULL;
+
+      if(user) pwd = strtok(NULL, ":");
+
+      if(user && pwd) {
+	if((flashstart = new Flashstart(user, pwd)) != NULL)
+	  ntop->set_flashstart(flashstart);
+      } else
+	ntop->getTrace()->traceEvent(TRACE_WARNING, "Invalid format for --%s<user>:<pwd>",
+				     FLASHSTART_STRING);
+    } else
+      ntop->getTrace()->traceEvent(TRACE_ERROR, "Unknown value %s for -k", optarg);
     break;
 
   case 'u':
@@ -477,7 +550,7 @@ int Prefs::setOption(int optkey, char *optarg) {
       disable_dns_responses_decoding();
       break;
     default:
-      help();
+      usage();
     }
     break;
 
@@ -490,6 +563,14 @@ int Prefs::setOption(int optkey, char *optarg) {
     enable_auto_logout = false;
     break;
 
+  case 'Q':
+    switch(atoi(optarg)) {
+    case 1:  setCaptureDirection(PCAP_D_IN);    break;
+    case 2:  setCaptureDirection(PCAP_D_OUT);   break;
+    default: setCaptureDirection(PCAP_D_INOUT); break;
+    }
+    break;
+
   case 'P':
     disable_host_persistency = true;
     break;
@@ -510,19 +591,39 @@ int Prefs::setOption(int optkey, char *optarg) {
     break;
 
   case 'w':
-    double_dot = strchr(optarg, ':');
-    if(double_dot)
-      http_port = atoi(&double_dot[1]), bind_http_to_loopback();
-    else
+    if (strchr(optarg, ':') == NULL){
+      // only the port
       http_port = atoi(optarg);
+    } else if (optarg[0] == ':'){
+      // first char == ':' binds to the loopback address
+      http_port = atoi(&optarg[1]);
+      bind_http_to_loopback();
+    } else {
+      // ':' is after the first character, so
+      // we need to parse both the ip address and the port
+      double_dot = strrchr(optarg, ':');
+      u_int len = double_dot - optarg;
+      http_binding_address = strndup(optarg, len);
+      http_port = atoi(&double_dot[1]);
+    }
     break;
 
   case 'W':
-    double_dot = strchr(optarg, ':');
-    if(double_dot)
-      https_port = atoi(&double_dot[1]), bind_https_to_loopback();
-    else
+    if (strchr(optarg, ':') == NULL){
+      // only the port
       https_port = atoi(optarg);
+    } else if (optarg[0] == ':'){
+      // first char == ':' binds to the loopback address
+      https_port = atoi(&optarg[1]);
+      bind_https_to_loopback();
+    } else {
+      // ':' is after the first character, so
+      // we need to parse both the ip address and the port
+      double_dot = strrchr(optarg, ':');
+      u_int len = double_dot - optarg;
+      https_binding_address = strndup(optarg, len);
+      https_port = atoi(&double_dot[1]);
+    }
     break;
 
   case 'Z':
@@ -542,6 +643,10 @@ int Prefs::setOption(int optkey, char *optarg) {
     }
     break;
 
+  case 'N':
+    instance_name = strndup(optarg, 256);
+    break;
+
   case 'r':
     {
       char buf[64], *r;
@@ -679,7 +784,9 @@ int Prefs::setOption(int optkey, char *optarg) {
 
       if(mysql_host && mysql_user) {
 	if((mysql_dbname == NULL) || (mysql_dbname[0] == '\0'))       mysql_dbname  = strdup("ntopng");
-	if((mysql_tablename == NULL) || (mysql_tablename[0] == '\0')) mysql_tablename  = strdup("flows");
+	if((mysql_tablename == NULL)
+	   || (mysql_tablename[0] == '\0')
+	   || 1 /*forcefully defaults the table name*/) mysql_tablename  = strdup("flows");
 	if((mysql_pw == NULL) || (mysql_pw[0] == '\0'))               mysql_pw  = strdup("");
 
 	dump_flows_on_mysql = true;
@@ -728,13 +835,21 @@ int Prefs::setOption(int optkey, char *optarg) {
     printf("System Id: %s\n", ntop->getPro()->get_system_id());
     printf("Built on:  %s\n", PACKAGE_OS);
 #endif
-    _exit(0);
+    exit(0);
     break;
 
   case 'X':
     max_num_flows = max_val(atoi(optarg), 1024);
     break;
 
+  case 210:
+    nDPIhelp();
+    break;
+
+  case 211:
+    online_license_check = true;
+    break;
+
   case 212:
     if(!strcmp(optarg, "ixia"))
       enable_ixia_timestamps = true;
@@ -749,13 +864,21 @@ int Prefs::setOption(int optkey, char *optarg) {
     shutdown_when_done = true;
     break;
 
+  case 214:
+    zmq_collector_mode = true;
+    break;
+
+  case 215:
+    zmq_encryption_pwd = strdup(optarg);
+    break;
+
 #ifdef NTOPNG_PRO
   case 252:
     /* Disable tracing messages */
     ntop->getTrace()->set_trace_level(0);
     ntop->registerPrefs(this, true);
     ntop->getPro()->check_maintenance_duration();
-    _exit(0);
+    exit(0);
     break;
 
   case 253:
@@ -763,7 +886,7 @@ int Prefs::setOption(int optkey, char *optarg) {
     ntop->getTrace()->set_trace_level(0);
     ntop->registerPrefs(this, true);
     ntop->getPro()->check_license_validity();
-    _exit(0);
+    exit(0);
     break;
 
   case 254:
@@ -817,6 +940,10 @@ int Prefs::checkOptions() {
   ntop->removeTrailingSlash(scripts_dir);
   ntop->removeTrailingSlash(callbacks_dir);
 
+  if(http_binding_address == NULL)
+    http_binding_address = strdup((char*)CONST_ANY_ADDRESS);
+  if(https_binding_address == NULL)
+    https_binding_address = strdup((char*)CONST_ANY_ADDRESS);
   return(0);
 }
 
@@ -826,7 +953,7 @@ int Prefs::loadFromCLI(int argc, char *argv[]) {
   u_char c;
 
   while((c = getopt_long(argc, argv,
-			 "c:k:eg:hi:w:r:sg:m:n:p:qd:t:x:1:2:3:l:uvA:B:CD:E:F:G:HI:O:S:TU:X:W:VZ:",
+			 "k:eg:hi:w:r:sg:m:n:p:qd:t:x:1:2:3:l:uvA:B:CD:E:F:N:G:HI:O:Q:S:TU:X:W:VZ:",
 			 long_options, NULL)) != '?') {
     if(c == 255) break;
     setOption(c, optarg);
@@ -834,7 +961,7 @@ int Prefs::loadFromCLI(int argc, char *argv[]) {
 
   if((http_port == 0) && (https_port == 0)) {
     ntop->getTrace()->traceEvent(TRACE_ERROR, "Both HTTP and HTTPS ports are disabled: quitting");
-    _exit(0);
+    exit(0);
   }
 
   return(checkOptions());
@@ -904,7 +1031,7 @@ void Prefs::add_network_interface(char *name, char *description) {
   int id = Utils::ifname2id(name);
 
   if(id < (MAX_NUM_INTERFACES-1)) {
-    ifNames[id].name = strdup(name);
+    ifNames[id].name = strdup(!strncmp(name, "-", 1) ? "stdin" : name);
     ifNames[id].description = strdup(description ? description : name);
     num_interfaces++;
   } else {
@@ -945,8 +1072,9 @@ void Prefs::lua(lua_State* vm) {
 
   lua_push_bool_table_entry(vm, "is_dns_resolution_enabled_for_all_hosts", resolve_all_host_ip);
   lua_push_bool_table_entry(vm, "is_dns_resolution_enabled", enable_dns_resolution);
-  lua_push_bool_table_entry(vm, "is_categorization_enabled", categorization_enabled);
-  lua_push_bool_table_entry(vm, "is_httpbl_enabled", httpbl_enabled);
+  lua_push_bool_table_entry(vm, "is_categorization_enabled", flashstart ? true : false);
+  lua_push_bool_table_entry(vm, "is_httpbl_enabled", is_httpbl_enabled());
+  lua_push_bool_table_entry(vm, "is_autologout_enabled", enable_auto_logout);
   lua_push_int_table_entry(vm, "http_port", http_port);
   lua_push_int_table_entry(vm, "local_host_max_idle", local_host_max_idle);
   lua_push_int_table_entry(vm, "non_local_host_max_idle", non_local_host_max_idle);
@@ -954,6 +1082,7 @@ void Prefs::lua(lua_State* vm) {
   lua_push_int_table_entry(vm, "max_num_hosts", max_num_hosts);
   lua_push_int_table_entry(vm, "max_num_flows", max_num_flows);
   lua_push_bool_table_entry(vm, "is_dump_flows_enabled", dump_flows_on_es || dump_flows_on_mysql);
+  lua_push_bool_table_entry(vm, "is_dump_flows_to_mysql_enabled", dump_flows_on_mysql);
   lua_push_int_table_entry(vm, "dump_hosts", dump_hosts_to_db);
 
   /* RRD prefs */
@@ -965,11 +1094,12 @@ void Prefs::lua(lua_State* vm) {
   lua_push_int_table_entry(vm, "other_rrd_1min_days", other_rrd_1min_days);
   lua_push_int_table_entry(vm, "other_rrd_1h_days", other_rrd_1h_days);
   lua_push_int_table_entry(vm, "other_rrd_1d_days", other_rrd_1d_days);
+  lua_push_int_table_entry(vm, "housekeeping_frequency", housekeeping_frequency);
 
+  lua_push_str_table_entry(vm, "instance_name", instance_name ? instance_name : (char*)"");
+  
 #ifdef NTOPNG_PRO
-  lua_push_str_table_entry(vm, "nagios_host", nagios_host);
-  lua_push_str_table_entry(vm, "nagios_port", nagios_port);
-  lua_push_str_table_entry(vm, "nagios_config", nagios_config);
+  if(ntop->getNagios()) ntop->getNagios()->lua(vm);
 
   memset(HTTP_stats_base_dir, '\0', MAX_PATH);
   strncat(HTTP_stats_base_dir, (const char*)ntop->get_working_dir(), MAX_PATH);
@@ -982,6 +1112,65 @@ void Prefs::lua(lua_State* vm) {
 
 bool Prefs::isView(char *name) { return(strncmp(name, "view:", 5) == 0 ? true : false); }
 
+
+/* *************************************** */
+
+int Prefs::refresh(const char *pref_name, const char *pref_value) {
+  if (!pref_name || !pref_value)
+    return -1;
+
+  if (!strncmp(pref_name,
+	       (char*)CONST_RUNTIME_PREFS_HOUSEKEEPING_FREQUENCY,
+	       strlen((char*)CONST_RUNTIME_PREFS_HOUSEKEEPING_FREQUENCY)))
+    housekeeping_frequency = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_LOCAL_HOST_IDLE_PREFS,
+		    strlen((char*)CONST_LOCAL_HOST_IDLE_PREFS)))
+    local_host_max_idle = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_REMOTE_HOST_IDLE_PREFS,
+		    strlen((char*)CONST_REMOTE_HOST_IDLE_PREFS)))
+    non_local_host_max_idle = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_FLOW_MAX_IDLE_PREFS,
+		    strlen((char*)CONST_FLOW_MAX_IDLE_PREFS)))
+    flow_max_idle = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_INTF_RRD_RAW_DAYS,
+		    strlen((char*)CONST_INTF_RRD_RAW_DAYS)))
+    intf_rrd_raw_days = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_INTF_RRD_1MIN_DAYS,
+		    strlen((char*)CONST_INTF_RRD_1MIN_DAYS)))
+    intf_rrd_1min_days = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_INTF_RRD_1H_DAYS,
+		    strlen((char*)CONST_INTF_RRD_1H_DAYS)))
+    intf_rrd_1h_days = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_INTF_RRD_1D_DAYS,
+		    strlen((char*)CONST_INTF_RRD_1D_DAYS)))
+    intf_rrd_1d_days = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_OTHER_RRD_RAW_DAYS,
+		    strlen((char*)CONST_OTHER_RRD_RAW_DAYS)))
+    other_rrd_raw_days = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_OTHER_RRD_1MIN_DAYS,
+		    strlen((char*)CONST_OTHER_RRD_1MIN_DAYS)))
+    other_rrd_1min_days = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_OTHER_RRD_1H_DAYS,
+		    strlen((char*)CONST_OTHER_RRD_1H_DAYS)))
+    other_rrd_1h_days = atoi(pref_value);
+  else if (!strncmp(pref_name,
+		    (char*)CONST_OTHER_RRD_1D_DAYS,
+		    strlen((char*)CONST_OTHER_RRD_1D_DAYS)))
+    other_rrd_1d_days = atoi(pref_value);
+
+  return 0;
+}
+
 /* *************************************** */
 
 void Prefs::registerNetworkInterfaces() {
