@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -27,14 +27,6 @@
 #include "third-party/hiredis/sds.c"
 #endif
 
-/* **************************************************** */
-
-static void* esLoop(void* ptr) {
-    ntop->getRedis()->pushEStemplate();  // sends ES ntopng template
-    ntop->getRedis()->indexESdata();
-  return(NULL);
-}
-
 /* **************************************** */
 
 Redis::Redis(char *_redis_host, u_int16_t _redis_port, u_int8_t _redis_db_id) {
@@ -54,12 +46,6 @@ Redis::~Redis() {
   delete l;
 }
 
-/* **************************************** */
-
-void Redis::startFlowDump() {
-  if(ntop->getPrefs()->do_dump_flows_on_es())
-    pthread_create(&esThreadLoop, NULL, esLoop, (void*)this);
-}
 
 /* **************************************** */
 
@@ -151,6 +137,27 @@ int Redis::get(char *key, char *rsp, u_int rsp_len) {
 
   return(rc);
 }
+/* **************************************** */
+
+int Redis::del(char *key){
+  int rc;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  reply = (redisReply*)redisCommand(redis, "DEL %s", key);
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR)){
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+    rc = -1;
+  } else {
+    rc = 0;
+  }
+
+  if(reply) freeReplyObject(reply);
+  l->unlock(__FILE__, __LINE__);
+
+  return(rc);
+}
 
 /* **************************************** */
 
@@ -292,24 +299,64 @@ u_int32_t Redis::incrKey(char *key) {
 /*
   Increment key.member of +value and keeps at most trim_len elements
 */
-int Redis::zincrbyAndTrim(char *key, char *member, u_int value, u_int trim_len) {
+int Redis::zIncr(char *key, char *member) {
   int rc;
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
-  reply = (redisReply*)redisCommand(redis, "ZINCRBY %s %u", key, value);
+  reply = (redisReply*)redisCommand(redis, "ZINCRBY %s 1 %s", key, member);
   if(!reply) reconnectRedis();
   if(reply && (reply->type == REDIS_REPLY_ERROR))
     ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
   if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
+  l->unlock(__FILE__, __LINE__);
 
-  if((rc == 0) && (trim_len > 0)) {
-    reply = (redisReply*)redisCommand(redis, "ZREMRANGEBYRANK %s 0 %u", key, -1*trim_len);
-    if(!reply) reconnectRedis();
-    if(reply && (reply->type == REDIS_REPLY_ERROR))
-      ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-    if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
+  return(rc);
+}
+
+/* **************************************** */
+
+/*
+  Increment key.member of +value and keeps at most trim_len elements
+*/
+int Redis::zTrim(char *key, u_int trim_len) {
+  int rc;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  reply = (redisReply*)redisCommand(redis, "ZREMRANGEBYRANK %s 0 %d", key, -1*trim_len);
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+  if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
+  l->unlock(__FILE__, __LINE__);
+
+  return(rc);
+}
+
+/* **************************************** */
+
+int Redis::zRevRange(const char *pattern, char ***keys_p) {
+  int rc = 0;
+  u_int i;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  reply = (redisReply*)redisCommand(redis, "ZREVRANGE %s 0 -1 WITHSCORES", pattern);
+  if(!reply) reconnectRedis();
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+
+  if(reply && (reply->type == REDIS_REPLY_ARRAY)) {
+    (*keys_p) = (char**) malloc(reply->elements * sizeof(char*));
+    rc = (int)reply->elements;
+
+    for(i = 0; i < reply->elements; i++) {
+      (*keys_p)[i] = strdup(reply->element[i]->str);
+    }
   }
+
+  if(reply) freeReplyObject(reply);
   l->unlock(__FILE__, __LINE__);
 
   return(rc);
@@ -378,12 +425,12 @@ int Redis::hashKeys(const char *pattern, char ***keys_p) {
 
 /* **************************************** */
 
-int Redis::del(char *key) {
+int Redis::oneOperator(const char *operation, char *key) {
   int rc;
   redisReply *reply;
 
   l->lock(__FILE__, __LINE__);
-  reply = (redisReply*)redisCommand(redis, "DEL %s", key);
+  reply = (redisReply*)redisCommand(redis, "%s %s", operation, key);
   if(!reply) reconnectRedis();
 
   if(reply && (reply->type == REDIS_REPLY_ERROR))
@@ -397,10 +444,33 @@ int Redis::del(char *key) {
 
 /* **************************************** */
 
-int Redis::pushHostToHTTPBL(char *hostname, bool dont_check_for_existance, bool localHost) {
-  if(!ntop->getPrefs()->is_httpbl_enabled()) return(0);
-  if(hostname == NULL) return(-1);
-  return(pushHost(HTTPBL_CACHE, HTTPBL_TO_RESOLVE, hostname, dont_check_for_existance, localHost));
+int Redis::twoOperators(const char *operation, char *op1, char *op2) {
+  int rc;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  reply = (redisReply*)redisCommand(redis, "%s %s %s", operation, op1, op2);
+  if(!reply) reconnectRedis();
+
+  if(reply
+     && (reply->type == REDIS_REPLY_ERROR)
+     && strcmp(reply->str, "ERR no such key"))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+
+  if(reply) freeReplyObject(reply), rc = 0; else rc = -1;
+  l->unlock(__FILE__, __LINE__);
+
+  return(rc);
+}
+
+/* **************************************** */
+
+int Redis::pushHostToTrafficFiltering(char *hostname, bool dont_check_for_existance, bool localHost) {
+  if(ntop->getPrefs()->is_httpbl_enabled()) {
+    if(hostname == NULL) return(-1);
+    return(pushHost(TRAFFIC_FILTERING_CACHE, TRAFFIC_FILTERING_TO_RESOLVE, hostname, dont_check_for_existance, localHost));
+  } else
+    return(0);
 }
 
 /* **************************************** */
@@ -467,8 +537,8 @@ int Redis::pushHost(const char* ns_cache, const char* ns_list, char *hostname,
 
 /* **************************************** */
 
-int Redis::popHostToHTTPBL(char *hostname, u_int hostname_len) {
-  return(popHost(HTTPBL_TO_RESOLVE, hostname, hostname_len));
+int Redis::popHostToTrafficFiltering(char *hostname, u_int hostname_len) {
+  return(popHost(TRAFFIC_FILTERING_TO_RESOLVE, hostname, hostname_len));
 }
 
 /* **************************************** */
@@ -485,18 +555,19 @@ int Redis::popHost(const char* ns_list, char *hostname, u_int hostname_len) {
 
 /* **************************************** */
 
-char* Redis::getHTTPBLCategory(char *numeric_ip, char *buf,
-			       u_int buf_len, bool categorize_if_unknown) {
+char* Redis::getTrafficFilteringCategory(char *numeric_ip, char *buf,
+					 u_int buf_len, bool categorize_if_unknown) {
   char key[CONST_MAX_LEN_REDIS_KEY];
   redisReply *reply;
 
+  if(!ntop->getPrefs()->is_httpbl_enabled()
+     && (!ntop->getPrefs()->is_flashstart_enabled()))
+    return(NULL);
+
   buf[0] = '\0';
-
-  if(!ntop->getPrefs()->is_httpbl_enabled())  return(NULL);
-
   l->lock(__FILE__, __LINE__);
 
-  snprintf(key, sizeof(key), "%s.%s", HTTPBL_CACHE, numeric_ip);
+  snprintf(key, sizeof(key), "%s.%s", TRAFFIC_FILTERING_CACHE, numeric_ip);
 
   /*
     Add only if the ip has not been checked against the blacklist
@@ -513,61 +584,7 @@ char* Redis::getHTTPBLCategory(char *numeric_ip, char *buf,
     buf[0] = '\0';
 
     if(categorize_if_unknown) {
-      reply = (redisReply*)redisCommand(redis, "RPUSH %s %s", HTTPBL_TO_RESOLVE, numeric_ip);
-      if(!reply) reconnectRedis();
-      if(reply && (reply->type == REDIS_REPLY_ERROR))
-	ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-      if(reply) freeReplyObject(reply);
-    }
-  }
-
-  l->unlock(__FILE__, __LINE__);
-
-  return(buf);
-}
-
-/* **************************************** */
-
-char* Redis::getFlowCategory(char *_domainname, char *buf,
-			     u_int buf_len, bool categorize_if_unknown) {
-  char key[CONST_MAX_LEN_REDIS_KEY], *domainname;
-  redisReply *reply;
-
-  buf[0] = 0;
-
-  if(!ntop->getPrefs()->is_categorization_enabled())  return(NULL);
-
-  domainname = Utils::get2ndLevelDomain(_domainname);
-
-  /* Check if the host is 'categorizable' */
-  if(Utils::isIPAddress(domainname)
-     || (!Utils::isGoodNameToCategorize(domainname))) {
-    return(buf);
-  }
-
-  if(hashGet((char*)DOMAIN_WHITELIST_CAT, domainname, key, sizeof(key)) == 0)
-    return(buf); /* Whitelisted domain */
-
-  l->lock(__FILE__, __LINE__);
-
-  snprintf(key, sizeof(key), "%s.%s", DOMAIN_CATEGORY, domainname);
-
-  /*
-    Add only if the domain has not been categorized yet
-  */
-  reply = (redisReply*)redisCommand(redis, "GET %s", key);
-  if(!reply) reconnectRedis();
-  if(reply && (reply->type == REDIS_REPLY_ERROR))
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
-
-  if(reply && reply->str) {
-    snprintf(buf, buf_len, "%s", reply->str);
-    freeReplyObject(reply);
-  } else {
-    buf[0] = 0;
-
-    if(categorize_if_unknown) {
-      reply = (redisReply*)redisCommand(redis, "RPUSH %s %s", DOMAIN_TO_CATEGORIZE, domainname);
+      reply = (redisReply*)redisCommand(redis, "RPUSH %s %s", TRAFFIC_FILTERING_TO_RESOLVE, numeric_ip);
       if(!reply) reconnectRedis();
       if(reply && (reply->type == REDIS_REPLY_ERROR))
 	ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
@@ -582,9 +599,11 @@ char* Redis::getFlowCategory(char *_domainname, char *buf,
 
 /* **************************************** */
 
+#ifdef NOTUSED
 int Redis::popDomainToCategorize(char *domainname, u_int domainname_len) {
   return(lpop(DOMAIN_TO_CATEGORIZE, domainname, domainname_len));
 }
+#endif
 
 /* **************************************** */
 
@@ -606,15 +625,15 @@ void Redis::setDefaults() {
 
 /* **************************************** */
 
-int Redis::getAddressHTTPBL(char *numeric_ip,
-			    NetworkInterface *iface,
-			    char *rsp, u_int rsp_len,
-			    bool queue_if_not_found) {
+int Redis::getAddressTrafficFiltering(char *numeric_ip,
+				      NetworkInterface *iface,
+				      char *rsp, u_int rsp_len,
+				      bool queue_if_not_found) {
   char key[CONST_MAX_LEN_REDIS_KEY];
   int rc;
 
   rsp[0] = '\0';
-  snprintf(key, sizeof(key), "%s.%s", HTTPBL_CACHE, numeric_ip);
+  snprintf(key, sizeof(key), "%s.%s", TRAFFIC_FILTERING_CACHE, numeric_ip);
 
   rc = get(key, rsp, rsp_len);
 
@@ -623,12 +642,12 @@ int Redis::getAddressHTTPBL(char *numeric_ip,
       char buf[64];
 
       snprintf(buf, sizeof(buf), "%s@%s", numeric_ip, iface->get_name());
-      pushHostToHTTPBL(buf, true, false);
+      pushHostToTrafficFiltering(buf, true, false);
     }
   } else {
     /* We need to extend expire */
 
-    expire(numeric_ip, HTTPBL_CACHE_DURATIION /* expire */);
+    expire(numeric_ip, TRAFFIC_FILTERING_CACHE_DURATIION /* expire */);
   }
 
   return(rc);
@@ -660,11 +679,11 @@ int Redis::getAddress(char *numeric_ip, char *rsp,
 
 /* **************************************** */
 
-int Redis::setHTTPBLAddress(char *numeric_ip, char *httpbl) {
+int Redis::setTrafficFilteringAddress(char *numeric_ip, char *httpbl) {
   char key[CONST_MAX_LEN_REDIS_KEY];
 
-  snprintf(key, sizeof(key), "%s.%s", HTTPBL_CACHE, numeric_ip);
-  return(set(key, httpbl, HTTPBL_CACHE_DURATIION));
+  snprintf(key, sizeof(key), "%s.%s", TRAFFIC_FILTERING_CACHE, numeric_ip);
+  return(set(key, httpbl, TRAFFIC_FILTERING_CACHE_DURATIION));
 }
 
 /* **************************************** */
@@ -729,6 +748,7 @@ char* Redis::getVersion(char *str, u_int str_len) {
 
 /* **************************************** */
 
+#ifdef NOTUSED
 int Redis::hashIncr(char *key, char *field, u_int32_t value) {
   int rc;
   redisReply *reply;
@@ -745,6 +765,7 @@ int Redis::hashIncr(char *key, char *field, u_int32_t value) {
 
   return(rc);
 }
+#endif
 
 /* **************************************** */
 
@@ -926,31 +947,26 @@ int Redis::msg_push(const char *cmd, const char *queue_name, char *msg, u_int qu
 
 /* ******************************************* */
 
-void Redis::queueAlert(AlertLevel level, AlertType t, char *msg) {
+void Redis::queueAlert(AlertLevel level, AlertStatus s, AlertType t, char *msg) {
   char what[1024];
 
   if(ntop->getPrefs()->are_alerts_disabled()) return;
 
-  snprintf(what, sizeof(what), "%u|%u|%u|%s",
+  snprintf(what, sizeof(what), "%u|%u|%u|%u|%s",
 	   (unsigned int)time(NULL), (unsigned int)level,
-	   (unsigned int)t, msg);
+	   (unsigned int)s, (unsigned int)t, msg);
 
 #ifndef WIN32
   // Print alerts into syslog
-  if(ntop->getRuntimePrefs()->are_alerts_syslog_enable()) {
-    if( alert_level_info == level) syslog(LOG_INFO, "%s", what);
-    else if( alert_level_warning == level) syslog(LOG_WARNING, "%s", what);
-    else if( alert_level_error == level) syslog(LOG_ALERT, "%s", what);
+  if(ntop->getRuntimePrefs()->are_alerts_syslog_enabled()) {
+    if(alert_level_info == level) syslog(LOG_INFO, "%s", what);
+    else if(alert_level_warning == level) syslog(LOG_WARNING, "%s", what);
+    else if(alert_level_error == level) syslog(LOG_ALERT, "%s", what);
   }
 #endif
 
   lpush(CONST_ALERT_MSG_QUEUE, what, CONST_MAX_ALERT_MSG_QUEUE_LEN);
 
-#ifdef NTOPNG_PRO
-  if(ntop->getNagios())
-    ntop->getNagios()->sendEvent(level, t, msg);
-#endif
-
 }
 
 /* ******************************************* */
@@ -997,6 +1013,81 @@ int Redis::lpop(const char *queue_name, char *buf, u_int buf_len) {
 
 /* ******************************************* */
 
+int Redis::lindex(const char *queue_name, int idx, char *buf, u_int buf_len) {
+  int rc;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+  reply = (redisReply*)redisCommand(redis, "LINDEX %s %d", queue_name, idx);
+
+  if(!reply) reconnectRedis();
+
+  if(reply && (reply->type == REDIS_REPLY_ERROR))
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+
+  if(reply && reply->str)
+    snprintf(buf, buf_len, "%s", reply->str), rc = 0;
+  else
+    buf[0] = '\0', rc = -1;
+
+  if(reply) freeReplyObject(reply);
+  l->unlock(__FILE__, __LINE__);
+
+  return(rc);
+}
+
+/* ******************************************* */
+
+int Redis::lpop(const char *queue_name, char ***elements, u_int num_elements) {
+  int rc;
+  redisReply *reply;
+
+  l->lock(__FILE__, __LINE__);
+
+  // make a redis pipeline that pops multile elements
+  // with just 1 redis command (so we pay only one RTT
+  // and the operation is atomic)
+  /*
+  reply = (redisReply*)redisCommand(redis,
+				    "LRANGE %s -%u -1 \r\n LTRIM %s 0 -%u \r\n",
+				    queue_name, num_elements, queue_name, num_elements + 1);
+  */
+  redisAppendCommand(redis, "LRANGE %s -%u -1", queue_name, num_elements);
+  redisAppendCommand(redis, "LTRIM %s 0 -%u",   queue_name, num_elements + 1);
+
+  redisGetReply(redis, (void**)&reply);  // reply for LRANGE
+  if(!reply) reconnectRedis();
+
+  if(reply && (reply->type == REDIS_REPLY_ERROR)){
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "%s", reply->str ? reply->str : "???");
+    rc = -1;
+
+  } else if(reply && (reply->type == REDIS_REPLY_ARRAY)) {
+    (*elements) = (char**) calloc(reply->elements, sizeof(char*));
+    rc = (int)reply->elements;
+
+    int i = rc - 1, j = 0;
+    for(; i >= 0; i--, j++) {
+      (*elements)[j] = strdup(reply->element[i]->str);
+    }
+
+  } else
+    rc = -1;
+
+  if(reply) freeReplyObject(reply);
+  // empty also the second reply for the LTRIM
+  redisGetReply(redis, (void**)&reply);  // reply for LTRIM
+  if(!reply)
+    reconnectRedis();
+  else
+    freeReplyObject(reply);
+
+  l->unlock(__FILE__, __LINE__);
+  return(rc);
+}
+
+/* ******************************************* */
+
 void Redis::deleteQueuedAlert(u_int32_t idx_to_delete) {
   redisReply *reply;
 
@@ -1046,113 +1137,4 @@ u_int Redis::getQueuedAlerts(patricia_tree_t *allowed_hosts, char **alerts, u_in
   return(i);
 }
 
-/* **************************************** */
 
-void Redis::indexESdata() {
-  const u_int watermark = 8, min_buf_size = 512;
-  char postbuf[16384];
-
-  while(!ntop->getGlobals()->isShutdown()) {
-    u_int l = llen(CONST_ES_QUEUE_NAME);
-
-    if(l >= watermark) {
-      u_int len, num_flows;
-      char index_name[64], header[256];
-      struct tm* tm_info;
-      struct timeval tv;
-      time_t t;
-
-      gettimeofday(&tv, NULL);
-      t = tv.tv_sec;
-      tm_info = gmtime(&t);
-
-      strftime(index_name, sizeof(index_name), ntop->getPrefs()->get_es_index(), tm_info);
-
-      snprintf(header, sizeof(header),
-	       "{\"index\": {\"_type\": \"%s\", \"_index\": \"%s\"}}",
-	       ntop->getPrefs()->get_es_type(), index_name);
-      len = 0, num_flows = 0;
-
-      for(u_int i=0; (i<watermark) && ((sizeof(postbuf)-len) > min_buf_size); i++) {
-	char rsp[4096];
-	int rc = lpop(CONST_ES_QUEUE_NAME, rsp, sizeof(rsp));
-
-	if(rc >= 0) {
-	  // ntop->getTrace()->traceEvent(TRACE_NORMAL, "%s", rsp);
-	  len += snprintf(&postbuf[len], sizeof(postbuf)-len, "%s\n%s\n", header, rsp), num_flows++;
-	} else
-	  break;
-      } /* for */
-
-      postbuf[len] = '\0';
-
-      if(!Utils::postHTTPJsonData(ntop->getPrefs()->get_es_user(),
-				  ntop->getPrefs()->get_es_pwd(),
-				  ntop->getPrefs()->get_es_url(),
-				  postbuf)) {
-	/* Post failure */
-	sleep(1);
-      } else
-	ntop->getTrace()->traceEvent(TRACE_INFO, "Sent %u flow(s) to ES", num_flows);
-    } else
-      sleep(1);
-  } /* while */
-}
-
-/* **************************************** */
-
-/* Send ntopng index template to Elastic Search */
-void Redis::pushEStemplate() {
-  char *postbuf = NULL, *es_host = NULL;
-  char template_path[MAX_PATH], es_template_url[MAX_PATH], es_url[MAX_PATH];
-  ifstream template_file;
-  u_int8_t max_attempts = 3;
-  u_int16_t length = 0;
-
-  // store the original es update url
-  strncpy(es_url, ntop->getPrefs()->get_es_url(), MAX_PATH);
-  // prepare the template file path...
-  snprintf(template_path, sizeof(template_path), "%s/misc/%s",
-	   ntop->get_docs_dir(), NTOP_ES_TEMPLATE);
-  ntop->fixPath(template_path);
-
-  // and the ES url (keep only host and port by retaining only characters left of the first slash)
-  if(!strncmp(es_url, "http://", 7)){  // url starts either with http or https
-    Utils::tokenizer(es_url + 7, '/', &es_host);
-    snprintf(es_template_url, MAX_PATH, "http://%s/_template/ntopng_template", es_host);
-  } else if(!strncmp(es_url, "https://", 8)){
-    Utils::tokenizer(es_url + 8, '/', &es_host);
-    snprintf(es_template_url, MAX_PATH, "https://%s/_template/ntopng_template", es_host);
-  } else {
-    Utils::tokenizer(es_url, '/', &es_host);
-    snprintf(es_template_url, MAX_PATH, "%s/_template/ntopng_template", es_host);
-  }
-
-  template_file.open(template_path);   // open input file
-  template_file.seekg(0, ios::end);    // go to the end
-  length = template_file.tellg();      // report location (this is the length)
-  template_file.seekg(0, ios::beg);    // go back to the beginning
-  postbuf = new char[length+1];        // allocate memory for a buffer of appropriate dimension
-  template_file.read(postbuf, length); // read the whole file into the buffer
-  postbuf[length] = '\0';
-  if(template_file.is_open())
-    template_file.close();           // close file handle
-
-  while(max_attempts > 0) {
-    if(!Utils::postHTTPJsonData(ntop->getPrefs()->get_es_user(),
-				ntop->getPrefs()->get_es_pwd(),
-				es_template_url,
-				postbuf)) {
-      /* Post failure */
-      sleep(1);
-    } else {
-      ntop->getTrace()->traceEvent(TRACE_INFO, "ntopng template successfully sent to ES");
-      if(postbuf) free(postbuf);
-      break;
-    }
-    max_attempts--;
-  } /* while */
-
-  if(max_attempts == 0)
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to send ntopng template (%s) to ES", template_path);
-}
