@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -24,11 +24,6 @@
 #include <curl/curl.h>
 #include <string.h>
 
-extern "C" {
-#include "../third-party/detectxsslib/detectxsslib.h"
-#include "../third-party/detectxsslib/detectxsslib.c"
-};
-
 // A simple struct for strings.
 typedef struct {
   char *s;
@@ -112,12 +107,15 @@ char* Utils::l4proto2name(u_int8_t proto) {
   case 2:   return((char*)"IGMP");
   case 6:   return((char*)"TCP");
   case 17:  return((char*)"UDP");
+  case 41:  return((char*)"IPv6");
+  case 46:  return((char*)"RSVP");
   case 47:  return((char*)"GRE");
   case 50:  return((char*)"ESP");
   case 58:  return((char*)"IPv6-ICMP");
   case 89:  return((char*)"OSPF");
   case 103: return((char*)"PIM");
   case 112: return((char*)"VRRP");
+  case 139: return((char*)"HIP");
 
   default:
     snprintf(proto_string, sizeof(proto_string), "%u", proto);
@@ -127,6 +125,7 @@ char* Utils::l4proto2name(u_int8_t proto) {
 
 /* ****************************************************** */
 
+#ifdef NOTUSED
 bool Utils::isIPAddress(char *ip) {
   struct in_addr addr4;
   struct in6_addr addr6;
@@ -144,6 +143,7 @@ bool Utils::isIPAddress(char *ip) {
 
   return(false);
 }
+#endif
 
 /* ****************************************************** */
 
@@ -204,6 +204,12 @@ float Utils::timeval2ms(struct timeval *tv) {
 
 /* ****************************************************** */
 
+u_int32_t Utils::timeval2usec(const struct timeval *tv) {
+  return(tv->tv_sec*1000000+tv->tv_usec);
+}
+
+/* ****************************************************** */
+
 bool Utils::file_exists(const char *path) {
   std::ifstream infile(path);
 
@@ -254,6 +260,37 @@ bool Utils::mkdir_tree(char *path) {
 
 /* **************************************************** */
 
+const char* Utils::flowstatus2str(FlowStatus s) {
+  switch(s) {
+  case 0:
+    return("Normal");
+    break;
+  case 1:
+    return("Slow TCP Connection");
+    break;
+  case 2:
+    return("Slow Application Header");
+    break;
+  case 3:
+    return("Slow Data Exchange (Slowloris?)");
+    break;
+  case 4:
+    return("Low Goodput");
+    break;
+  case 5:
+    return("Suspicious TCP Probing (or server port down)");
+    break;
+  case 6:
+    return("TCP Connection Reset");
+    break;
+  default:
+    return("Unknown status");
+    break;
+  }
+}
+
+/* **************************************************** */
+
 const char* Utils::trend2str(ValueTrend t) {
   switch(t) {
   case trend_up:
@@ -326,6 +363,7 @@ static inline bool is_base64(unsigned char c) {
 
 /* **************************************************** */
 
+#ifdef NOTUSED
 std::string Utils::base64_encode(unsigned char const* bytes_to_encode, unsigned int in_len) {
   std::string ret;
   int i = 0;
@@ -365,6 +403,7 @@ std::string Utils::base64_encode(unsigned char const* bytes_to_encode, unsigned
 
   return ret;
 }
+#endif
 
 /* **************************************************** */
 
@@ -507,7 +546,9 @@ const char *strcasestr(const char *haystack, const char *needle) {
 int Utils::ifname2id(const char *name) {
   char rsp[256];
 
-  if(name == NULL || !strncmp(name, "dummy", 5)) return(DUMMY_IFACE_ID);
+  if(name == NULL)                    return(-1);
+  else if(!strncmp(name, "dummy", 5)) return(DUMMY_IFACE_ID);
+  else if(!strncmp(name, "stdin", 5) || !strncmp(name, "-", 1)) return(STDIN_IFACE_ID);
 
   if(ntop->getRedis()->hashGet((char*)CONST_IFACE_ID_PREFS, (char*)name, rsp, sizeof(rsp)) == 0) {
     /* Found */
@@ -560,6 +601,38 @@ char* Utils::sanitizeHostName(char *str) {
 
 /* **************************************************** */
 
+char* Utils::stripHTML(const char * const str) {
+    if (!str) return NULL;
+    int len = strlen(str), j = 0;
+    char *stripped_str = NULL;
+    try {
+        stripped_str = new char[len + 1];
+    } catch(std::bad_alloc& ba) {
+      static bool oom_warning_sent = false;
+      if(!oom_warning_sent) {
+	ntop->getTrace()->traceEvent(TRACE_WARNING, "Not enough memory");
+	oom_warning_sent = true;
+      }
+      return NULL;
+    }
+
+    // scan string
+    for (int i = 0; i < len; i++) {
+        // found an open '<', scan for its close
+        if (str[i] == '<') {
+            // charge ahead in the string until it runs out or we find what we're looking for
+            for (; i < len && str[i] != '>'; i++);
+        } else {
+            stripped_str[j] = str[i];
+            j++;
+        }
+    }
+    stripped_str[j] = 0;
+    return stripped_str;
+}
+
+/* **************************************************** */
+
 char* Utils::urlDecode(const char *src, char *dst, u_int dst_len) {
   char *ret = dst;
   u_int i = 0;
@@ -576,7 +649,7 @@ char* Utils::urlDecode(const char *src, char *dst, u_int dst_len) {
       char h[3] = { a, b, 0 };
       char hexval = (char)strtol(h, (char **)NULL, 16);
 
-      if(isprint(hexval))
+      //      if(iswprint(hexval))
 	*dst++ = hexval;
 
       src += 3;
@@ -623,28 +696,69 @@ bool Utils::isUserAdministrator(lua_State* vm) {
 
 /* **************************************************** */
 
+
 /**
  * @brief Purify the HTTP parameter
  *
  * @param param   The parameter to purify (remove unliked chars with _)
  */
 
+static const char* xssAttempts[] = {
+  "<?import",
+  "<applet",
+  "<base",
+  "<embed",
+  "<frame",
+  "<iframe",
+  "<implementation",
+  "<import",
+  "<link",
+  "<meta",
+  "<object",
+  "<script",
+  "<style",
+  "charset",
+  "classid",
+  "code",
+  "codetype",
+  /* "data", */
+  "href",
+  "http-equiv",
+  "javascript:",
+  "vbscript:",
+  "vmlframe",
+  "xlink:href",
+  "=",
+  NULL
+};
+
+/* ************************************************************ */
+
+/* http://www.ascii-code.com */
+
+bool Utils::isPrintableChar(u_char c) {
+  if(isprint(c)) return(true);
+
+  if((c >= 192) && (c <= 255))
+    return(true);
+
+  return(false);
+}
+
+/* ************************************************************ */
+
 void Utils::purifyHTTPparam(char *param, bool strict) {
-  xsslibUrl url;
-
-  /* Fix for http://packetstormsecurity.com/files/127329/Ntop-NG-1.1-Cross-Site-Scripting.html */
-  xsslibUrlInit(&url);
-  xsslibUrlSetUrl(&url, param);
-
-  if(xsslibUrlScan(&url) != XssClean) {
-    ntop->getTrace()->traceEvent(TRACE_WARNING, "Found possible XSS attempt: %s", param);
-    param[0] = '\0';
+  if(strict) {
+    for(int i=0; xssAttempts[i] != NULL; i++) {
+      if(strstr(param, xssAttempts[i])) {
+	ntop->getTrace()->traceEvent(TRACE_WARNING, "Found possible XSS attempt: %s [%s]", param, xssAttempts[i]);
+	param[0] = '\0';
+	return;
+      }
+    }
   }
 
-
-#if 0
   for(int i=0; param[i] != '\0'; i++) {
-
     bool is_good;
 
     if(strict) {
@@ -661,7 +775,7 @@ void Utils::purifyHTTPparam(char *param, bool strict) {
 	// || (param[i] == '.')
 	;
     } else {
-      is_good = isprint(param[i])
+      is_good = Utils::isPrintableChar(param[i])
 	&& (param[i] != '<')
 	&& (param[i] != '>');
     }
@@ -680,7 +794,6 @@ void Utils::purifyHTTPparam(char *param, bool strict) {
       param[i-1] = '_', param[i] = '_'; /* Invalidate the path */
     }
   }
-#endif
 }
 
 /* **************************************************** */
@@ -786,8 +899,8 @@ bool Utils::httpGet(lua_State* vm, char *url, char *username,
   bool ret = true;
 
   curl = curl_easy_init();
+
   if(curl) {
-    curl_version_info_data *v;
     DownloadState *state = NULL;
     long response_code;
     char *content_type, *redirection;
@@ -828,30 +941,38 @@ bool Utils::httpGet(lua_State* vm, char *url, char *username,
     curl_easy_setopt(curl, CURLOPT_NOSIGNAL, 1);
     curl_easy_setopt(curl, CURLOPT_TIMEOUT, timeout);
     curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT, timeout);
-    curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT_MS, timeout*1000);
 
-    v = curl_version_info(CURLVERSION_NOW);
-    snprintf(ua, sizeof(ua), "ntopng v.%s (curl %s)", PACKAGE_VERSION, v->version);
+#ifdef CURLOPT_CONNECTTIMEOUT_MS
+    curl_easy_setopt(curl, CURLOPT_CONNECTTIMEOUT_MS, timeout*1000);
+#endif
+
+    snprintf(ua, sizeof(ua), "%s [%s][%s]",
+	     PACKAGE_STRING, PACKAGE_MACHINE, PACKAGE_OS);
     curl_easy_setopt(curl, CURLOPT_USERAGENT, ua);
 
-    lua_newtable(vm);
+    if(vm) lua_newtable(vm);
 
     if(curl_easy_perform(curl) == CURLE_OK) {
-      if(return_content) {
+      if(return_content && vm) {
 	lua_push_str_table_entry(vm, "CONTENT", state->outbuf);
 	lua_push_int_table_entry(vm, "CONTENT_LEN", state->num_bytes);
       }
+
+      ret = true;
+    } else
+      ret = false;
+
+    if(vm) {
+      if(curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code) == CURLE_OK)
+	lua_push_int_table_entry(vm, "RESPONSE_CODE", response_code);
+
+      if((curl_easy_getinfo(curl, CURLINFO_CONTENT_TYPE, &content_type) == CURLE_OK) && content_type)
+	lua_push_str_table_entry(vm, "CONTENT_TYPE", content_type);
+
+      if(curl_easy_getinfo(curl, CURLINFO_EFFECTIVE_URL, &redirection) == CURLE_OK)
+	lua_push_str_table_entry(vm, "EFFECTIVE_URL", redirection);
     }
 
-    if(curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &response_code) == CURLE_OK)
-      lua_push_int_table_entry(vm, "RESPONSE_CODE", response_code);
-
-    if((curl_easy_getinfo(curl, CURLINFO_CONTENT_TYPE, &content_type) == CURLE_OK) && content_type)
-      lua_push_str_table_entry(vm, "CONTENT_TYPE", content_type);
-
-    if(curl_easy_getinfo(curl, CURLINFO_EFFECTIVE_URL, &redirection) == CURLE_OK)
-      lua_push_str_table_entry(vm, "EFFECTIVE_URL", redirection);
-
     if(return_content && state)
       free(state);
 
@@ -889,11 +1010,16 @@ char* Utils::getURL(char *url, char *buf, u_int buf_len) {
 
 // Support functions for 'urlEncode'.
 
+#ifdef NOTUSED
 static char to_hex(char code) {
   static char hex[] = "0123456789ABCDEF";
   return hex[code & 15];
 }
+#endif
 
+/* **************************************************** */
+
+#ifdef NOTUSED
 static int alphanum(char code) {
   int i;
   static char alnum[] = "0123456789abcdefghijklmnopqrstuvwxyz";
@@ -902,8 +1028,12 @@ static int alphanum(char code) {
   }
   return 0;
 }
+#endif
+
+/* **************************************************** */
 
 // Encodes a URL to hexadecimal format.
+#ifdef NOTUSED
 char* Utils::urlEncode(char *url) {
   char *pstr = url;
   char *buf = (char *) malloc(strlen(url) * 3 + 1);
@@ -925,10 +1055,12 @@ char* Utils::urlEncode(char *url) {
   *pbuf = '\0';
   return buf;
 }
+#endif
 
 /* **************************************** */
 
 // The following one initializes a new string.
+#ifdef NOTUSED
 static void newString(String *str) {
   str->l = 0;
   str->s = (char *) malloc((str->l) + 1);
@@ -941,8 +1073,12 @@ static void newString(String *str) {
   }
   return;
 }
+#endif
+
+/* **************************************************** */
 
 // This callback function will be passed to 'curl_easy_setopt' in order to write curl output to a variable.
+#ifdef NOTUSED
 static size_t writeFunc(void *ptr, size_t size, size_t nmemb, String *str) {
   size_t new_len = str->l + (size * nmemb);
   str->s = (char *) realloc(str->s, new_len + 1);
@@ -956,7 +1092,11 @@ static size_t writeFunc(void *ptr, size_t size, size_t nmemb, String *str) {
 
   return (size * nmemb);
 }
+#endif
 
+/* **************************************************** */
+
+#ifdef NOTUSED
 // Adding this function that performs a simple HTTP GET request using libcurl.
 // The function returns a string that contains the reply.
 char* Utils::curlHTTPGet(char *url, long *http_code) {
@@ -984,6 +1124,7 @@ char* Utils::curlHTTPGet(char *url, long *http_code) {
   }
   return NULL;
 }
+#endif
 
 /* **************************************** */
 
@@ -1165,9 +1306,8 @@ bool Utils::discardOldFilesExceeding(const char *path, const unsigned long max_s
 
 /* **************************************** */
 
-#ifdef linux
 
-static char* macaddr_str (const char *mac, char *buf) {
+char* Utils::macaddr_str (const char *mac, char *buf) {
   sprintf(buf, "%02X:%02X:%02X:%02X:%02X:%02X",
           mac[0] & 0xFF, mac[1] & 0xFF, mac[2] & 0xFF,
           mac[3] & 0xFF, mac[4] & 0xFF, mac[5] & 0xFF);
@@ -1176,19 +1316,44 @@ static char* macaddr_str (const char *mac, char *buf) {
 
 /* **************************************** */
 
-void Utils::readMac(char *ifname, dump_mac_t mac_addr) {
+
+u_int64_t Utils::macaddr_int (const u_int8_t *mac) {
+  u_int64_t mac_int = 0;
+  for(u_int8_t i=0; i<6; i++){
+    mac_int |= (mac[i] & 0xFF) << (5-i)*8;
+  }
+  return mac_int;
+}
+
+/* **************************************** */
+
+#ifdef linux
+
+void Utils::readMac(char *_ifname, dump_mac_t mac_addr) {
   int _sock, res;
   struct ifreq ifr;
   macstr_t mac_addr_buf;
+  char *colon, *at;
+  char ifname[32];
 
   memset (&ifr, 0, sizeof(struct ifreq));
 
+  /* Handle PF_RING interfaces zc:ens2f1@3 */
+  colon = strchr(_ifname, ':');
+  if (colon != NULL) /* removing pf_ring module prefix (e.g. zc:ethX) */
+    _ifname = colon+1;
+
+  snprintf(ifname, sizeof(ifname), "%s", _ifname);
+  at = strchr(ifname, '@');
+  if(at != NULL)
+    at[0] = '\0';
+
   /* Dummy socket, just to make ioctls with */
   _sock = socket(PF_INET, SOCK_DGRAM, 0);
   strncpy(ifr.ifr_name, ifname, IFNAMSIZ-1);
   res = ioctl(_sock, SIOCGIFHWADDR, &ifr);
   if(res < 0) {
-    ntop->getTrace()->traceEvent(TRACE_ERROR, "Cannot get hw addr");
+    ntop->getTrace()->traceEvent(TRACE_ERROR, "Cannot get hw addr for %s", ifname);
   } else
     memcpy(mac_addr, ifr.ifr_ifru.ifru_hwaddr.sa_data, 6);
 
@@ -1228,7 +1393,7 @@ u_int16_t Utils::getIfMTU(const char *ifname) {
       if(mtu > ((u_int16_t)-1))
 	mtu = ((u_int16_t)-1);
     }
-    
+
     closesocket(fd);
   }
 
@@ -1239,12 +1404,14 @@ u_int16_t Utils::getIfMTU(const char *ifname) {
 /* **************************************** */
 
 u_int32_t Utils::getMaxIfSpeed(const char *ifname) {
-#ifdef linux
+#if defined(linux) && (!defined(__GNUC_RH_RELEASE__) || (__GNUC_RH_RELEASE__ != 44))
   int sock, rc;
   struct ifreq ifr;
   struct ethtool_cmd edata;
   u_int32_t ifSpeed = 1000;
 
+  if(strncmp(ifname, "zc:", 3) == 0) ifname = &ifname[3];
+
   memset(&ifr, 0, sizeof(struct ifreq));
 
   sock = socket(PF_INET, SOCK_DGRAM, 0);
@@ -1390,3 +1557,13 @@ char* Utils::intoaV6(struct ndpi_in6_addr ipv6, u_int8_t bitmask, char* buf, u_s
     return(ret);
 }
 
+/* ****************************************************** */
+
+void Utils::xor_encdec(u_char *data, int data_len, u_char *key) {
+  int i, y;
+
+  for(i = 0, y = 0; i < data_len; i++) {
+    data[i] ^= key[y++];
+    if(key[y] == 0) y = 0;
+  }
+}
