@@ -1,6 +1,6 @@
 /*
  *
- * (C) 2013-15 - ntop.org
+ * (C) 2013-16 - ntop.org
  *
  *
  * This program is free software; you can redistribute it and/or modify
@@ -68,8 +68,8 @@ void initWinsock32() {
   if( err != 0 ) {
     /* Tell the user that we could not find a usable */
     /* WinSock DLL.                                  */
-    printf("FATAL ERROR: unable to initialise Winsock 2.x.\n");
-    _exit(-1);
+    printf("FATAL ERROR: unable to initialize Winsock 2.x.\n");
+    exit(-1);
   }
 }
 
@@ -138,22 +138,27 @@ int main(int argc, char *argv[])
     if((ifName = ntop->get_if_name(i)) == NULL)
       continue;
 
-    /* [ zmq-collector.lua@tcp://127.0.0.1:5556 ] */
-    if((strstr(ifName, "tcp://") || strstr(ifName, "ipc://"))
-       ) {
-      char *at = strchr(ifName, '@');
-      char *topic = (char*)"flow", *endpoint;
+    try {
+      /* [ zmq-collector.lua@tcp://127.0.0.1:5556 ] */
+      if(!strcmp(ifName, "dummy")) {
+	iface = new DummyInterface();
+      } else if((strstr(ifName, "tcp://") || strstr(ifName, "ipc://"))) {
+	char *at = strchr(ifName, '@');
+	char *endpoint;
 
-      if(at != NULL)
-	endpoint = &at[1];
-      else
-	endpoint = ifName;
+	if(at != NULL)
+	  endpoint = &at[1];
+	else
+	  endpoint = ifName;
 
-      iface = new CollectorInterface(endpoint, topic);
-    } else {
-      iface = NULL;
+	iface = new CollectorInterface(endpoint);
+#if defined(HAVE_PF_RING) && (!defined(__mips)) && (!defined(__arm__))
+      } else if(strstr(ifName, "zcflow:")) {
+	iface = new ZCCollectorInterface(ifName);
+#endif
+      } else {
+	iface = NULL;
 
-      try {
 #ifdef NTOPNG_PRO
 	if(strncmp(ifName, "bridge:", 7) == 0)
 	  iface = new PacketBridge(ifName);
@@ -164,46 +169,59 @@ int main(int argc, char *argv[])
           iface = new NetfilterInterface(ifName);
 #endif
 
+#if defined(__FreeBSD__) || defined(__NetBSD__) || defined(__OpenBSD__) || defined(__APPLE__)
+        if(iface == NULL && strncmp(ifName, "divert:", 7) == 0)
+          iface = new DivertInterface(ifName);
+#endif
+	
 #ifdef HAVE_PF_RING
 	if(iface == NULL)
 	  iface = new PF_RINGInterface(ifName);
 #endif
-      } catch(int) {
-	delete iface;
+      }
+    } catch(...) {
+      iface = NULL;
+    }
+
+    if(iface == NULL) {
+      try {
+	iface = new PcapInterface(ifName);
+      } catch(...) {
+	ntop->getTrace()->traceEvent(TRACE_ERROR, "Unable to create interface %s", ifName);
 	iface = NULL;
       }
     }
 
-    if(iface == NULL) iface = new PcapInterface(ifName);
-
-    if(affinity != NULL) {
-      if(indexAffinity == 0)
-        core_id_s = strtok(affinity, ",");
-      else 
-        core_id_s = strtok(NULL, ",");
+    if(iface) {
+      if(affinity != NULL) {
+	if(indexAffinity == 0)
+	  core_id_s = strtok(affinity, ",");
+	else 
+	  core_id_s = strtok(NULL, ",");
             
-      if(core_id_s != NULL)
-        core_id = atoi(core_id_s);
-      else
-        core_id = indexAffinity;
+	if(core_id_s != NULL)
+	  core_id = atoi(core_id_s);
+	else
+	  core_id = indexAffinity;
       
-      indexAffinity++;
-      iface->setCPUAffinity(core_id);
+	indexAffinity++;
+	iface->setCPUAffinity(core_id);
+      }
+
+      if(prefs->get_packet_filter() != NULL)
+	iface->set_packet_filter(prefs->get_packet_filter());
+
+      ntop->registerInterface(iface);
     }
-
-    if(prefs->get_packet_filter() != NULL)
-      iface->set_packet_filter(prefs->get_packet_filter());
-
-    ntop->registerInterface(iface);
-  }
+  } /* for */
 
   ntop->createExportInterface();
 
-  ntop->getRedis()->startFlowDump();
+  ntop->getElasticSearch()->startFlowDump();
 
   if(ntop->getInterfaceAtId(0) == NULL) {
     ntop->getTrace()->traceEvent(TRACE_ERROR, "Startup error: missing super-user privileges ?");
-    _exit(0);
+    exit(0);
   }
 
   for(int i = 0 ; i < MAX_NUM_INTERFACES ; i++) {
@@ -223,9 +241,6 @@ int main(int argc, char *argv[])
     }
   }
 
-  if(prefs->do_change_user())
-    Utils::dropPrivileges();
-
 #ifndef WIN32
   if(prefs->get_pid_path() != NULL) {
     FILE *fd;
@@ -236,7 +251,7 @@ int main(int argc, char *argv[])
       fclose(fd);
 
       if(n > 0) {
-	chmod(prefs->get_pid_path(), 0777);
+	chmod(prefs->get_pid_path(), 0644);
 	ntop->getTrace()->traceEvent(TRACE_NORMAL, "PID stored in file %s",
 				     prefs->get_pid_path());
       } else
@@ -248,6 +263,9 @@ int main(int argc, char *argv[])
   }
 #endif
 
+  if(prefs->do_change_user())
+    Utils::dropPrivileges();
+
   ntop->loadGeolocation(prefs->get_docs_dir());
   ntop->registerHTTPserver(new HTTPserver(prefs->get_http_port(),
 					  prefs->get_docs_dir(),
@@ -265,27 +283,14 @@ int main(int argc, char *argv[])
     ntop->getTrace()->traceEvent(TRACE_ERROR,
 				 "Unable to write on %s [%s]: please specify a different directory (-d)",
 				 ntop->get_working_dir(), path);
-    _exit(0);
+    exit(0);
   } else {
     fclose(fd); /* All right */
     unlink(path);
   }
   
-  if(prefs->get_categorization_key() != NULL) {
-    ntop->setCategorization(new Categorization(prefs->get_categorization_key()));
-    prefs->enable_categorization();
-    ntop->getTrace()->traceEvent(TRACE_NORMAL,
-				 "Enabled Host categorization with key %s",
-				 prefs->get_categorization_key());
-
-  }
-
-  if(prefs->get_httpbl_key() != NULL) {
-    ntop->getTrace()->traceEvent(TRACE_WARNING,
-        "HTTPBL categorization is not enabled: using default key");
+  if(prefs->get_httpbl_key() != NULL)
     ntop->setHTTPBL(new HTTPBL(prefs->get_httpbl_key()));
-    prefs->enable_httpbl();
-  }
 
   ntop->getTrace()->traceEvent(TRACE_NORMAL, "Working directory: %s",
 			       ntop->get_working_dir());
@@ -296,12 +301,12 @@ int main(int argc, char *argv[])
   signal(SIGTERM, sigproc);
   signal(SIGINT, sigproc);
 
-  #if defined(WIN32) && defined(DEMO_WIN32)
-    ntop->getTrace()->traceEvent(TRACE_NORMAL, "-----------------------------------------------------------");
-    ntop->getTrace()->traceEvent(TRACE_WARNING, "This is a demo version of ntopng limited to %d packets", MAX_NUM_PACKETS);
-    ntop->getTrace()->traceEvent(TRACE_WARNING, "Please go to http://shop.ntop.org for getting the full version");
-    ntop->getTrace()->traceEvent(TRACE_NORMAL, "-----------------------------------------------------------");
-  #endif
+#if defined(WIN32) && defined(DEMO_WIN32)
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "-----------------------------------------------------------");
+  ntop->getTrace()->traceEvent(TRACE_WARNING, "This is a demo version of ntopng limited to %d packets", MAX_NUM_PACKETS);
+  ntop->getTrace()->traceEvent(TRACE_WARNING, "Please go to http://shop.ntop.org for getting the full version");
+  ntop->getTrace()->traceEvent(TRACE_NORMAL, "-----------------------------------------------------------");
+#endif
 
   ntop->start();
 
